// -- File: ARRAYEX.CC
// This file contains two array classes -- ArrayEX and zArray

CLASS ArrayEX(a1, a2) 
/*
   ----------------------------------------------------------------
   Description:

      This class contains a set of array routines, and is designed 
      to enhance the standard array class in Visual dBASE. This is
      not a class derived from array! 
         
      The output of many of the methods herein is an array, 
      and such arrays are assigned pointers to the methods of this class, 
      so that such output arrays DO carry the functionality 
      of this class.

      This file includes zArray as well, see below the main class
      (after the "endclass" statement) to see the zArray class
      and documentation for that class.

   Programmers:

    Ken Mayer
    Adam L. Menkes
    Jay Parsons 
    Werner Borsbach 
    Bowen Moursund 
    Marcello Perathoner 
    Bruce Beacham
 
    **** NOTE: If you do not use the release() method of ArrayEx,
         you will not be able to release the reference to the
         MathEx class that is created in the init() method.
 
    Instantiation:
 
       set procedure to :dUFLP:arrayex.cc additive
       oMyArray = new ArrayEx()

    or, to provide one array to the methods:
       oMyArrayex = new ArrayEx(MyArray)
 
    Typical usage would be to call individual methods from here,
    such as:
 
      ? oMyArray.Shuffle( aArrayName )

    Alternatively, some methods may be called thus:
       nResult = new arrayex(MyArray).Min()

    Be sure to release it when done:
       oMyArrayEx.release()
 
    Methods:
      Note, more details are in the headers for each method, 
      including a listing of parameters, and what they mean, as 
      well as the complete history (as we know it) for each 
      method.
 
       Init()   
         handles any "initialization" necessary, is handled
         when a new instance of the ArrayEx class is created.
 
       Release() 
         releases this class and other classes if they weren't
         instantiated previously
 
       ADiff()
         Compares two arrays, returns match/mismatch information
 
       ADiffA()
         Compares two arrays, returns match/mismatch array
  
       Avg()
         Averages row or column of an array.
 
       Copy()
         Copies an array, returns pointer to new array "aCopy"
 
       CopySimple()
         Copies a simple 1D array, returns pointer to new array "aCopy"
         This is used to copy arrays with a size property but without
         other standard properties.   
         Needed for example to create copies of FieldArray and 
         IndexArray objects, so that normal array methods are available 
         for manipulation of those special arrays (eg the scan method)
 
       Flip()
         Rotates array so rows become columns and vice versa
 
       FlipCopy()
         Rotates and returns a copy of the given array so rows become 
         columns and vice versa but the original array stays unaltered.

       Shuffle()
         Shuffles elements of a one-dimensional array
 
       Txt2Arr()
         Adds text file to array
 
       Variance()
         Variance of a row or column of array

       Min()
         Lowest collation or numeric value found in 
         one row, or one column, of an array.
         (Uses MinMax() as its engine).
 
       Max()
         Highest collation or numeric value found in 
         one row, or one column, of an array.
         (Uses MinMax() as its engine).

       CarveOut()
         Returns as a 1D array just one row or column of a 2D array

       Upper()
         Returns a dplicate of an array with all character elements capitalised.

       ListArray()
         Lists the elements in an array to the results pane.   
         Can list assocarrays.
         Can enumerate object properties.

      GridArray()
         Creates a temporary table of the array elements and shows them
         in a grid in a form.

      StringArray()
         Creates a string out of the contents of an array (AssocArray or
         normal Array), which can then be used in an editor object.

       AssignMethods()
         Assigns the methods of this class to an array.   This is done for each
         of up to two arrays provided to the class upon instantiation.    
         It is also done for any array produced as the output of one of the 
         methods herein.   (And of course therefore for any array produced as  
         the output of one of the methods when it is called as a method of an array
         already the output of one of these methods!).
 
   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           mathex.cc
   **           stringex.cc
   **           root.cc
   **           setproc.prg

   ----------------------------------------------------------------
*/
   this.a1 = a1
   this.a2 = a2
   class::Init()
   if type("this.a1") $ "AO"
      class::AssignMethods((this.a1))
   endif
   if type("this.a2") $ "AO"
      class::AssignMethods((this.a2))
   endif


   FUNCTION Init
   /* -------------------------------------------------------------------
     Programmer..: Ken Mayer
     Date........: 11/10/1996
     Notes.......: Ensures that routines needed are available
                   It is required that this be used (see instructions
                   above) for this class to function properly.
     Written for.: Visual dBASE 5.5
     Rev. History: 11/10/1996 -- Original Code
     Calls.......: math.cc
     Usage.......: arrayex.Init()
     Example.....: (Called on instantiation of object)
     Returns.....: Nul
     Parameters..: None
    -------------------------------------------------------------------*/

      set procedure to :dUFLP:SetProc.prg additive
      if not SetProc( ":dUFLP:MATHEX.CC" )
         set procedure to :dUFLP:mathex.cc additive
         this.SetMath = true
      else
         this.SetMath = false
      endif
      this.Math        = new MathEx()
      this.Math.parent = this
      
   RETURN NULL
   // -- EoM: Init()
   
   FUNCTION Release
   /* -------------------------------------------------------------------
     Programmer..: Ken Mayer
     Date........: 11/27/1996
     Notes.......: Used to release this object from memory.
     Written for.: Visual dBASE 5.5
     Rev. History: 11/24/1996 -- Original Code
     Calls.......: none
     Usage.......: array.Release()
     Example.....: oMyArray.Release()
                   oMyArray = NULL  && not required, good idea
     Returns.....: Null
     Parameters..: None
    ------------------------------------------------------------------- */

      this.math.release()
      this.math = null
      if this.SetMath
         close procedure :dUFLP:mathex.cc
      endif
      release object this
   
   RETURN NULL
   // -- EoM: Release()

   FUNCTION ADiff         // ( oArray1, oArray2, nLen )
   /* -------------------------------------------------------------------
     Programmer..: Werner Borsbach
     Date........: 11/10/1996
     Notes.......: Compares two arrays, returns index of first 
                   differing element. 0 if all match.
                   Ignores all elements of either array past <nLen>.
     Written for.: dBASE IV v1.5
     Rev. History: 02/09/1993 Original Release as CmpArray procedure
                   05/16/1993 modified for use with - modified - 
                              aRec2Arr()
                   Fall, 1994 -- revised for dBASE 5.0 for Windows.
                                 Jay Parsons, 72662,1302                                     ,
                   11/10/1996 -- Modified to work with this class.
     Calls.......: None
     Usage.......: arrayex.ADiff ([<Array1>,<Array2>][,<nLen>])
     Example.....: a1 = aex.Rec2Arr ()
                   edit recno ()
                   a2 = aex.Rec2Arr ()
                   ? aex.ADiff (a1, a2, a1.size)

                  a = new array()
                  a.dir()
                  b = new array(alen(a, 1), alen(a, 2))
                  acopy(a, b)
                  ? new arrayex(a, b).adiff(10)			
                  // Checks agreement of elements 1 - 10 of a and b.   
                  // Returns 0

     Returns.....: 0 if arrays match, else index of first differing 
                   element
     Parameters..: a1, a2 = arrays to compare.  Optional if provided 
                            on instantiation.
                   nLen = Number of elements
    -------------------------------------------------------------------*/
      
      local i, nLen
      private a1, a2           // need private for type ()
      a1 = argvector(1)        //oArray1
      a2 = argvector(2)        //oArray2
      nlen = argvector(3)
      
      IF PCount () = 3
         // Arrays and Extent have been provided in the method call.
         nLen = min (nLen, a1.size)
         nLen = min (nLen, a2.size)
      ELSEIF PCount() = 1 and type("argvector(1)") = "A"
         if type("this") = "A"
           /* The comparison array has been provided as a parameter, 
              but not the extent.   Source array has had arrayex method 
              calls attached by arrayex::AssignMethods()
              Parameter to this method is the array to be compared 
              with this.   Remember it returns a different array 
              as a result.
           */
            a1 = this
            a2 = argvector(1)
            nLen = min (a1.size, a2.size)
         elseif type("this.a1") = "A"
            // The original array was provided upon instantiation.
            // The comparison array has been provided as a parameter, 
            // but not the extent.
            a1 = this.a1
            a2 = argvector(1)
            nLen = min (a1.size, a2.size)
         else
            msgbox("AnArray for comparison has not been provided.",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 1 and type("argvector(1)") = "N"
         // Only the Extent has been provided in the method call.
         if type("this.a1") = "A" and type("this.a2") = "A"
            // The original and comparison arrays were provided 
            // upon instantiation. Only applies to an arrayex(), 
            // not to a subsequent array().
            a1 = this.a1
            a2 = this.a2
            nLen = argvector(1)
            nLen = min (nLen, a1.size)
            nLen = min (nLen, a2.size)
         else
            msgbox("Arrays for comparison have not been provided.   ",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 0
         if type("this.a1") = "A" and type("this.a2") = "A"
            // The arrays were given upon instantiation.
            a1 = this.a1
            a2 = this.a2
            nLen = min (a1.size, a2.size)
         else
            msgbox("Arrays for comparison have not been provided.",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 2 and type("argvector(1)") = "A" and ;
                              type("argvector(2)") = "A"
         // Extent must be set at the default - the maximum possible.
         nLen = min (a1.size, a2.size)
      ELSE
         msgbox("Arrays for comparison have not been provided", ;
                "ArrayEx.ADiffA", 48)
         return false
      ENDIF
      
      FOR i = 1 TO nLen
         IF Type ("a1 [i]") <> Type ("a2 [i]")
            RETURN i
         ENDIF
         IF a1 [i] <> a2 [i]
            RETURN i
         ENDIF
      NEXT
      
   RETURN 0
   // -- EoM: ADiff()
   
   FUNCTION ADiffA          //  ( oArray1, oArray2, nLen )
   /* -------------------------------------------------------------------
     Programmer..: Werner Borsbach
     Date........: 11/10/1996
     Notes.......: Compares two arrays, returns an array 
                   containing .t. for every element in that  
                   the arrays match.
                   Ignores all elements of either array past <nLen>.
     Written for.: dBASE IV v1.5
     Rev. History: 02/09/1993 Original Release as CmpArray procedure
                   05/16/1993 modified for use with - modified - 
                              aRec2Arr()
                   Fall, 1994 -- revised for dBASE 5.0 for Windows.
                                 Jay Parsons, 72662,1302                                     ,
                   11/10/1996 -- Modified to work with this class.
     Calls.......: None
     Usage.......: arrayex.ADiff ([<Array1>,<Array2>][,<nLen>])
     Example.....: // create/load two arrays, named a1 and a2,
                   // then: 
                   a3 = aex.ADiff (a1, a2, a1.size)

                   a = new array()
                   a.dir()
                   b = new array(alen(a, 1), alen(a, 2))
                   acopy(a, b)
                   b[2,2] += 190
                   b[5,1] = "Changed"
                   ? new arrayex(a, b).adiffa(10)			
                   // Checks agreement of elements 1 - 10 of a and b.
                   // Returns a 1D array of ten rows, each true 
                   // except row 7 which is false.
           
                   a = new array()
                   a.dir()
                   b = new arrayex(a)
                   c =  b.adiffa(a, (b.a1))
                   ? b.listarray(c)        // a list of falses and a 
                     count of them.

     Returns.....: array 
                     Also sets the returned array as the arrayex objects a1 property.
     Parameters..: a1, a2 = arrays to compare 
                   nLen = Number of elements
    ------------------------------------------------------------------- */
      
      local i
      private a1, a2      // need private for type ()
      a1 = argvector(1)   //oArray1
      a2 = argvector(2)   //oArray2
      nLen = argvector(3)
      
      IF PCount () = 3
         // Arrays and Extent have been provided in the method call.
         nLen = min (nLen, a1.size)
         nLen = min (nLen, a2.size)
      ELSEIF PCount() = 1 and type("argvector(1)") = "A"
         if type("this") = "A"
            /* 
              The comparison array has been provided as a parameter, 
              but not the extent. Array has had arrayex method calls 
              attached by arrayex::AssignMethods() Parameter to 
              this method is the array to be compared with this.
              Remember it returns a different array as a result.
            */
            a1 = this
            a2 = argvector(1)
            nLen = min (a1.size, a2.size)
         elseif type("this.a1") = "A"
            // The original array was provided upon instantiation.
            // The comparison array has been provided as a parameter, 
            // but not the extent.
            a1 = this.a1
            a2 = argvector(1)
            nLen = min (a1.size, a2.size)
         else
            msgbox("An Array for comparison has not been provided.",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 1 and type("argvector(1)") = "N"
         // Only the Extent has been provided in the method call.
         if type("this.a1") = "A" and type("this.a2") = "A"
            // The original and comparison arrays were provided upon 
            // instantiation. Only applies to an arrayex(), not 
            // to a subsequent array().
            a1 = this.a1
            a2 = this.a2
            nLen = argvector(1)
            nLen = min (nLen, a1.size)
            nLen = min (nLen, a2.size)
         else
            msgbox("Arrays for comparison have not been provided.   ",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 0
            if type("this.a1") = "A" and type("this.a2") = "A"
            // The arrays were given upon instantiation.
            a1 = this.a1
            a2 = this.a2
            nLen = min (a1.size, a2.size)
         else
            msgbox("Arrays for comparison have not been provided.",;
                   "ArrayEx.ADiffA", 48)
            return false
         endif
      ELSEIF PCount() = 2 and type("argvector(1)") = "A" and;
                              type("argvector(2)") = "A"
         // Extent must be set at the default - the maximum possible.
         nLen = min (a1.size, a2.size)
      ELSE
         msgbox("Arrays for comparison have not been provided.",;
                "ArrayEx.ADiffA", 48)
         return false
      ENDIF
      if type("zArray::zArray") = "FP"
         aDiff = new zArray (nLen)
      else
         aDiff = new array (nLen)
      endif
      
      FOR i = 1 TO nLen
         aDiff [i] = (Type ("a1 [i]") = Type ("a2 [i]")) ;
            .and. (a1 [i] = a2 [i])
      NEXT
      
      this.a1 = (aDiff)
      this.AssignMethods((this.a1))

   RETURN aDiff
   // -- EoM: ADiffA ()

   FUNCTION Avg           //( aName, nRowCol, nWhich )
   /* -------------------------------------------------------------------
     Programmer..: Jay Parsons
     Date........: 11/10/1996
     Notes.......: Mean of non-blank numeric or date values in 
                   specified row or column of a specified array. If 
                   the first value is a date, averages only dates.  
                   If first value is numeric or float, averages only 
                   numerics and floats.  Returns .F. if any value in 
                   row or column is any other type, if specified row
                   or column does not exist or if there are no 
                   nonblank values.
     Written for.: dBASE 5.0 for Windows
     Rev. History: 10/01/1994 -- Original Release
                   11/10/1996 -- Modified to work with this class.   
     Calls.......: None
     Usage.......: arrayex.Avg( [<aName>] [, <nRowCol> [, <nWhich>]] )
     Example.....: ? oMyArray.Avg( Myarray, 1, 1 )
     Returns.....: A numeric, float or date value, the mean or 
                   average, or .F. If any of the averaged items are 
                   floats, the result will be.   ??
                   Assigns a valid return value to a lastAverage property 
                   of this arrayex.
     Parameters..: aName   = the name of the array, not in quotes, 
                             ie an array object. If parameter 1 is 
                             not an array, the first array provided on 
                             instantiation is used and the parameters 
                             are shifted..
                   nRowCol = the number of the row or column to 
                             average, default being 1
                   nWhich  = one of these values:
                                1 = nRowCol specifies a row (default)
                                2 = nRowCol specifies a column
    ------------------------------------------------------------------- */

      private aName, nR, nW, nRowCol, nWhich
      
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         aName = this
         nRowCol = argvector(1)
         nWhich = argvector(2)
      elseif type("argvector(1)") = "A"
         aName = argvector(1)
         nRowCol = argvector(2)
         nWhich = argvector(3)
      elseif type("argvector(1)") <> "A" and type("this.a1") = "A"
         aName = this.a1
         nRowCol = argvector(1)
         nWhich = argvector(2)
      else
         msgbox("No array given.   ", "Arrayex.Avg", 48)
         return false
      endif

      local xReturn, cType, nAt, nTot, nCount, xNext, cType2
      
      nW = iif( nWhich = 2, 2, 1 )
      nR = iif( type("nRowCol")="N", nRowCol, 1 )
      xReturn = .F.
      nDims = aName.dimensions
      if nDims = 1
         if nR # 1 .or. nW # 1
            RETURN xReturn
         else
            cType = type( "aName[ nR ]" )   
         endif
      else
         if nW = 2
            cType = type( "aName[1,nR]" )
         else
            cType = type( "aName[nR,1]" )
         endif
      endif
      if .not. cType $ "DFN"
         RETURN xReturn
      endif
      store 0 to nTot, nCount
      for nAt = 1 to alen( aName, nW )
          do case
             case nDims = 1
                  xNext = aName[ nAt ]
             case nW = 1
                  xNext = aName[ nR, nAt ]
             otherwise
                  xNext = aName[ nAt, nR ]
          endcase
          cType2 = type( xNext )
          if ( .not. cType2 $ "DFN" ) .or. ;
             ( ( cType = "D" ) # ( cType2 = "D" ) )
             exit
          endif
          if isblank( xNext )
             loop
          endif
          xReturn = .T.
          if cType2 = "D"
             xNext = xNext - {01/01/2001}		
                    // added century for certainty.
          endif
          nTot = nTot + xNext
          nCount = nCount + 1
      endfor
      if xReturn # .F.
         xReturn = nTot/nCount + iif( cType ="D",{01/01/2001},0)		
                    // added century for certainty.
      endif
      
      this.lastAverage = xReturn
      
   RETURN xReturn
   // -- EoM: Avg()

   FUNCTION Copy              //( oArray )
   /* -------------------------------------------------------------------
      Programmer..: Marcello Perathoner
      Date........: 01/11/1997
      Notes.......: Returns a copy of the array if 1 or 2-dimensional
      Written for.: dBASE 5.0 for Windows
      Rev. History: 01/11/1997 -- Original
                    8/8/2001: Added capability of using class 
                              parameters.
                    March 9, 2002:  Fixed bug when array has no rows.
      Usage.......: arrayex.copy (oArray)
      Example.....: oCopyOfArray = oArray.copy (oOriginalArray)
      Returns.....: array
      ------------------------------------------------------------------ 
   */
      parameter oArray

      if not empty(oArray) and type("oArray") = "A"
         //Given the array to be copied as a parameter.
      elseif not empty(oArray)
         msgbox("Object given is not an Array.   ", "Arrayex.Copy", 48)
            return false
      elseif type("this") = "A"
         // Array to be copied is the host object.
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oArray = this
      elseif type("this.a1") = "A"
         // Array is the default object of the arrayex object.
         oArray = this.a1
      else
         msgbox("No array given.   ", "Arrayex.Copy", 48)
         return false
      endif

      local aCopy

      DO CASE 
         CASE oArray.dimensions = 1 and ;
              type("zArray::zArray") = "FP"
            aCopy = new zArray (oArray.size)
         CASE oArray.dimensions = 1
            aCopy = new ARRAY (oArray.size)
         CASE oArray.dimensions = 2 and ;
              type("zArray::zArray") = "FP"
            aCopy = new zArray (ALEN (oArray, 1), ALEN (oArray, 2))
         CASE oArray.dimensions = 2
            aCopy = new ARRAY (ALEN (oArray, 1), ALEN (oArray, 2))
         OTHERWISE
            RETURN NULL
      ENDCASE

      if alen(oArray, 1) > 0
         ACOPY (oArray, aCopy)
      endif
      this.AssignMethods((aCopy))// already assigned if a zArray.
      
   RETURN aCopy
   // -- EoM: Copy()

   FUNCTION CopySimple
   /* 
      -------------------------------------------------------------------
      Programmer..: Bruce Beacham
      Date........: 10/09/2001
      Notes.......: Returns a copy of a simple 1D array.    
                    This is used to copy 1D arrays with a size 
                    property but without other standard properties, 
                    especially not dimensions. Thus it copes with 
                    special arrays like IndexArray and FieldArray.
                    Needed for example to create copies of FieldArray 
                    and IndexArray objects, so that normal array methods 
                    are available for manipulation of those special 
                    arrays (eg the scan method).

                    It also uses a count method to substitute a size 
                    property, so it can thus recognise and cope 
                    with AssocArrays. 
      Written for.: dB2K
      Rev. History: 10/09/2001 -- Original
                    March 9, 2002: Fixed bug in if branch # 5.
      Usage.......: arrayex.copySimple (oSimpleArray)
      Example.....: oCopyOfArray = oArray.copySimple (oOriginalArray)
                    OR  (a form of assocArray)
                    new zArray().copysimple(_app.sourcealiases).listarray()

      Returns.....: 1-Dimensional array
      ------------------------------------------------------------------ 
   */
      parameter oArray

      if not empty(oArray) and type("oArray.size") = "N"
         //Given the array to be copied as a parameter.
      elseif not empty(oArray) and type("oArray.count") = "FP"
         // An AssocArray (or a Rowset - ugh!)
         oArray.size = oArray.count()     // create a property.
         oArray.sizeCounted = true    // create a flag that it's 
                                   // an assocarray or similar.
      elseif not empty(oArray)
         msgbox("Object given is not an Array with a size property.   ",;
            "Arrayex.CopySimple", 48)
         return false
      elseif type("this") $ "AO" and type("this.size") = "N"
         // Array to be copied is the host object.
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oArray = this
      elseif type("this.a1") $ "AO" and type("this.a1.size") = "N"
         // Array is the default object of the arrayex object.
         oArray = this.a1
      elseif type("this.a1") $ "AO" and type("this.a1.count") = "FP"
         // An AssocArray (or a Rowset - ugh!)
         oArray = this.a1
         oArray.size = oArray.count()     // create a property.
         oArray.sizeCounted = true    // create a flag that it's 
                                    // an assocarray or similar.
      else
         msgbox("No simple Array given.", "Arrayex.CopySimple", 48)
         return false
      endif

      if type("oArray.dimensions") = "N" .and. oArray.dimensions > 1
         msgbox("Array given is not a simple 1D array.   ", ;
            "Arrayex.CopySimple", 48)
         return false
      endif

      local aCopy, s

      DO CASE 
         CASE type("zArray::zArray") = "FP"
            aCopy = new zArray (oArray.size)
         OTHERWISE
            aCopy = new ARRAY (oArray.size)
      ENDCASE

      if type("oArray.sizeCounted") = "L"
         // an AssocArray
         local aakey
         aakey = oArray.firstkey
         for s = 1 to oArray.size
            aCopy[s] = oArray[aakey]
            aakey = oArray.nextkey(aakey)
         endfor
      else
         for s = 1 to oArray.size      // permitted to be zero.
            aCopy[s] = oArray[s]
         endfor
      endif

      this.AssignMethods((aCopy)) // already assigned if a zArray.
      
   RETURN aCopy
   // -- EoM: CopySimple()

   FUNCTION Flip( aName )
   /* -------------------------------------------------------------------
     Programmer..: Jay Parsons 
     Date........: 11/10/1996
     Notes.......: Flips a two-dimensional array so its rows become 
                   its columns and vice versa, so you may then use 
                   a function such as array method sort() which works 
                   only on rows. Returns .T. if array flipped, else .F.
     Written for.: dBASE 5.0 for Windows
     Rev. History: 10/02/1994 -- original function.
                   11/10/1996 -- Modified to work with this class.   
                   19 July 2001 -- Modified to work with an array 
                   parameterised into the class, and error message if none.
     Calls.......: None
     Called by...: Any
     Usage.......: arrayex.Flip( <aName> )
     Example.....: oMyArray.Flip( aMyArray )
     Returns.....: true if array successfully flipped, else false
                    Sets the arrayex object's a1 property to the flipped array.
     Parameters..: aName = array name, not in quotes (ie an array object) 
    ------------------------------------------------------------------- */
      
      local lSuccess, nRows, nCols, nMax, nDiag, nAt, xTemp, n1, n2
      
      if not empty(aName)
      elseif type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         aName = this
      elseif type("this.a1") = "A"
         aName = this.a1
      else
         msgbox("No array given.   ", "ArrayEx.Flip", 48)
         return false
      endif

      lSuccess = false
      nCols = alen( aName, 2 )
      nRows = alen( aName, 1 )
      if nCols = 0
         RETURN lSuccess
      endif
      nMax = max( nRows, nCols )
      if nRows # nCols
         if aresize( aName, nMax, nMax, 1 ) # nMax * nMax
            RETURN lSuccess
         endif
      endif
      for nDiag = 1 to min( nRows, nCols )
          for nAt = nDiag + 1 to nMax
              xTemp = aName[ nDiag, nAt ]
              aName[ nDiag, nAt ] = aName[ nAt, nDiag ]
              aName[ nAt, nDiag ] = xTemp
          endfor
      endfor
      lSuccess = true
      if nRows # nCols
         lSuccess = ( aresize( aName, nCols, nRows, 1 ) = ;
                   nRows * nCols )
      endif
      
      this.a1 = (aName)

   RETURN lSuccess
   // -- EoM: Flip()

   FUNCTION FlipCopy( aName )
   /* 
      -----------------------------------------------------------------
      Programmer...: Bruce Beacham
      Date.........: 19 July 2001
      Notes........: Flips a copy of a two-dimensional array 
                     so the rows become its columns and vice versa, 
                     so you may then use a function such as array 
                     method sort() which works only on rows.
                     Returns the flipped copy array, or false.
                     Leaves the original array untouched.
       Written for.: dB2K
       Rev. History: None
       Calls.......: None
       Called by...: Any
       Usage.......: arrayex.FlipCopy( oArray )
                     arrayex( oArray ).FlipCopy()
       Example.....: a2 = new arrayex( a1 ).FlipCopy()
       Returns.....: fipped copy array, else false
       Parameters..: aName = array object
       ----------------------------------------------------------------
   */

      if not empty(aName)
      elseif type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         aName = this
      elseif type("this.a1") = "A"
         aName = this.a1
      else
         msgbox("No array given.   ", "ArrayEx.FlipCopy", 48)
         return false
      endif

      local oA
      oA = class::Copy( aName )
      if class::Flip(oA)
      else
         return false
      endif

      this.a1 = (oA)
      this.AssignMethods((this.a1))
   return (oA)
   // end of method FlipCopy

   FUNCTION Shuffle( aName )
   /* -------------------------------------------------------------------
     Programmer..: Jay Parsons
     Date........: 11/10/1996
     Notes.......: Random shuffle of elements of a one-dimensioned 
                   array
     Written for.: dBASE 5.0 for Windows
     Rev. History: 10/01/1994 -- Original Release
                   11/10/1996 -- Modified to work with this class.   
                   May 7, 2002 -- Modified to return the 'shuffled' 
                                  array
     Calls.......: mathex.dice()
     Usage.......: arrayex.Shuffle( aName  )
     Example.....: lX = oMyArray.Shuffle( MyDeck )
     Returns.....: NULL
                   But sets the arrayex's a1 property (this.a1) to 
                   the shuffled array. (NB This may already be 
                   implied if the original array was given on 
                   instantiation).
     Parameters..: aName = Name of array, not in quotes
     Side effects: Rearranges elements of the array
                   Uses some random numbers; does not reseed 
                   generator
    ------------------------------------------------------------------- */
      
      local nLen, nAt, nRand, nTemp

      if not empty(aName)
      elseif type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         aName = this
      elseif type("this.a1") = "A"
         aName = this.a1
      else
         msgbox("No array given.   ", "ArrayEx.Shuffle", 48)
         return false
      endif

      nLen = alen( aName )
      for nAt = 1 to nLen - 1
          nRand = this.Math.Dice( nLen - nAt ) + nAt
          nTemp = aName[ nRand ]
          aName[ nRand ] = aName[ nAt ]
          aName[ nAt ] = nTemp
      endfor

      this.a1 = aName
      
   RETURN (aName)
   // -- EoM: Shuffle()

   FUNCTION Txt2Arr ( cFileName ) && FIX: changed name
   /* -------------------------------------------------------------------
     Programmer..: Adam L. Menkes
     Date........: 11/10/1996
     Notes.......: Stuffs each line of a text file into one array 
                   cell. File must be an ASCII Txt formatted file.
                   Taken from Technotes, April, 1992.
     Written for.: dBASE 5.0 for Windows
     Rev. History: 04/01/1992 -- Original
                   10/26/1993 -- Angus Scott-Fleming, release 
                                 "ALL LIKE"
                   09/26/1994 -- Jay Parsons, removed size limits
                   Fall, 1994 -- revised for dBASE 5.0 for Windows.
                                 Jay Parsons, 72662,1302
                   11/10/1996 -- Modified to work with this class.
                                 Removed 'release' of array,
                                 changed to instantiate new array
                                 class, instead, which effectively 
                                 does the same thing.
                   01/11/1997 -- changed to return array 
     Usage.......: arrayex.txt2arr (<cFileName>)
     Example.....: oArray = oArrayEx.txt2arr("CONFIG.DB")
     Returns.....: array
     Parameters..: cFileName  = Name of DOS Text file to read into 
                                array
    ------------------------------------------------------------------ */
      
      local oArray, nSize, nHandle

      nSize = 0
      if type("zArray::zArray") = "FP"
         oArray = new zArray (nSize)
      else
         oArray = new array (nSize)
      endif
      
      // -- get file handle
      nHandle = 0
      nHandle = fopen( cFileName )
      IF nHandle > 0
         // -- store the file into the array
         local nLines
         nLines = 0
         DO WHILE .not. FEOF ( nHandle )
            nLines++
            IF nSize < nLines
               // -- make room in array 
               nSize = nSize + 100
               oArray.resize (nSize)
            ENDIF
            oArray [nLines] = FGETS (nHandle)
         ENDDO

         // -- close the file
         fClose( nHandle )
         oArray.resize (nLines)
      ENDIF
      
      this.AssignMethods((oArray))

   RETURN oArray
   // -- EoM: Txt2Arr()

   FUNCTION Variance      //( aName, nRowCol, nWhich )
   /* -------------------------------------------------------------------
     Programmer..: Jay Parsons
     Date........: 11/10/1996
     Notes.......: Variance of non-blank numeric or date values in 
                   given row or column of a specified array.  If the 
                   first value is a date, considers only dates.  If 
                   first value is numeric or float, considers only 
                   numerics and floats.  Returns .F. if any value in 
                   row or column is any other type, if specified row 
                   or column does not exist or if there are no non
                   blank values. To adapt this to find the sample 
                   variance, substitute
                   "( nCount - 1 )" for the final "nCount" in the 
                   last line calculating xReturn.
     Written for.: dBASE 5.0 for Windows
     Rev. History: 10/01/1994 -- Original Release
                   11/10/1996 -- Modified to work with this class.
     Calls.......: None
     Usage.......: arrayex.Variance( <aName> [, <nRowCol> ;
                                     [, <nWhich>]] )
     Example.....: ? oMyArray.Variance( Myarray, 1, 1 )
     Returns.....: A float value, the population variance.
     Parameters..: aName   = the name of the array, not in quotes.
                   nRowCol = the number of the row or column to 
                             average, default being 1
                   nWhich  = one of these values:
                             1 = nRowCol specifies a row (default)
                             2 = nRowCol specifies a column
    ------------------------------------------------------------------- */
      
      private aName, nR, nW, nRowCol, nWhich
      
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oA = this
         nRowCol = argvector(1)
         nWhich = argvector(2)
      elseif type("argvector(1)") = "A"
         aName = argvector(1)
         nRowCol = argvector(2)
         nWhich = argvector(3)
      elseif type("argvector(1)") <> "A" and type("this.a1") = "A"
         aName = this.a1
         nRowCol = argvector(1)
         nWhich = argvector(2)
      else
         msgbox("No array given.   ", "Arrayex.Avg", 48)
         return false
      endif

      local xReturn, cType, nAt, nTot, nCount, nTotSq, xNext, cType2
      
      nW = iif( nWhich = 2, 2, 1 )
      nR = iif( type("nRowCol")="N", nRowCol, 1 )
      xReturn = .F.
      nDims = aName.dimensions
      if nDims = 1
         if nR # 1 .or. nW # 1
            RETURN xReturn
         else
            cType = type( "aName[ nR ]" ) 
         endif
      else
         if nW = 2
            cType = type( "aName[1,nR]" )
         else
            cType = type( "aName[nR,1]" )
         endif
      endif
      if .not. cType $ "DFN"
         RETURN xReturn
      endif
      store 0 to nTot, nCount, nTotSq
      for nAt = 1 to alen( aName, nW )
          do case
             case nDims = 1
                  xNext = aName[ nAt ]
             case nW = 1
                  xNext = aName[ nR, nAt ]
             otherwise
                  xNext = aName[ nAt, nR ]
          endcase
          cType2 = type( xNext )
          if ( .not. cType2 $ "DFN" ) .or. ;
             ( ( cType = "D" ) # ( cType2 = "D" ) )
             exit
          endif
          if isblank( xNext )
             loop
          endif
          xReturn = true
          if cType2 = "D"
             xNext = xNext - {01/01/01}
          endif
          nTot = nTot + xNext
          nTotSq = nTotSq + xNext * xNext
          nCount++ 
       endfor
       if xReturn # false
          xReturn = ( nTotSq - nTot * nTot / nCount ) / ;
                    nCount
       endif
       
   RETURN xReturn
   // -- EoM: Variance()


   FUNCTION Min
   /*
     ------------------------------------------------------------------
     Programmer..: Bruce Beacham
     Date........: 19 July 2001
     Notes.......: Gives the minimum of the values in a 1D array, or 
                   the elements or a specified row or column of a 
                   2D array. The array should be given as method 
                   parameter 1. If not given as parameter 1, the 
                   method looks for arrayex property a1, which may 
                   be parameterised-in on instantiation, 
                   or otherwise assigned.
     Written for.:  dB2K
     Rev. History: None
     Calls.......: 
     Usage.......: arrayex.Min( [<aName> [,] <nRowCol> [, <nWhich>]] )
     Example.....: ? new Arrayex({5, 2, 7}).Min()
                   ? oMyArray.Min(My2DArray, 2, 2)
                   a = new array()
                   a.dir()
                   ? new arrayex(a).min(3, 2)
     Returns.....: The minimum value found.   Or false, if no values.
                   The object properties MinMax and MinMaxRowCol are 
                   set by the sub-method, holding respectively the 
                   result and the Row/Column in which it was found.
     Parameters..: aName   = a reference to the array object.  
                             Optional:  no comma needed if omitted.
                   nRowCol = the number of the row or column holding 
                             the values the minimum of which is to 
                             be reported, default being 1
                   nWhich  = one of these values:
                             1 = nRowCol specifies that a row is to 
                                 be searched (default)
                             2 = nRowCol specifies a column
      Warning: Sort() will throw an error if the row or column 
               does not contain all elements of the same type.
     ------------------------------------------------------------------
   */

      local result
      result = class::MinMax(argvector(1), argvector(2), argvector(3), 0)

   return (result)
   // end of method Min
   	
   FUNCTION Max
   /*
     -------------------------------------------------------------------
     Programmer..: Bruce Beacham
     Date........: 19 July 2001
     Notes.......: Gives the maximum of the values in a 1D array, or the 
                   elements or a specified row or column of a 2D array.
                   The array should be given as method parameter 1.   
                   If not given as parameter 1, the method looks for 
                   arrayex property a1, which may be parameterised-in 
                   on instantiation, or otherwise assigned.
     Written for.: dB2K
     Rev. History: None
     Calls.......: 
     Usage.......: arrayex.Max( [<aName> [,] <nRowCol> [, <nWhich>]] )
     Example.....: ? new Arrayex({5, 2, 7}).Max()
                   ? oMyArray.Max(My2DArray, 2, 2)
                   a = new array()
                   a.dir()
                   ? new arrayex(a).max(3, 2)
     Returns.....: The maximum value found.   Or false, if no values.
                   The object properties MinMax and MinMaxRowCol are 
                   set by the sub-method, holding respectively the 
                   result and the Row/Column in which it was found.
     Parameters..: aName   = a reference to the array object.  
                             Optional:  no comma needed if omitted.
                   nRowCol = the number of the row or column holding 
                             the values the maximum of which is to 
                             be reported, default being 1
                   nWhich  = one of these values:
                             1 = nRowCol specifies that a row is to 
                                 be searched (default)
                             2 = nRowCol specifies a column
      Warning: Sort() will throw an error if the row or column does not
               contain all elements of the same type.
      ------------------------------------------------------------------- */

      local result
      result = class::MinMax(argvector(1), argvector(2), argvector(3), 1)
   return (result)
   // end of method Max


   FUNCTION MinMax
      /*
         This is the common core of the Min() and Max() methods.
         It sets the arrayex object properties MinMax (the result of 
         the Min() or Max() and MinMaxRowCol (the row or column 
         (which the user specified) that contains the minimum or 
         maximum value found).
      */
      local oA, s, rows, cols
      private a, nWhich, nRowCol
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oA = this
         nRowCol = argvector(1)
         nWhich = argvector(2)
      elseif type("argvector(1)") = "A"// given an array to this method
         oA = argvector(1)
         nRowCol = argvector(2)
         nWhich = argvector(3)
      elseif type("this.a1") = "A"   // provided either on 
                                     // instantiation, or subsequently.
         oA = this.a1
         nRowCol = argvector(1)
         nWhich = argvector(2)
      else
         msgbox("Cannot find an array.   ", "ArrayEx.Min", 48)
         return false
      endif

      MinMax = argvector(4)

      if alen(oA, 1) = 0
         msgbox("No rows in this array.   ", "ArrayEx.Min", 48)
         return false
      endif

      if type("nRowCol") = "N" and type("nWhich") = "N"
         if nRowCol < 1
            msgbox("Specified a sort row or column less than 1.   ",;
                   "ArrayEx.MinMax", 48)
            return false
         elseif nWhich = 1 and nRowCol > alen(oA, 1)
            msgbox("Specified a row beyond the array dimensions.   ",;
                   "ArrayEx.MinMax", 48)
            return false
         elseif nWhich = 2 and nRowCol > alen(oA, 2)
            msgbox("Specified a column beyond the array dimensions.   ",;
                   "ArrayEx.MinMax", 48)
            return false
         elseif nWhich <> 1 and nWhich <> 2
            msgbox("Specified " + nwhich + ;
                   ", which is neither Row nor Column.",;
                   "ArrayEx.MinMax", 48)
            return false
         endif
      elseif type("nRowCol") = "N"
         nWhich = 1
      elseif type("nWhich") = "N"
         nRowCol = 1
      else
         store 1 to nWhich, nRowCol
      endif
   	
      a = class::copy(oA)  // make a sortable copy.

      local result
      do case
      case a.dimensions = 1
         nRowCol = 1
      case nWhich = 1
         // Sort on a row's elements.
         class::Flip(a)    // the desired row is now a column.
      case nWhich = 2
         // Sort on a column's elements.
      endcase

      a.grow(2)
      rows = alen(a, 1)
      cols = alen(a, 2)
      for s = 1 to rows ; a[s, cols] = s ; endfor
      a.sort(nRowCol, alen(a, 1), MinMax)  
           // an error here means you have mixed data types.
      this.MinMax = a[1, nRowCol]
      this.MinMaxRowCol = a[1, cols]

   return (this.MinMax)
   // end of method MinMax().

   FUNCTION CarveOut
   /*
     ------------------------------------------------------------------
     Programmer...: Bruce Beacham
     Date.........: 8 August 2001
     Notes........: Returns a row or a column of a 2D array as a 1D 
                    array. NB Thus it flips a row into a column.
                    Sets the new 1D array as the arrayex object's 
                    a1 array (this.a1) ready for subsequent methods.   
                    Assigns the arrayex methods to the new array
     Written for.:  dB2K
     Rev. History:  28 October 2002  BB fixed error arising when a 1D array 
                   is unexpectedly given as the source array.
     Calls.......: 
     Usage.......: arrayex.CarveOut( [<aName> ,]  [<nRowCol> ;
                   [,<nWhich>]] )
     Example.....: ? new arrayex(My2DArray).CarveOut(2, 1)
                   // returns a 1D array of the elements in 
                   // the second row of the 2DArray.

                   a = new array()
                   a.dir()
                   ? new arrayex(a).CarveOut(2, 2) 
                        // returns just the file sizes as a 1D array.
                   ? new arrayex(a).CarveOut()    
                        // returns just the file names as a 1D array.

     Parameters..: aName   = a reference to the array object.  
                     Optional:  no comma needed if omitted.
                   nRowCol = the number of the row or column holding 
                           the values to be Carved out into a 1D array, 
                           default 1.
                   nWhich  = one of these values:
                           1 = nRowCol specifies that a row is to be 
                               searched (NOT the default)
                           2 = nRowCol specifies a column (default)
     ------------------------------------------------------------------
   */

      local oA, s, rows, cols
      private a, nWhich, nRowCol
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oA = this
         nRowCol = argvector(1)
         nWhich = argvector(2)
      elseif type("argvector(1)") = "A"// given an array to this method.
         oA = argvector(1)
         nRowCol = argvector(2)
         nWhich = argvector(3)
      elseif type("this.a1") = "A"   // provided either on 
                                     // instantiation, or subsequently.
         oA = this.a1
         nRowCol = argvector(1)
         nWhich = argvector(2)
      else
         msgbox("Cannot find an array.   ", "ArrayEx.CarveOut", 48)
         return false
      endif

      if alen(oA, 1) = 0
         nWhich =0      // bypasses further action below.
      endif

      if type("nRowCol") = "N" and type("nWhich") = "N"
         if nRowCol < 1
            msgbox("Specified a sort row or column less than 1.   ",;
                   "ArrayEx.CarveOut", 48)
            return false
         elseif nWhich = 1 and nRowCol > alen(oA, 1)
            msgbox("Specified a row beyond the array dimensions.   ",;
                   "ArrayEx.CarveOut", 48)
            return false
         elseif nWhich = 2 and nRowCol > alen(oA, 2)
            msgbox("Specified a column beyond the array dimensions.   ",;
                   "ArrayEx.CarveOut", 48)
            return false
         elseif nWhich <> 1 and nWhich <> 2
            msgbox("Specified " + nwhich + ;
                   ", which is neither Row nor Column.",;
                   "ArrayEx.CarveOut", 48)
            return false
         endif
      elseif type("nRowCol") = "N"
         nWhich = 2
      elseif type("nWhich") = "N"
         nRowCol = 1
      else
         nRowCol = 1
         nWhich = 2
      endif

      local aNew, s
      if type("zArray::zArray") = "FP"
         aNew = new zArray()
      else
         aNew = new array()
      endif

      do case
         case nWhich = 0
            // No rows in the array - return an empty array.
         case nWhich = 1 and oA.dimensions = 1
            // Carve out a row.
            // 1D source array, so returns one element only.
            aNew.add( oA[nRowCol] )
         case nWhich = 1
            // Carve out a row.    Return it as a column.
            for s = 1 to alen(oA, 2)
               aNew.add( oA[nRowCol, s] )
            endfor
         case nWhich = 2 and oA.dimensions = 1
            // Carve out a column.
            // 1D source array, so returns one element only.
            for s = 1 to alen(oA, 1)
               aNew.add( oA[nRowCol] )
            endfor
         case nWhich = 2
            // Carve out a column.
            for s = 1 to alen(oA, 1)
               aNew.add( oA[s, nRowCol] )
            endfor
         otherwise
            aNew = false
      endcase

      this.a1 = (aNew)
      this.AssignMethods((this.a1))
   return (aNew)
   // end of method CarveOut


   FUNCTION Upper
   /* 
     ------------------------------------------------------------------
     Programmer..: Bruce Beacham
     Date........: 8 August 2001
     Notes.......: Returns a COPY of an array with all character 
                   elements capitalised.
                   The array submitted, or the array of which this is a 
                   method (if the array is a zArray), is unaltered.
     Written for.: dB2K
     Rev. History: None
     Calls.......: 
     Usage.......: bName = new arrayex().Upper( [<aName>])
     Example.....: listarray( new arrayex(My2DArray).Upper())
                   // lists My2DArray with all character elements 
                   // capitalised.

                   a = new array()
                   a.dir()
                   b = new arrayex(a)
                   b.CarveOut()
                   b.Upper()    // returns a 1D array of capitalised 
                                // filenames.
                   b.listarray()  // lists the file names, capitalised.
                     OR
                   a = new array()
                   a.dir()
                   new arrayex(a).CarveOut().Upper().listarray()
                   (To use chaining thus, you must start with an 
                   
                   If you start with an array:
                   i = new zArray()
                   i.dir()
                   i.upper()              // does NOT alter i.
                   i.upper().listarray()   // does not alter i 
                             // but lists it in capitals.  Compare with...
                   i.listarray()    // lists the array still unaltered.
                   k = i.upper()       // Produces an uppercase array k.

     Returns.....: A capitalised duplicate array, which is also set 
                   as the arrayex object's a1 property.
     Parameter..:  A reference to the array object to be duplicated 
                   in capitals. Optional if the array was provided 
                   as the first parameter upon instantiation.
     ------------------------------------------------------------------
   */
      private aNew, nRow, nCol
      local oA, rows, cols
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         oA = this
      elseif type("argvector(1)") = "A"// given an array to this method.
         oA = argvector(1)             
      elseif type("this.a1") = "A"  // provided either on 
                                    // instantiation, or subsequently.
         oA = this.a1
      else
         msgbox("Cannot find an array.   ", "ArrayEx.Upper", 48)
         return false
      endif

      if oA.dimensions = 1 and type("zArray::zArray") = "FP"
         aNew = new zArray(oA.size)
      elseif oA.dimensions = 1
         aNew = new array(oA.size)
      elseif oA.dimensions = 1 and type("zArray::zArray") = "FP"
         aNew = new zArray(alen(oA, 1), alen(oA, 2))
      else
         aNew = new array(alen(oA, 1), alen(oA, 2))
      endif

      if alen(oA, 1) > 0
         aCopy(oA, aNew)
      endif
	
      if aNew.dimensions = 1
         rows = aNew.size
         for nRow = 1 to rows
            if type("aNew[nRow]") = "C"
               aNew[nRow] = upper(aNew[nRow])
            endif
         endfor
      else
         rows = alen(aNew, 1)
         cols = alen(aNew, 2)
         for nRow = 1 to rows
            for nCol = 1 to cols
               if type("aNew[nRow, nCol]") = "C"
                  aNew[nRow, nCol] = upper(aNew[nRow, nCol])
               endif
            endfor
         endfor
      endif

      this.a1 = (aNew)
      this.AssignMethods((this.a1))
   return (aNew)
   // end of method Upper

   FUNCTION ListArray
   /* 
     ------------------------------------------------------------------
     Programmer..: Bruce Beacham, from code first seen in the 
                   Programming NG.
     Date........: 8 August 2001
     Notes.......: Lists an array given as a parameter, failing which 
                   the arrayex object's a1 property.
                   This method can list:
                        an array
                        an assocarray
                        a sourcealiasarray (dB2K)
                        a tabledef fields array
                        an object, stock or otherwise, 
                           for which an enumeration of properties etc 
                           is desired.
     Written for.: dB2K
     Rev. History: None
     Calls.......: 
     Usage.......: new arrayex().listarray(MyArray)
     Example.....: b = new arrayex(My2DArray)
                   b.Upper()
                   b.listarray()
                   // lists (a duplicate of) My2DArray with all 
                   // character elements capitalised.

                   a = new array()
                   a.dir()
                   b = new arrayex(a)
                   b.CarveOut()
                   ? b.listarray()  // lists the file names and 
                                    // returns the number of them.
                   b = new arrayex(new form())
                   ? b.listarray()	// enumerates the properties 
                                    // and returns the number of 
                                    // elements (2x the properties).
     Returns.....: The number of rows.
     Parameter...: A reference to the array object to be listed.
                   Optional if the array was provided as the first 
                   parameter upon instantiation.
     ------------------------------------------------------------------
   */

      private aArg
      if type("this") = "A"
         // Array has had arrayex method calls attached by 
         // arrayex::AssignMethods()
         aArg = this
      elseif type("argvector(1)") $ "OA"// given an array or other object 
                                        // to this method.
         aArg = argvector(1)
      elseif type("this.a1") $ "OA"  // provided either on instantiation, 
                                     // or subsequently.
         aArg = this.a1
      else
         msgbox("Cannot find an array.   ", "ArrayEx.ListArray", 48)
         return false
      endif

      local nElement, nCols, nRows, rv
      private nCol, nRow
      do case
         case .not. type("aArg") $ "AO"
            msgbox("Not an array.   ", "ArrayEx.ListArray", 48)
            rv = false
         case type("aArg.classname") <> "C"
            // Probably an object with no stock properties, like 
            // _app.appslist. Create an enumeration, which is an 
            // assocarray of the properties, methods and events.
            a = enumerate(aArg)
            ?
            local f
            f = a.firstkey
            do while .not. empty(f)
               ? f, ":", a[ f ], iif(type("a[ f ]") = "O", ;
                         a[ f ].classname, "")
               f = a.nextkey(f)
            enddo
            rv = a.count()
         case aArg.classname = "ASSOCARRAY" .or. ;
              aArg.classname = "SOURCEALIASARRAY"
            ?
            local f
            f = aArg.firstkey
            do while .not. empty(f)
               ? f, ":", aArg[ f ], iif(type("aArg[ f ]") = "O",;
                         aArg[ f ].classname, "")
               f = aArg.nextkey(f)
            enddo
            rv = aArg.count()
         case type("aArg.dimensions") = "U" .and. type("aArg.size") = "N"
            // May be a TableDef fields array -- that is a list of 
            // field objects in a 1D array.
            nRows = aArg.size   // Determine # of rows
            nCols = 4           // Determine # of columns
            fa = new array(4)
            for nRow = 1 to nRows
               ?                // Each row on its own line
               fa[1] = aArg[ nRow ].fieldname 
               fa[2] = aArg[ nRow ].type 
               fa[3] = aArg[ nRow ].length 
               fa[4] = iif(type("aArg[ nRow ].decimallength") = "U", ;
                       "", aArg[ nRow ].decimallength)
               for nCol = 1 to nCols // Display each row as before
                  ?? fa[ nCol ] + "  :  "
               endfor
               if type("aArg[nRow].value") <> "U"
                  ?? "value=:"
                  ?? aArg[nRow].value
                  ?? ":"
               endif
            endfor
            rv = aArg.size
         case type("aArg.dimensions") = "U"
            // A different class of object, probably not an array, 
            // eg a rowset. Create an enumeration, which is an 
            // assocarray of the properties, methods and events.
            a = enumerate(aArg)
            ?
            local f
            f = a.firstkey
            do while .not. empty(f)
               ? f, ":", a[ f ], iif(type("a[ f ]") = "O",;
                         a[ f ].classname, "")
               f = a.nextkey(f)
            enddo
            rv = a.count()
         case aArg.dimensions == 1
            ?
            for nElement = 1 to aArg.size  // Display elements
               ? aArg[ nElement ]
            endfor
            rv = aArg.size
         case aArg.dimensions == 2
            nRows = alen( aArg, 1 )        // Determine # of rows
            nCols = alen( aArg, 2 )        // Determine # of columns
            for nRow = 1 to nRows
              ?                            // Each row on its own line
              for nCol = 1 to nCols        // Display each row as before
                ?? aArg[ nRow, nCol ] + "  :  "
              endfor
            endfor
            rv = alen(aArg, 1)
         otherwise
            msgbox( "Cannot cope with an array of more than two dimensions.",;
                    "ArrayEx.ListArray", 48)
            rv = false
      endcase
   return (rv)
   // end of method ListArray

   FUNCTION GridArray
      /*
        Programmer..: Bruce Beacham
        Date........: 3 August 2002
        Notes.......:
          Copies the relevant array to a table and opens it
          in a grid.   The grid is then opened in a form.
         Returns a reference to the form object.   Or false.

         2 December 2006:  BHB: Made the temp file field widths
         sensitive to the column headings.   Headings help the
         viewer to widen the columns displayed to suit.
         12 Jan 2007:   BHB:  Catered for null fields in the first row
         of the grid (used for defining the column's field type).
         Added logical parameter to force display of grid col headings,
         to make it easy for the user to widen the columns as desired.
         29 May 2007:  BHB:  Made the intermediate structure table's
         field widths more responsive to the lengths of
         the column headers provided.
         9 Feb 2008:   BHB:  Fixed bug when there is a headings array
         for a 1D array.
         Also, added facility for specifying column widths.
         Also, revised method of ensuring column headings are unique.

      Example:
         a = new array()
         a.dirext()
         // optional line:   restricts the number of cols displayed:
         a.headings = {"Name", "Size", "Date Created", "Time"}
         b = new arrayex(a)
         f = b.GridArray()             // returns the resulting form.
      */

      parameter forceheadings       // logical.

      private aArg
      if type("this") = "A"
         // Array has had arrayex method calls attached by
         // arrayex::AssignMethods()
         aArg = this
      elseif type("argvector(1)") $ "OA"
         // given an array or other object to this method.
         aArg = argvector(1)
      elseif type("this.a1") $ "OA"
         // provided either on instantiation, or subsequently.
         aArg = this.a1
      else
         msgbox("Cannot find an array.   ", "ArrayEx.ListArray", 48)
         return false
      endif

      * Validate.
      do case
      case not type("aArg") $ "A"
         * We do not attempt to cater for assocarrays and other
         *  special 1D arrays - they can be inspected
         *  in the :dUFLP:KMInspector.
         msgbox("Not an array.   ", "ArrayEx.GridArray", 48)
         return false
      case type("aArg.dimensions") <> "N"
         msgbox("No dimensions property.   Not a regular array.   ", ;
            "ArrayEx.GridArray", 48)
         return false
      case type("aArg.size") <> "N"
         msgbox("No size property.   Not a regular array.   ", ;
            "ArrayEx.GridArray", 48)
         return false
      case aArg.size < 1
         msgbox("No rows or columns in the array.    ", ;
            "ArrayEx.GridArray", 48)
         return false
      endcase

      * Create the temporary file's structure.
      private tf1, tf2, maxHeadLength           // 2 Dec 2006
      local numbCols, hasheadings, ss, multicolumn
      local colType, colWidth, colDecs
      private s, cct
      tf1 = funique("tempgrid1_????.dbf")
      tf2 = funique("tempgrid2_????.dbf")
      if type("aArg.headings") = "A"
         * The array has been provided with headings in a 1D array
         *  which we shall use as field names and column headings.
         * Also, the number of headings will be used to limit
         *  the number of columns (the first n) we take from the array.
         * That prevents errors lower down.
         if aArg.dimensions = 1
            // Must catch a 1D array as alen(aArg, 2) = 0 and
            // so file creation below fails for lack of fields.
            numbCols = 1
            if aArg.headings.size = 0
               aArg.headings.add("Col 1")
            endif
         else
            numbCols = min(alen(aArg, 2), aArg.headings.size)
         endif
         hasheadings = true

         if aArg.headings.dimensions > 1
            msgbox("Headings must be 1D.  Ignored.   ", "GridArray", 48)
            hasheadings = false
         endif
         * Make headings unique.
         local ah, s1, s2, s3, hd
         ah = aArg.headings
         for s1 = 1 to ah.size - 1
            hd = upper(""+ah[s1])
            s3 = 1
            for s2 = s1+1 to ah.size
               if hd == upper(ah[s2])
                  ah[s2] += "_" + s3
                  s3 ++
               endif
            endfor
         endfor
         aArg.headings = ah

         maxHeadLength = 10         // starting length.
         // Work out the max length needed for the field headings given.
         for s = 1 to aArg.headings.size
            maxHeadLength = max(maxHeadLength, clen(aArg.headings[s]))
         endfor
         // Make a table accordingly.    29 May 2007
         cct = 'create table "' + tf1 + '" ' ;
            + '(field_name char(' + maxHeadLength + '), ' ;
            + 'field_type char(1), field_len integer, ' ;
            + 'field_dec integer)'
         &cct.
         use (tf1)
      else
         numbCols = max(alen(aArg, 2), 1)
         hasheadings = false
         create (tf1) structure extended
      endif

      multicolumn = aArg.dimensions > 1
      * Create fields for each column in the array.
      for s = 1 to numbCols
         append blank
         if multicolumn
            colType = type("aArg[1, s]")
            if colType = "C"
               * Get the maximum width of the data.
               if hasheadings          // 2 Dec 2006
                  * You can make these headings wide enough
                  *  to suit your data.   This determines the
                  *  field length in the temp file.
                  * Note that the grid comes with standard col widths
                  *  and you widen them with the heading width handles
                  *  to see fields that are wider than the display.
                  colWidth = clen(aArg.headings[s])
               else
                  colWidth = 1     // must be at least 1.  12 Jan 2007
               endif
               // Run down the column and ascertain max width of data.
               for ss = 1 to alen(aArg, 1)
                  colWidth = max(colWidth, len(trim(aArg[ss, s])))
               endfor
               replace field_len with colWidth
            elseif colType = "N"
               colWidth = 1
               coldecs = false
               for ss = 1 to alen(aArg, 1)
                  colWidth = max(colWidth, aArg[ss, s])
                  coldecs = ;
                     iif(coldecs, true, mod(aArg[ss, s], 1) > 0)
               endfor
               if coldecs
                  replace field_len ;
                     with log10(colWidth)+2 + set("deci")+1, ;
                     field_dec with set("deci")
               else
                  replace field_len ;
                     with log10(colWidth)+2, ;
                     field_dec with 0
               endif
            elseif colType = "U"     // 12 Jan 07
               // probably null
               replace field_len with 10
            endif
         else
            * There is only one column in the array.
            colType = type("aArg[1]")
            if colType = "C"
               * Get the maximum width of the data.
               colWidth = 0
               for ss = 1 to alen(aArg, 1)
                  colWidth = max(colWidth, len(trim(aArg[ss])))
               endfor
               replace field_len with colWidth
            elseif colType = "N"
               colWidth = 0
               coldecs = false
               for ss = 1 to alen(aArg, 1)
                  colWidth = max(colWidth, aArg[ss])
                  coldecs = ;
                     iif(coldecs, true, mod(aArg[ss], 1) > 0)
               endfor
               if coldecs
                  replace field_len ;
                     with log10(colWidth)+2 + set("deci")+1, ;
                     field_dec with set("deci")
               else
                  replace field_len ;
                     with log10(colWidth)+2, ;
                     field_dec with 0
               endif
            elseif colType = "U"     // 12 Jan 07
               // probably null
               replace field_len with 10
            endif
         endif
         replace field_type with ;
               iif(colType = "U", "C", coltype)     // 12 Jan 07
         if hasheadings and not empty(aArg.headings[s])
            replace field_name with aArg.headings[s]
                 // removed 9 Feb 08 due to check above.   + "_" + s
         else
            replace field_name with "Field_" + s
         endif
      endfor
      create (tf2) from (tf1)
      use
      delete file (tf1)

      * Now get the array elements into the table that will
      *  appear in the grid.
      if select("GridArr") <> 0
         use in GridArr
      endif
      if multicolumn
         use (tf2) in select() alias GridArr
         append from array aArg
         use
      else
         local q
         q = new query([select * from "] + tf2 + ["])
         for s = 1 to aArg.size
            q.rowset.beginappend()
            q.rowset.fields[1].value = aArg[s]
         endfor
         q.rowset.save()
         q.active = false
         release object q
      endif

      * Now make a form and grid to show the data.
      local f
      f = new form()
      f.text = "Display of an array"
      f.scaleFontSize = 8
      f.move(5, 2, 20, 10)
      f.g = new grid(f)
      f.g.anchor = 6    // container
      f.g.bgColor = "white"
      f.g.cellHeight = 0.8
      f.g.hasColumnLines = false
      f.g.hasRowLines = false
      f.g.hasColumnHeadings = hasheadings or forceheadings
      f.g.allowediting = false
      f.q = new query([select * from "] + tf2 + ["])
      f.g.datalink = f.q.rowset
      f.rowset = f.q.rowset
      f.onClose = {; flnm = this.q.rowset.tablename ; ;
         this.q.active = false ; release object this.q ; ;
         delete file (flnm) }

      * Added 9 Feb 08.    Allows programmer to specify column widths.
      local haswidths
      haswidths = (type("aArg.fieldwidths") = "A") ;
         and aArg.fieldwidths.dimensions = 1
         * The array has been provided with column widths in a 1D array.
      private gc, nS, ccol
      for nS = 1 to iif(aArg.dimensions = 1, 1, alen(aArg, 2))
         gc = new gridcolumn(f.g)
         gc.datalink = f.q.rowset.fields[nS]
           // invalid subscript here means the headings array is
           // less than the number of array columns.   Checked above.
         gc.headingcontrol.value := f.q.rowset.fields[nS].fieldname
         if haswidths and aArg.fieldwidths.size >= nS
            gc.width := aArg.fieldwidths[nS]
         endif
         ccol = "column" + nS
         f.g.columns[ccol] = gc
         f.g.columns.add(gc)
      endfor


      f.open()
   return (f)
   // end of method GridArray

   FUNCTION AssignMethods
   /* 
     ------------------------------------------------------------------
     Programmer..: Bruce Beacham
     Date........: 30 August 2001
     Notes.......: Assigns some of the methods of this class to an 
                   array which is returned from this class - this 
                   enables a chain of the methods of this class to 
                   be applied in a single command line. If there is 
                   already a method or property of the method name, 
                   the method is not assigned. The assignment is a 
                   pointer, so this arrayex class must stay on the path.
                   New arrays created in class arrayex are zArrays 
                   if that class is on the path.
     Written for.: dB2K
     Rev. History: None
     Calls.......: 
     Usage.......: Not called externally.   
                   Principally (Only, at present) called by the 
                   constructor code, to be applied to an array 
                   parameterised in (or both, if true).
     Example.....: (Of the effect of this method's work):
                   a = new array()
                   a.dir()
                   ? new arrayex(a).CarveOut().Upper().listarray()
                     // (the ? is optional, it merely tells the 
                     number of rows in the array)
     Returns.....: The array parameterised-in with extra methods added.
     Parameter..:  A reference to the array object to have methods 
                   assigned to it.    Optional if the array was provided 
                   as the first parameter upon instantiation.
     ------------------------------------------------------------------
  */

     private aArg
     if type("argvector(1)") $ "OA"	// given an array or other 
                                    // object to this method.
        aArg = argvector(1)
     elseif type("this.a1") $ "OA"	// provided either on 
                                    // instantiation, or subsequently.
        aArg = this.a1
     else
        msgbox("Cannot find an array.   ", "ArrayEx.ListArray", 48)
        return false
     endif

     if type("aArg.Min") = "U"
        aArg.Min = class::Min
     endif
     if type("aArg.Max") = "U"
        aArg.Max = class::Max
     endif
     if type("aArg.ListArray") = "U"
        aArg.ListArray = class::ListArray
     endif
     if type("aArg.StringArray") = "U"
        aArg.StringArray = class::StringArray
     endif
     if type("aArg.GridArray") = "U"
        aArg.GridArray = class::GridArray
     endif
     if type("aArg.Copy") = "U"
        aArg.Copy = class::Copy
     endif
     if type("aArg.CopySimple") = "U"
        aArg.CopySimple = class::CopySimple
     endif
     if type("aArg.FlipCopy") = "U"
        aArg.FlipCopy = class::FlipCopy
     endif
     if type("aArg.ADiff") = "U"
        aArg.ADiff = class::ADiff
     endif
     if type("aArg.ADiffA") = "U"
        aArg.ADiffA = class::ADiffA
     endif
     if type("aArg.Avg") = "U"
        aArg.Avg = class::Avg
     endif
     if type("aArg.CarveOut") = "U"
        aArg.CarveOut = class::CarveOut
     endif
     if type("aArg.Shuffle") = "U"
        aArg.Shuffle = class::Shuffle
     endif
     if type("aArg.Upper") = "U"
        aArg.Upper = class::Upper
     endif
     if type("aArg.AssignMethods") = "U"
        // Needed to propogate arrays throughout a chain of method calls.
        aArg.AssignMethods = class::AssignMethods
     endif
   return
   // end of method AssignMethods

   FUNCTION StringArray
      /*
        ------------------------------------------------------------------
        Programmer..: Bruce Beacham
        Date........: April 9, 2005
        Notes.......: Produces a string from the elements of an array
                      given as a parameter,
                      failing which the arrayex object's a1 property.
                      This method can list:
                           an array
                           an assocarray
                           a sourcealiasarray (dB2K)
                           a tabledef fields array
                           an object, stock or otherwise,
                              for which an enumeration of properties etc
                              is desired.
        Written for.: Plus
        Rev. History: None
        Calls.......:
        Usage.......: myform.editor1.value = ;
                              new arrayex().StringArray(MyArray)
        Example.....: b = new arrayex(My2DArray)
                      b.Upper()
                      ? b.stringarray()
                      // returns a string and thus
                      // lists (a duplicate of) My2DArray with all
                      // character elements capitalised.

                      // Turnkey:
                      a = new zArray()
                      a.dir()
                      b = a..CarveOut()
                      f = new form()
                      f.ed = new editor(f)
                      f.ed.anchor = 6
                      f.ed.value = b.stringarray().toLowerCase()
                      f.open()
                              // produces a string object
                              // which lists the file names in lower case
                              // and which is then thrown to a window.

                      b = new arrayex(new form())
                      ? b.stringarray()	// enumerates the properties
                                       // and returns a string of them.
        Returns.....: A string object.
        Parameter...: A reference to the array object to be listed.
                      Optional if the array was provided as the first
                      parameter upon instantiation.
        ------------------------------------------------------------------
      */

      private aArg
      if type("this") = "A"
         // Array has had arrayex method calls attached by
         // arrayex::AssignMethods()
         aArg = this
      elseif type("argvector(1)") $ "OA"
         // given an array or other object to this method.
         aArg = argvector(1)
      elseif type("this.a1") $ "OA"
         // provided either on instantiation, or subsequently.
         aArg = this.a1
      else
         msgbox("Cannot find an array.   ", "ArrayEx.ListArray", 48)
         return false
      endif

      local nElement, nCols, nRows, rv
      private nCol, nRow, oS
      oS = new string()

      do case
         case .not. type("aArg") $ "AO"
            msgbox("Not an array.   ", "ArrayEx.ListArray", 48)
            rv = false
         case type("aArg.classname") <> "C"
            // Probably an object with no stock properties, like
            // _app.appslist. Create an enumeration, which is an
            // assocarray of the properties, methods and events.
            a = enumerate(aArg)
            local f
            f = a.firstkey
            do while .not. empty(f)
               oS.string += f + " : " + a[ f ] + " : " ;
                  + iif(type("a[ f ]") = "O", a[ f ].classname, "")
               oS.string += chr(13)
               f = a.nextkey(f)
            enddo
            rv = a.count()
         case aArg.classname = "ASSOCARRAY" .or. ;
              aArg.classname = "SOURCEALIASARRAY"
            local f
            f = aArg.firstkey
            do while .not. empty(f)
               oS.string += f + " : " + aArg[ f ] + " : " ;
                  + iif(type("aArg[ f ]") = "O", ;
                  aArg[ f ].classname, "")
               oS.string += chr(13)
               f = aArg.nextkey(f)
            enddo
            rv = aArg.count()
         case type("aArg.dimensions") = "U" ;
               and type("aArg.size") = "N"
            // May be a TableDef fields array -- that is a list of
            // field objects in a 1D array.
            nRows = aArg.size   // Determine # of rows
            nCols = 4           // Determine # of columns
            fa = new array(4)
            for nRow = 1 to nRows
               fa[1] = aArg[ nRow ].fieldname
               fa[2] = aArg[ nRow ].type
               fa[3] = aArg[ nRow ].length
               fa[4] = iif(type("aArg[ nRow ].decimallength") = "U", ;
                    "", aArg[ nRow ].decimallength)
               for nCol = 1 to nCols // Display each row as before
                  oS.string += fa[ nCol ] + "  :  "
               endfor
               if type("aArg[nRow].value") <> "U"
                  oS.string += "value=:"
                  oS.string += aArg[nRow].value
                  oS.string += ":"
               endif
               oS.string += chr(13)    // Each row on its own line
            endfor
            rv = aArg.size
         case type("aArg.dimensions") = "U"
            // A different class of object, probably not an array,
            // eg a rowset. Create an enumeration, which is an
            // assocarray of the properties, methods and events.
            a = enumerate(aArg)
            local f
            f = a.firstkey
            do while .not. empty(f)
               oS.string += f + " : " + a[ f ] + " : " ;
                  + iif(type("a[ f ]") = "O",;
                  a[ f ].classname, "")
               oS.string += chr(13)
               f = a.nextkey(f)
            enddo
            rv = a.count()
         case aArg.dimensions == 1
            for nElement = 1 to aArg.size  // Display elements
               oS.string += aArg[ nElement ]
               oS.string += chr(13)
            endfor
            rv = aArg.size
         case aArg.dimensions == 2
            nRows = alen( aArg, 1 )        // Determine # of rows
            nCols = alen( aArg, 2 )        // Determine # of columns
            for nRow = 1 to nRows
              for nCol = 1 to nCols        // Display each row as before
                oS.string += aArg[ nRow, nCol ] + "  :  "
              endfor
              oS.string += chr(13)
            endfor
            rv = alen(aArg, 1)
         otherwise
            msgbox( "Cannot cope with an array of" ;
               + "more than two dimensions.",;
               "ArrayEx.ListArray", 48)
            rv = false
      endcase

      os.count = rv    // the number of rows put in the string
   return (oS)
   // end of method: StringArray()


ENDCLASS
// -- End of Class: ArrayEx


CLASS zArray(r, c) of Array
   /*
     -----------------------------------------------------------------
     Description:

       This class extends the set of methods of a normal array.
       It includes methods which simplify some (IMO) slightly clumsy
       processes for normal use of an array, especially when 2D.
       It also adds-in the methods of the dUFLP arrayex class.
       Not suitable for arrays with more than 2 dimensions (I can't 
       get my brain round that).

    Programmer:
       Bruce Beacham
 
    History: 
 
       August 26, 2001 -- Written.
       December 17, 2001 -- minor bugfix in scanCol by Bruce.
       May 5 , 2002 -- adjusted constructor code to allow the object 
            to auto-destroy,  and accordingly added the Shuffle 
            submethod.
       Nov., 2005 -- ScanCol() method updated to allow a backward
            scan of an array column.
 
    Instantiation:
 
       set procedure to :dUFLP:arrayex.cc additive
       oMyArray = new zArray()

    Methods:
      Note, more details are in the headers for each method, 
      including a listing of parameters, and what they mean.
      See also the methods in the arrayex class.
 
      AddRow()
         Adds a row to a 1D or 2D array and adds the data provided
         as (up to four) parameters to the columns.   If a 1D array is
         provided its elements are used to fill the array to the lesser
         of the number of elements given and the number of columns
         in this array.

      CopyIn()
         Makes this array a copy of the array parameterised-in.
         Thus any extra properties of this array are preserved but the
         array elements and dimensions of the copied array are adopted.

      DeleteRow()
         Deletes a row and resizes the array to remove the spare last 
         row

      Empty()
         Returns whether the array is empty, ie whether any single 
         residual row contains only elements = false.

      Grow()
         Subclasses the native grow() method to avoid the need to
         consider whether this is the first additional row in a
         2D array (which is created upon instantiation).   Returns the
         resulting number of rows or columns.
 
      InitialiseColumn()
         Sets all the elements in a specified column to a specified 
         value.

      Example:
         a = new zArray()
         a.dir()
         a.listarray()
         a.CarveOut().Upper().listarray()
         a.release()
 
     ------------------------------------------------------------------
   */ 

   do case
      case r = false or (r = 0 and c = false)
         // Array is 1D with no rows.
      case c = false
         // Array is 1D with r rows
         this.resize(r)
      otherwise
         // Array is 2D with r rows and c columns.
         // r must be >= 1 but c can be 0.   See resize() in the OLH.
         if r < 1
            r = 1
         endif
         this.resize(r, c)
   endcase

   // Extend this array to have the powers of an arrayex.
   local a
   a = new arrayex(this)                  // assigns extra methods.
   // Remove the cross-reference that prevents this zArray object 
   // from auto-destruction when all external references to it
   // are removed.
   // The mathex object is recreated in the subclassed Shuffle herein.
   a.release()



   FUNCTION AddRow
      /*
         Adds a row to a 1D or 2D array.
         d1 is the data to add to col1.   If given, d2 -d4 is the data
         to put in cols 2-4. Returns the row number (for data to be
         added to further columns).
         d1 may be an array, in which case d2, d3 and d4 are ignored.
         User is NOT warned if more or less data is provided in
         an incoming array than this array has columns.
         It returns the number of rows in the array.
      */
      parameters d1, d2, d3, d4

      local givenarray
      givenarray = type("d1") = "A"
      if givenarray and d1.dimensions <> 1
         msgbox("Given an array that is not a 1D array.   " ;
            + "Data not added to array.   ", "zArray.AddRow", 48)
         return (alen(a, 1))
      elseif givenarray and d1.size < 1
         msgbox("Given an array that has no elements.   " ;
            + "No row added to array.   ", "zArray.AddRow", 48)
         return (alen(a, 1))
      endif

      if this.dimensions = 1
         if givenarray
            this.add(d1[1])     // even if d1 contains more
                                // than 1 element.
         else
            this.add(d1)
         endif
      elseif this.dimensions = 2
         this.grow(1) // doesn't add a row if the spare row
                      // created on instantiation is still present.
         local nLastrow
         nLastrow = alen(this, 1)
         if givenarray
            local nElement
            // Put each element in the incoming data array into
            // succeeding columns on the new row in this array.
            for nElement = 1 to min(alen(d1, 1), alen(this, 2))
               this[nLastrow, nElement] = d1[nElement]
            endfor
         else
            // Put up to four bits of data in the first four columns
            // of this array.
            this[nLastrow, 1] = d1
            if alen(this, 2) < pcount()
               msgbox("Cannot assign " + pcount() + ;
                   " parameters to " + alen(this, 2) +;
                   " columns.   ", "zArray.AddRow", 48)
            else
               if pcount() > 1
                  this[nLastrow, 2] = d2
               endif
               if pcount() > 2
                  this[nLastrow, 3] = d3
               endif
               if pcount() > 3
                  this[nLastrow, 4] = d4
               endif
            endif
         endif
      else
         msgbox("Cannot cope with an array of more than two dimensions.   ",;
                "zArray.AddRow", 48)
      endif

   return (alen(this, 1))

   FUNCTION DeleteRow(n)
      /*
         Deletes a single row and resizes the array.
         n is the row number to delete.
      */
      local len
      len = alen(this, 1)
      activerows = len - 1
      if n > len
         msgbox("Cannot delete a row beyond the array boundary.",;
                "zArray.DeleteRow", 48)
         return (len)
      endif

      if this.dimensions = 1
         this.delete(n)
         if len = 1
            // Can't resize a 1D below 1 row.   But let activerows 
            // return the meaningful number.
         else
            // Delete a row.
            this.resize(activerows)
         endif
      elseif this.dimensions = 2
         this.delete(n, 1)     // the second parameter is 1 for a 
                               // row, the default, and 2 for a col.
         if len = 1
            // Can't resize a 2D below 1 row.   But let activerows 
            // return the meaningful number.
         else
            // Delete a row.
            this.resize(activerows)
         endif
      else
         msgbox("Cannot cope with an array of more than two dimensions.   ",;
                "zArray.DeleteRow", 48)
         return (alen(this, 1))
      endif

   return (activerows)

   FUNCTION grow(n)
      /*
         Subclasses the native grow() method of the array class.
         This does not add a row if this is a 2D array with an empty
         first row (which is created upon instantiation)
         Returns the number of the last row in the array.
      */
      if this.dimensions = 1
         super::grow(n)
      elseif this.dimensions = 2 and alen(this, 1) = 1 and;
         type("this[1,1]") = "L" and this[1,1] = false
         // We have a 2D array that looks as if it's freshly created.
         if n = 1
            // No action.   Assume the array is new and row 1 hasn't
            // been used yet.
         else
            super::grow(2)
         endif
      else
         super::grow(n)
      endif
   return (alen(this, n))

   FUNCTION CopyIn            //( oArray )
      /*
         Adapted from class arrayex copy() in dUFLP.
         oArray is an array of which this zArray is to be a copy.
      */
      parameter oArray

      if empty(oArray)
         msgbox("No array given as a parameter", ;
               "zArray.CopyIn", 48)
         return false
      endif
      if type("oArray") <> "A"
         msgbox("Parameter given is not an Array.   ", ;
               "zArray.CopyIn", 48)
         return false
      endif
      if alen(oArray, 1) = 0
         msgbox("Cannot resize to a zero row 1D array.   ", ;
               "zArray.CopyIn", 48)
         return false
      endif

      do case
         case oArray.dimensions = 1 // we know it has one or more rows.
            this.resize(oArray.size)
         case oArray.dimensions = 2
            this.resize(alen(oArray, 1), alen(oArray, 2))
         otherwise
            msgbox("Cannot cope with an array of more than 2 dimensions.   ",;
                   "zArray.CopyIn", 48)
            return false
      endcase
      acopy(oArray, this)
      
   return (this)

   FUNCTION Empty
      /*
         Works IMO more sensibly than empty(array())
         If an array has one row and all elements are false, then it 
         it deemed to be empty.
         This is true of a 1D as well as a 2D, because even though a 
         1D can have 0 rows at instantiation, it cannot be resized 
         down to 0 rows, so DeleteRow() has to leave the single 
         remaining row but with false in all columns.
      */
      if this.dimensions = 1 and this.size = 0
         return true
      elseif this.dimensions = 1 and alen(this, 1) = 1
         return (this[1] = false)
      elseif this.dimensions = 2 and alen(this, 1) = 1
         local s
         for s = 1 to alen(this, 2)
            if not this[1, s] = false
               return false
            endif
         endfor
         return true
      endif

   return false

   FUNCTION ScanCol
      /*
         Like array().scan, but allows specification of a column to
         scan. Parameter 2 is the column to scan, default is column 1,
         1D is always column 1. The start and limit parameters of the
         array().scan() method are not used. Works on exact match
         unless parameter 3 is given and true, in which case nCol
         may be numeric or false
         Parameter backscan is optional and is a logical, specifying
         whether to scan from the bottom of the array rather than
         from the top.  Default false (so scan from the top as normal).
      */
      parameter exp, nCol, partialMatch, backscan

      if type("nCol") <> "N"
         nCol = 1
      endif

      *

      if nCol < 1
         msgbox("Specified a sort row or column less than 1.   ",;
                "zArray.ScanCol", 48)
         return false
      elseif this.dimensions = 1
         // Column is always 1, so nCol is not used.
      elseif nCol > alen(this, 2)
         msgbox("Specified a column (" + nCol +;
                 ") beyond the array dimensions (" + ;
                 alen(this, 2) + " cols).   ", ;
                 "zArray.ScanCol", 48)
         return false
      endif

      // Added 6 Nov 2005 by Bruce Beacham.
      local scanstep, scanstartrow, scanendrow
      if backscan
         scanstep = -1
         scanstartrow = alen(this, 1)
         scanendrow = 1
      else
         scanstep = 1
         scanstartrow = 1
         scanendrow = alen(this, 1)
      endif

      private exactness ; exactness = set("exact")
      set exact off
      local s, found
      found = 0
      do case
         case this.dimensions = 1 and partialMatch
            for s = scanstartrow to scanendrow step scanstep
               if this[s] = exp
                  found = s
                  exit
               endif
            endfor
         case this.dimensions = 1
            for s = scanstartrow to scanendrow step scanstep
               if this[s] == exp
                  found = s
                  exit
               endif
            endfor
         case this.dimensions = 2 and partialMatch
            for s = scanstartrow to scanendrow step scanstep
               if this[s, nCol] = exp
                  found = s
                  exit
               endif
            endfor
         case this.dimensions = 2
            for s = scanstartrow to scanendrow step scanstep
               if this[s, nCol] == exp
                  found = s
                  exit
               endif
            endfor
      endcase
      set exact &exactness.

   return (found)

   FUNCTION InitialiseColumn
      /*
        Sets all the elements in one column of an existing sized 
        array to a single value. Might be used to set default values 
        (eg 0 or "") in a new column. Parameter nCol is the column 
        number, iVal is the value to be set. Parameter nCol may 
        be omitted in a 1D array.
      */

      parameter nCol, iVal
      if this.dimensions <> 1 and type("nCol") <> "N"
         msgbox("Not given a numeric value for a 2D array.",;
                "zArray.InitialiseColumn", 48)
         return false
      endif
      if this.dimensions <> 1 and pcount() < 2
         msgbox("Not given two parameters for a 2D array.",;
                "zArray.InitialiseColumn", 48)
         return false
      endif
      if this.dimensions = 1 and pcount() > 1
         msgbox("Given two parameters for a 1D array." ;
            + "Ignoring second parameter.   ",;
             "zArray.InitialiseColumn", 48)
      endif

      local s
      if this.dimensions = 1
         for s = 1 to this.size
            this[s] = iVal
         endfor
      elseif this.dimensions = 2
         for s = 1 to alen(this, 1)
            this[s, nCol] = iVal
         endfor
      else
         msgbox("Cannot cope with an array of more than 2 dimensions.",;
                "zArray.InitialiseColumn", 48)
         return false
      endif

   return true

   FUNCTION Shuffle( aName )
      /* Supersedes the arrayex shuffle method which would otherwise 
      be assigned in arrayex::AssignMethods().   The reason is that 
      the arrayex Shuffle method expects the math object to be present,
      (see the constructor code of the arrayex class)
      but it has been deleted in the zArray to avoid circularity.   
      Here it is re-made, and then destroyed.
      */
      this.math = new mathex()
      local s
      s = arrayex::Shuffle( aName )       // returns a zArray()
      this.math.release()
      this.math = null
      this.a1 = null          // circular reference to this zArray 
                              // object.
      return (s)
      /* If called as   
      a = new zArray()
      a.add("Fred")
      a.add("Janet")
      a.add("Jim")
      //Shuffle this object, not a 3rd party array.
      s = a.Shuffle()
      - then
      ? (s = a)       // returns true.
      */

ENDCLASS				
// End of class: zArray().				

/* -----------------------------------------------------------------------
   End of File: ARRAYEX.CC
   ----------------------------------------------------------------------- */

