// Metric: 0 - Chars or 6 - Pixels only
/*
    DragListContainer.cc
    This is a Listbox that allows you to drag objects up and
    down inside the listbox. It is based heavily on code
    by David Peters. Created by Ken Mayer.

    Modified by Bryan Toporowski, January, 2004
       ** Added code to deal with sizing the listbox to fit
          the container, and other code that I'm still
          trying to figure out.

    It should be completely self-contained within the Container
    object. The two entryfields are "hidden", as it were, 
    and are used to deal with the scrolling up and down
    in the listbox. For more details, see the code below.

    Usage:
       Set procedure to :dUFLP:DragListContainer.cc additive
         * in the form designer, place this control onto
           the form. Move around, rearrange as needed.
           *** THE METRIC IS 0 // Chars -- if your form uses
               another metric, change the form's metric to 0,
               then drag this control to the form, then re-set
               the form's metric ...
               ** NOTE: as of July, this may work with
                  Pixels -- no 100% guarantees, but
                  testing shows it as working.

       In the form's onOpen code, or an overridden open or
       readmodal event, set the array on the control to:
           form.DragListContainer1.Items = myarrayobject
       where "myarrayobject" is an array that contains
       whatever list you want the user to be able to manipulate.

       In the form's canClose event, you should grab the 
       array: form.DragListContainer1.items and do something
       with it, possibly:
          form.Items = form.DragListContainer1.items
       because when the form closes (onClose) it's too late,
       this appears to be removed from memory ... If you get
       it in the form's canClose event, it's still there, and
       you can get a copy of the array. Then you can manipulate
       it however you need to.

      Usage:

         Set Procedure To DRAGLISTCONTAINER.cc additive
         local f
         f = new FORM()
         f.DRAGLISTCONTAINER1 = new DRAGLISTCONTAINER(f)
            with (f.DRAGLISTCONTAINER1)
               left = 2
               top = 2
               width = 22.1429
               height = 10
            endwith

         // if you want a demonstration (sample data in
         // listbox ...)
         f.DRAGLISTCONTAINER1.want_demo = true

         f.open()

     Properties:

         want_demo -- puts this into a 'demo mode' by creating
                      a sample array ... logical value (true/false)
                      defaults to false.


*/
#define HIGHLIGHTCOLOR "white/navy"
#define DRAGCOLOR      "white/gray"

class DRAGLISTCONTAINER( oParent ) of CONTAINER( oParent ) custom

   with (this)
      onOpen = class::DRAGLISTCONTAINER_ONOPEN
      // these functions arent neccesary when controls totally fill 
      // the container which is done in the onopen routine
      // I dont know why you wouldnt want this
      //  onLeftMouseUp = class::DRAGLISTCONTAINER_ONLEFTMOUSEUP
      //  onMouseMove   = class::DRAGLISTCONTAINER_ONMOUSEMOVE
      left = 1.7143
      top = 0.4091
      width = 22.1429
      height = 10
      metric = 0	// Chars
      borderStyle = 3	// None
      onDesignOpen = class::onDesignOpen
   endwith

   // items array always has to be present so put in constructor
   this.items = new ARRAY()

   // this can be changed before form open
   this.want_demo= false


   this.LISTBOXTOP = new ENTRYFIELD(this)
   with (this.LISTBOXTOP)
      when = {||false}
      *onLeftMouseUp = class::listboxtop_ONLEFTMOUSEUP
      onLeftMouseUp = class::LISTBOXends_ONLEFTMOUSEUP
      onMouseMove = class::LISTBOXTOP_ONMOUSEMOVE
      height = 1
      left = 0.4286
      top = 0.1364
      width = 8
      colorNormal = "btnface"
      border = false
      value = ""
      mousePointer = 1	// Arrow
      pageno = 0
      borderStyle = 3	// None
   endwith

   this.DRAGLISTBOX = new LISTBOX(this)
   with (this.DRAGLISTBOX)
      onLeftMouseDown = class::draglistbox_ONLEFTMOUSEDOWN
      onLeftMouseUp = class::DRAGLISTBOX_ONLEFTMOUSEUP
      onMouseMove = class::DRAGLISTBOX_ONMOUSEMOVE
      height = 9.8636
      left = 0
      top = -0.0909
      width = 21.5714
      id = 102
      pageno = 0
      colorHighlight = HIGHLIGHTCOLOR
   endwith

   this.LISTBOXBOTTOM = new ENTRYFIELD(this)
   with (this.LISTBOXBOTTOM)
      when = {||false}
      onLeftMouseUp = class::listboxends_ONLEFTMOUSEUP
      onMouseMove = class::LISTBOXBOTTOM_ONMOUSEMOVE
      height = 1
      left = 0.4286
      top = 10.2727
      width = 8
      colorNormal = "btnface"
      border = false
      border = true
      value = ""
      mousePointer = 1	// Arrow
      pageno = 0
      borderStyle = 3	// None

   endwith

   function onDesignOpen( lFromPalette )
      if form.metric # 0 and form.metric # 6
         msgbox( "This control can only be used on a form "+;
                 "with metric=0 (Char) or metric=6 (Pixel).",;
                 "Can't do it!", 16 )
         return
      endif

      // deal with form.metric == 6 // Pixels
      if lFromPalette 
         if form.metric == 6
            this.height             := 220
            this.width              := 155
            this.DragListBox.top    := -1
            this.DragListBox.left   := 0
            this.DragListBox.height := 217
            this.DragListbox.width  := 151
            this.ListBoxBottom.top  := 226
            this.ListBoxBottom.left := 3
            this.ListBoxTop.top     := 3
            this.ListBoxTop.left    := 3
         endif
      endif

   function DRAGLISTCONTAINER_onOpen
      if type("mouse_event") # "FP"
         extern CVOID mouse_event( CWORD, CWORD, CWORD, ;
                                   CWORD, CWORD ) User32
      endif

      // arrange controls to fit container. listboxtop & 
      // listboxbottom have to precede & follow listbox for 
      // mouse events
      this.listboxtop.top      := 0
      this.listboxtop.width    := this.width
      this.draglistbox.top     := 1
      this.draglistbox.height  := this.height-2
      this.draglistbox.width   := this.width-2
      this.listboxbottom.top   := this.height-1
      this.listboxbottom.width := this.width

      // Setup custom properties of the form
      this.leftDrag = false
      this.leftDragMove = false
      this.leftRow = 0

      // Create a timer to do the off-screen listbox scrolling
      this.timer1 = new TIMER()
      this.timer1.parent    = this
      this.timer1.onTimer  := class::TIMER1_ONTIMER
      this.timer1.interval := 0.15
      this.timer1.direction = "DOWN"

      if this.want_demo	
         // Put in some dummy information into the array for testing,
         // if the developer has not provided an array of data:
         if type( "this.items" ) # "A" 
            // Create array to hold the listbox items
            this.items = new ARRAY()
         endif
         for n =1 to 20
            this.items.add(STR(n,2)+"- LISTBOX ITEM -"+STR(n,2))
         endfor
      endif

      // Assign our array to the listbox and lets get cooking
      this.DragListBox.dataSource = 'ARRAY this.items'
   return

   function listboxends_onLeftMouseUp(flags, col, row)
      // If dragging then better tell the listbox that the 
      // left button is not down anymore
      if this.parent.leftDrag
         class::dragListBox_onLeftMouseUp(-1)
      endif
   return

   function listboxtop_onLeftMouseUp(flags, col, row)
      // If dragging then better tell the listbox that the 
      // left button is not down anymore
      if this.parent.leftDrag
         class::DragListBox_onLeftMouseUp(-1)
      endif
   return

   function LISTBOXBOTTOM_onMouseMove(flags, col, row)
      if this.parent.leftDrag and bitSet(flags, 0)
         // If we are below the listbox and dragging then start scrolling
         // down if we can
         if ( not this.parent.timer1.enabled and ;
              this.parent.DragListbox.curSel < this.parent.items.size )
            this.parent.timer1.direction := "DOWN"
            this.parent.timer1.enabled   := true
         endif
      endif      
   return

   function DRAGLISTBOX_onLeftMouseDown(flags, col, row)
      // Take note of the item being dragged
      this.parent.leftSel = this.curSel

      // Put the container & listbox into dragging mode
      with (this.parent)
         leftDrag = true
         DragListBox.colorhighlight := DRAGCOLOR
         DragListBox.mousePointer   := 7
         DragListBox.mousePointer   := 7
         listboxBottom.mousePointer := 7
         mousePointer               := 7
      endwith

   return

   function DRAGLISTBOX_onLeftMouseUp(flags, col, row)
	*this comment no longer applies:
      // Note dont refer to 'this' as function gets called by form when
      // 'this' is is out of scope.  Use form.listbox1 instead.

      // Turn off dragging
      this.parent.leftDrag       = false
      this.parent.timer1.enabled := false
      this.colorhighlight        := HIGHLIGHTCOLOR

      // If listbox item has been dragged to a new location then resort
      // the array and repaint it
      if this.parent.leftSel # this.parent.draglistbox.curSel
         temp = this.parent.items[this.parent.leftSel]
         this.parent.items.delete(this.parent.leftSel)
         this.parent.items.insert(this.parent.draglistbox.curSel)
         this.parent.items[this.parent.draglistbox.curSel] = temp
         this.parent.draglistbox.dataSource := "array this.parent.items"
      endif

      // Put the mouse pointers back to normal
      with (this.parent)
         DragListBox.mousePointer   := 1
         listboxTop.mousePointer    := 1
         listboxBottom.mousePointer := 1
         mousePointer               := 1
      endwith
   return

   function DRAGLISTBOX_onMouseMove(flags, col, row)
      // Are we dragging
      if this.parent.leftDrag
         // If left button is not depressed then stop dragging
         if not bitset(flags, 0)
            class::DragListBox_onLeftMouseUp(-1)
         else
            // Turn off the timer for scrolling if we are in the listbox
            // as it is not needed here.
            if this.parent.timer1.enabled
               this.parent.timer1.enabled := false
            endif

            // The mouse has moved up or down about a row then press the
            // right mouse button on our current location to select a new
            // item in the listbox.

            // The 0.78 is a bit of a magic figure, it is about the height
            // of one listbox row in Arial 10 font.  Why it isn't 1 I dont
            // know!  If you are using different fonts/sizes you may need
            // to fine tune this figure.  Its main purpose it to only press
            // the right button when the mouse has moved about a row rather
            // than doing it all the time for every pixel of movement.
            if INT(row/0.78)+1 # this.curSel
               mouse_event(8,0,0,0,0) // right button down
               mouse_event(16,0,0,0,0) // right button up
            endif
         endif
      endif
   return


   function LISTBOXTOP_onMouseMove(flags, col, row)
      if this.parent.leftDrag and bitSet(flags, 0)
         // If we are above the listbox and dragging then 
         // start scrolling up if we can
         if not this.parent.timer1.enabled and ;
            this.parent.DragListBox.curSel > 1
            this.parent.timer1.direction := "UP"
            this.parent.timer1.enabled   := true
         endif
      endif
   return

   function TIMER1_onTimer
      // Lets be really pesimistic, if anything does not look right then
      // turn the timer off
      TRY
         do case
         case ( this.parent.parent.activeControl.name == "DRAGLISTBOX" ;
                and this.direction == "UP" )
            // Move the listbox up if we can
            if this.parent.draglistbox.curSel > 1
               this.parent.draglistbox.curSel--
            else
               this.enabled := false
            endif

         case ( this.parent.parent.activeControl.name == "DRAGLISTBOX" ;
                and this.direction == "DOWN" )
            // Move the listbox down one if we can
            if this.parent.draglistbox.curSel < this.parent.items.size
               // NOTE: I was using this.parent.listbox.curSel++ here but it
               // had the unfortunate habit of making the listbox 'jump' down
               // and entire page.  I now just press the down arrow.  Ugly,
               // but it works, and looks much nicer.
               KEYB CHR(24)
            else
               this.enabled := false
            endif

         other
            this.enabled := false
         endcase
      CATCH (Exception e)
         this.enabled := false
      ENDTRY
   return

endclass

