// -- Metric: Not metric specific ...

////////////////////////////////////////////////////////////////////////////////
// Filename:      FlashImage.CC                                               //
// Written For:   dBASE 32-bit versions.                                      //
// Written By:    Jim Sare - jimsare@royaloak.net                             //
// Usage:         See FlashImage.WFM for an example of usage.                 //
////////////////////////////////////////////////////////////////////////////////
/*
    More information by Ken Mayer for use:

    set procedure to :dUFLP:FlashImage.cc additive
     ** in the form designer drag an instance of this
        control to your form
       Using the inspector:
       * Set the dataSource property to the image you
         wish to display by default (starting image)
       * Click on the events tab of the inspector and
         select the control's onOpen event, click on the
         tool (wrench) button.

         Enter code like (without the // in front of the PROCEDURE
         statement) 
            // PROCEDURE FLASHIMAGE1_OnOpen
               this.OnMouseOver := CLASS::FLASHIMAGE1_OnMouseOver
               this.OnMouseOut := CLASS::FLASHIMAGE1_OnMouseOut
         Then add the following (again without the // in
         front of the PROCEDURE statements) -- the reason you
         are adding these is that these are not normally
         methods of the image control, they are defined in
         this custom class:
            // PROCEDURE FLASHIMAGE1_OnMouseOver
              this.dataSource := "FILENAME :dQuery:\images\ButtonWinAppHi.jpg"

            // PROCEDURE FLASHIMAGE1_OnMouseOut
              this.dataSource := "FILENAME :dQuery:\images\ButtonWinApp.jpg"

       * Next go back to the inspector, click on the tool
         button for the onLeftMouseDown event, and enter code
         like:
             this.dataSource := "FILENAME :dQuery:\images\ButtonWinappPressed.jpg"
       * Finally, back on the inspector, click on the tool
         button for the onLeftMouseUp event, and enter
         code like:
             this.dataSource := "FILENAME :dQuery:\images\ButtonWinappHi.jpg"
       * Add whatever code in the onLeftMouseUp event that you
         want to execute when the 'button' is 'clicked'.

    *** NOTES:
        There should be three different images:
          "Up" image
          "Down" image
          "Mouse Over" image
        When deploying, make sure that you deploy the images
        appropriately, either in the same folder as the .exe,
        or in DEO specified folders.
*/
// Versioning and History Comments:
#define FI_VERSION "1000"  // 03/07/2003 = Original version.

CLASS FlashImage(o, n) of Image(o, n) custom

   // Declare Windows API functions needed.
   if type("GetCapture") # "FP"
      extern CHANDLE GetCapture() User32
   endIf
   if type("GetClientRect") # "FP"
      extern CLOGICAL GetClientRect(CHANDLE, CPTR) User32
   endIf
   if type("GetCursorPos") # "FP"
      extern CLOGICAL GetCursorPos(CPTR) User32
   endIf
   if type("ReleaseCapture") # "FP"
      extern CLOGICAL ReleaseCapture() User32
   endIf
   if type("ScreenToClient") # "FP"
      extern CLOGICAL ScreenToClient(CHANDLE, CPTR) User32
   endIf
   if type("SetCapture") # "FP"
      extern CHANDLE SetCapture(CHANDLE) User32
   endIf

   // Standard property members.
   this.alignment :=          4        // True Size
   this.borderStyle :=        3        // None

   // Custom property members.
   this.version =             FI_VERSION
   this.onMouseOver =         null     // Function to call when mouse pointer enters image.
   this.onMouseOut =          null     // Function to call when mouse pointer leaves image.

   // Protected custom property members.
   this.xPOINT =              replicate(chr(0), 4)
   this.PPUX =                0        // Pixels Per Unit horz
   this.PPUY =                0        // Pixels Per Unit vert

   PROTECT xPOINT, PPUX, PPUY

   PROCEDURE OnMouseMove(nFlags, nX, nY)
      LOCAL xRECT, n, nXPix, nYPix

      GetCursorPos(this.xPOINT)               // Get mouse pointer position in screen.
      ScreenToClient(this.hWND, this.xPOINT)  // Convert relative to image position.
      nX = int(this.xPOINT.getByte(0) +;      // Extract the horz position.
           bitLShift(this.xPOINT.getByte(1), 8) +;
           bitLShift(this.xPOINT.getByte(2), 16) +;
           bitLShift(this.xPOINT.getByte(3), 24))
      if bitSet(nX, 31)                       // If it's a negative value.
         nX := -bitXOr(hToI("FFFFFFFF"), nX)
      endIf
      nY = int(this.xPOINT.getByte(4) +;      // Extract the vert position.
           bitLShift(this.xPOINT.getByte(5), 8) +;
           bitLShift(this.xPOINT.getByte(6), 16) +;
           bitLShift(this.xPOINT.getByte(7), 24))
      if bitSet(nY, 31)                       // If it's a negative value.
         nY := -bitXOr(hToI("FFFFFFFF"), nY)
      endIf
      if this.PPUX = 0                        // Only need to do this piece once.
         xRECT = space(8)
         GetClientRect(this.hWND, xRECT)      // Get image client size in pixels.
         n = int(xRECT.getByte(8) +;          // Extract the width.
             bitLShift(xRECT.getByte(9), 8) +;
             bitLShift(xRECT.getByte(10), 16) +;
             bitLShift(xRECT.getByte(11), 24))
         this.PPUX := n / this.width          // Compute and save horz pixels/form unit.
         n = int(xRECT.getByte(12) +;         // Extract the height.
             bitLShift(xRECT.getByte(13), 8) +;
             bitLShift(xRECT.getByte(14), 16) +;
             bitLShift(xRECT.getByte(15), 24))
         this.PPUY := n / this.height         // Compute and save vert pixels/form unit.
      endIf
      nX /= this.PPUX                         // Convert horz pixels to form units.
      nY /= this.PPUY                         // Convert vert pixels to form units.
      // If the mouse pointer is within the image area.
      if (nX >= 0) and (nX < this.width) and (nY >= 0) and (nY < this.height)
         if GetCapture() = this.hWND          // If capture is alread set, get out.
            RETURN
         endIf
         SetCapture(this.hWND)                // Send all mouse events to the image.
         // The mouse pointer has entered the image area for the first time.
         if (type("this.onMouseOver") $ "CB|FP") and (not empty(this.onMouseOver))
            this.onMouseOver(nFlags, nX, nY)  // Call the onMouseOver handler.
         endIf
      else                                    // Mouse pointer has moved out of image area.
         ReleaseCapture()                     // Stop capturing mouse events.
         if (type("this.onMouseOut") $ "CB|FP") and (not empty(this.onMouseOut))
            this.onMouseOut(nFlags, nX, nY)   // Call the onMouseOut handler.
         endIf
      endIf

ENDCLASS  // FlashImage of Image

