/*
   This program contains code that you may wish to attach to a custom
   grid. It is not independant code, and should not be run outside 
   of a custom grid class. It is suggested that you create your own
   grid class that adds any of these events, which you can then 
   call in whatever way you feel a need to ...

   GridSelectAll() -- used to select all rows in a grid, only useful
                      if the grid's multiSelect property is set to
                      true.

   Basically, copy the required function into a custom grid's
   constructor code (before the ENDCLASS statement) ...
*/

msgbox( "This program is not meant to be executed, "+;
        "but contains useful API calls and other functions.","Don't run this!", 16 )
return


   function GridSelectAll
      /*
          GridSelectAll()
          Marc Van den Berghen

          This can be used, perhaps with a pushbutton on a form, in the
          onClick event -- you need to pass the grid object reference to
          this method:

               function SelectAllBtn_OnClick
                  class::GridSelectAll( form.MyGrid )
               return

          Or if you set the onLeftDblClick event handler for the grid:

              function GRID1_onLeftDblClick(flags, col, row)
                 class::GridSelectAll( this ) // "this" is the grid!
              return
             
          Modified January, 2008, to include some error handling,
          and to allow for use with more than one grid on a form,
          with different rowsets dataLinked to it ...
      */
      parameter oGrid
      if type( "oGrid" ) # "O" // object
         msgbox( "Please pass an object reference for"+chr(13)+;
                 "the grid you wish to select all rows in.",;
                 "Programmer Error!", 16 )
         return
      endif

      local keymap,tmp
      // The following externalization can be done in the onopen of the form
      #define WM_LBUTTONDOWN 0x0201
      #define MK_SHIFT 0x0004
      #define VK_SHIFT (0x10)
      extern cint DbiGetPriorRecord(chandle,cint,cint,cint) idapi32
      extern cint DbiSetToEnd(chandle) idapi32
      extern cint DbiSetToBegin(chandle) idapi32
      extern cuint SendMessage(chandle,cuint,clong,clong) user32 ;
         from "SendMessageA"
      extern clogical GetKeyboardState(cptr) user32
      extern clogical SetKeyboardState(cptr) user32
      //
      keymap=space(128)

      // do we have a rowset datalinked to this grid?
      try
         oGrid.dataLink.first()
      catch( exception e )
         msgbox( "Unexpected Error:"+chr(13)+chr(13)+;
                 "Error:   "+e.code+" - "+e.message+chr(13)+;
                 "Program: "+e.filename+chr(13)+;
                 "Line:    "+e.lineNo+chr(13)+chr(13)+;
                 "The chances are that the grid  '"+oGrid.name+"'"+chr(13)+;
                 "does not have a rowset dataLinked to it.",;
                 "Programmer Error", 16 )
         return
      endtry
      // is the grid's multiSelect set to true?
      if oGrid.multiSelect # true
         msgbox( "Unexpected Error:"+chr(13)+chr(13)+;
                 "Grid  '"+oGrid.name+"' does not have the"+chr(13)+;
                 "multiSelect property set to true.",;
                 "Programmer Error", 16 )
         return
      endif

      if dbisettoend(oGrid.dataLink.handle)=0  // 0 if no error, might want to check
         dbierror=dbigetpriorrecord(oGrid.dataLink.handle,0,0,0)
         callerror=getkeyboardstate(keymap)
         tmp=keymap.getbyte(vk_shift)
         keymap.setbyte(vk_shift,128)
         setkeyboardstate(keymap)
         // grid-reference passed as parameter, i.e. form.grid1
         sendmessage(oGrid.hwnd,wm_lbuttondown,mk_shift,0x220018)
         keymap.setbyte(vk_shift,tmp)
         setkeyboardstate(keymap)
         dbisettobegin(oGrid.dataLink.handle)
      endif
      // Note: the 0x220018 in the sendmessage call represent the coordinates
      // where you fake the mouseclick. If you have no headings, that should be
      // changed to x0E0018 
   return

