/*
   ----------------------------------------------------------------
   Program...: QRYFOX.CC
   Programmer: Philippe Constant
   Date......: 7/16/1998 (date posted on VdBASE Newsgroups)

   This was created by Philippe because of the problems he 
   was having with FoxPro table multiple indexes (the FoxPro 
   version of the .MDX, the .CDX) -- some of the rowset methods 
   in Visual dBASE 7 just don't quite work properly. So, he 
   did a lot of work with the BDE API, and came up with this. 

   NOTE: these problems appear to have disappeared with the
   BDE 5.x versions ...

   Usage:
      set procedure to qryfox.cc additive
      q = new qryfox()
      q.sql = ...
      q.active = true

   Methods (assigned to the _rowset_):
      SetRange()
      FindKey()
      CanGetRow()

   DEPENDENCIES:
      BDE4API.H -- this should be installed in the SAMPLES
      directory/folder of dB2K. If you need to use
      this routine, you may want to copy the BDE4API.H file
      to the dB2K 01\INCLUDE folder.
   ----------------------------------------------------------------
*/

#include "bde4api.h"

*#define DEBUG

CLASS QRYFOX of QUERY Custom


   /* ---------------------------------------------------------------

      Execute method -- overriden method to ensure that the necessary
      BDE API calls are made.

      ---------------------------------------------------------------- */

   PROCEDURE execute

      super::execute()

      IF ( TYPE("DbiSetRange") # "FP" )
         extern DBIResult DbiSetRange (hDBICur, BOOL, UINT16, ;
          UINT16, pBYTE, BOOL, UINT16, UINT16, pBYTE, BOOL) "idapi32.dll"
         extern DBIResult DbiGetRecordForKey (hDBICur, BOOL, UINT16, ;
          UINT16, pBYTE, pBYTE) "idapi32.dll"
        extern DBIResult DbiGetErrorString  (hDBICur, pBYTE) "idapi32.dll"
      ENDIF

      this.Rowset.SetRange  := Class::SetRange
      this.Rowset.FindKey   := Class::FindKey
      this.Rowset.CanGetRow := Class::CanGetRow     
                              // This is for the GRID refresh BUG

   RETURN


   /* ---------------------------------------------------------------
      Method: CanGetRow() -- see onLine help for details
      This was created specifically for some sort of bug in the grid's
      refresh ...?
      ---------------------------------------------------------------- */

      FUNCTION CanGetRow

         #ifdef DEBUG
            ? PROGRAM()
         #endif

      RETURN( true )


   /* ---------------------------------------------------------------
      Method: SetRange() -- see onLine help for details
      ---------------------------------------------------------------- */

      FUNCTION SetRange(pDep, pFin)

         LOCAL mErr
         LOCAL iBuf
         LOCAL aBufDep
         LOCAL mBufDep
         LOCAL aBufFin
         LOCAL mBufFin

         IF ( PCOUNT() <= 1 )
            pFin = pDep
         ENDIF

         aBufDep = LEN(pDep)
         mBufDep = SPACE(aBufDep)

         aBufFin = LEN(pFin)
         mBufFin = SPACE(aBufDep)

         FOR iBuf = 0 TO aBufDep - 1
            mBufDep.SetByte(iBuf, ASC(pDep.CharAt(iBuf)))
         NEXT

         FOR iBuf = 0 TO aBufFin - 1
            mBufFin.SetByte(iBuf, ASC(pFin.CharAt(iBuf)))
         NEXT

         mErr = DbiSetRange(This.Handle, TRUE, 0, aBufDep, ;
                            mBufDep, TRUE, 0, aBufFin, mBufFin, TRUE)

         IF ( mErr == DBIERR_NONE )
            This.First()
         ENDIF

         #ifdef DEBUG
            ? PROGRAM(), "|", pDep, "|", pFin, "|", mErr, "|", ;
                        Class::BdeErr(mErr)
         #endif

      RETURN(mErr == DBIERR_NONE)


   /* ---------------------------------------------------------------
      Method: FindKey() -- see onLine help for details
      ---------------------------------------------------------------- */

      FUNCTION FindKey(pCle)

         LOCAL mErr

         LOCAL iBuf
         LOCAL aBuf
         LOCAL mBuf

         aBuf = LEN(pCle)
         mBuf = SPACE(aBuf)

         FOR iBuf = 0 TO aBuf - 1
            mBuf.SetByte(iBuf, ASC(pCle.CharAt(iBuf)))
         NEXT

         mErr = DbiGetRecordForKey(This.Handle, TRUE, 0, aBuf, ;
                                   mBuf, BDENULL)

         #ifdef DEBUG
            ? PROGRAM(), "|", pCle, "|", mErr, "|", Class::BdeErr(mErr)
         #endif

	   RETURN(mErr == DBIERR_NONE)


   /* ---------------------------------------------------------------
      Method: BDEErr() (used for debugging)
      ---------------------------------------------------------------- */

      FUNCTION BdeErr(pErr)

         LOCAL mMsg

         LOCAL iBuf
         LOCAL mBuf

         mBuf = SPACE(DBIMAXMSGLEN+1)

         DbiGetErrorString(pErr, mBuf)

         mMsg = ""
         FOR iBuf = 0 TO LEN(mBuf) - 1
            IF ( CHR(mBuf.GetByte(iBuf)) == 0 )
               EXIT
            ENDIF
            mMsg += CHR(mBuf.GetByte(iBuf))
         NEXT
      RETURN(mMsg)

ENDCLASS
/*
   End of file: qryfox.cc
*/

