/*
   ----------------------------------------------------------------
   Description:

      This class is derived from SendMail.cc and extends its
      function Send().
   
      While SendMail.cc uses Outlook Express this class first tries
      to send the email with Outlook. If this attempt leads to
      an error, then Outlook Express is used.
   
   
   Programmers:
   
      Adapted by Bernd Hohenester from some code snippets in the 
      newsgroups.
   
   
   History:
   
      10 Oct 2005  initial release
   
   
   Usage:
   
      set procedure to SendMaEx.cc additive
      EmailMsg = new EmailEx()
      EmailMsg.Dest = "username@domain.ext"
      EmailMsg.ccDest = ""
      EmailMsg.bccDest = ""
      EmailMsg.Subject = "Title / Subject"
      EmailMsg.Message = "Enter your message here..."
      EmailMsg.Attachments.add("FileName1.ext")
      EmailMsg.Attachments.add("FileName2.ext")
      EmailMsg.Attachments.add("FileName3.ext")
      EmailMsg.Send(bSendWithOutlookExpress)
      close procedure SendMaEx.cc
   
   
      If you are using SendMail.cc you can easily swap to SendMailEx.
      Change the two lines
      
         set procedure to :dUFLP:SendMail.cc additive
         EmailMsg = new Email()
      
      to
      
         set procedure to :dUFLP:SendMailEx.cc additive
         EmailMsg = new EmailEx()
      
      
   Properties:
      
      As the class is derived from SendMail.cc it uses the same 
      properties. Please look in the header of SendMail.cc for 
      a description.
      
      Its function Send() uses a new parameter:
      
      bSendWithOutlookExpress - if true forces the class to use 
                  Outlook Express instead of Outlook. If false 
                  or none the class first tries to send the email 
                  with Outlook.
*/
      
      
class EmailEx of Email custom from "sendmail.cc"

   this.recipients = new array()


   function Send(bSendWithOutlookExpress)

      // forced to send the email with OE
      if bSendWithOutlookExpress
         return super::send()
      endif

      local oOutlook, oItem, cTemp, i, ch

      try
         oOutlook = new OleAutoClient("Outlook.Application")
         oItem = oOutlook.CreateItem(0)
      
         // convert recipients from string to an array
         this.recipients = new array()
         cTemp = ""
         for i = 1 to len(this.Dest)
            ch = substr(this.Dest, i, 1)
            if not ch $ ";,"
               cTemp += ch
            else
               if len(ltrim(rtrim(cTemp))) > 0
                  this.recipients.add(ltrim(rtrim(cTemp)))
                  cTemp = ""
               endif
            endif
         endfor
         if len(ltrim(rtrim(cTemp))) > 0
            this.recipients.add(ltrim(rtrim(cTemp)))
         endif

         // add recipients to email
         for i = 1 to this.recipients.size
            oItem.recipients.add(this.recipients[i])
         endfor
         this.AutoSend = oItem.recipients.ResolveAll

         // add subject
         oItem.subject = this.Subject

         // add message
         oItem.body = this.Message

         // add ccDest
         oItem.cc = this.ccDest

         // add bccDest
         oItem.bcc = this.bccDest

         // add attachments
         for i = 1 to this.Attachments.size
            oItem.attachments.add(this.Attachments[i])
         endfor

         if this.AutoSend
            // send the email
            oItem.Send()
            if type("oOutlook.Assistant") <> "O"
               oOutlook.quit()
            endif
         else
            oItem.Display()
         endif

         // housekeeping
         release oItem
         release oOutlook

      catch(exception e)

         // call Send() from SendMail.cc
         super::send()

      endtry

   return

endclass
// End of class: SendMailEx.cc
