/*

	TaskBarTrayTest.prg

   This program file will demonstrate the TaskBarTray.cc class.

   Version:		0.0.1.0
   Date:			26-Jan-2008
   Author:		Andrew Shimmin

   Version:		0.0.1.1
   Date:			27-Jan-2008
   Author:		Andrew Shimmin

   Modified:	Added TestTaskBarTrayExit() function to eliminate a race condition
               on application exit and release the form and timer objects.	

   Version:		0.0.1.2
   Date:			27-Jan-2008
   Author:		Andrew Shimmin

   Modified:	Added application start and end command pane logging entry.	

   Version:		0.0.1.3
   Date:			28-Jan-2008
   Author:		Andrew Shimmin

   Modified:	Added support for the dUFLP and changed the program and popup
   				file names and some other cosmetic stuff.	

*/

   TaskBarTrayTestStart()																		//	start it up
   return

   function TaskBarTrayTestStart()
   //
   //	Function to start the application.
   //

      clear

      if	type("TaskBarTray::IamLoaded") # "FP"											//	class not loaded?	
         try
      		set procedure to :dUFLP:TaskBarTray.cc additive                   //	try dUFLP
         catch (exception e)
         	try
            	set procedure to TaskBarTray.cc additive                       //	try current folder
            catch (exception e)
            	msgbox("Cannot locate TaskBarTray.cc",program()+" - initialise",16)
               return
            endtry
         endtry
      endif

   	_app.allowYieldOnMsg := true															//	allow windows messages to be processed

      _app.cStatus = "active"																	//	application status property
      _app.lIde = .not.("runtime"$lower(version(0)))									//	set application runtime environment property
      _app.lPopUpActive = false																//	set application popUp active property

      _app.nTimerInterval = 0.25																	//	timer interval in seconds
                                                                                                	
      _app.nIntervalsPer_1_second = 4															//	timer intervals/second	
      _app.nIntervalsPer_15_second = 15*_app.nIntervalsPer_1_second					//	timer intervals/15 seconds

      _app.n_1_secondCounter = _app.nIntervalsPer_1_second*_app.nTimerInterval	//	timer 1 second counter
      _app.n_15_secondCounter = _app.nIntervalsPer_15_second*_app.nTimerInterval	//	timer 15 second counter

      _app.nElapsedSeconds = 0                                              		//	timer elapsed seconds

      TaskBarTrayIconOpen(_dbWinhome+"\Samples\fish.ico")

      _app.oTimer = new timer()
      _app.oTimer.interval := _app.nTimerInterval										//	start it
      _app.oTimer.onTimer := TaskBarTrayTimer											//	set timer handler
      _app.oTimer.enabled := true															//	start application heartbeat

      _app.lCommandPaneLogging = _app.lIde												//	set command pane logging property
      	
		TaskBarTrayLog("TaskBarTrayTest - started","after")

   	return

   function TaskBarTrayTestExit()
   //
   //	Function to exit the application.
   //

   	_app.oTbForm := null
      release object _app.oTimer
      _app.oTimer := null

		TaskBarTrayLog("TaskBarTrayTest - ended","before")

      return

	function TaskBarTrayTimer()
   //
   //	Function to provide application suspension and heartbeat.
   //
   //	This timer is needed as there is no visual form that inherently suspends
   //	the application execution until the user interacts with a visual control.
   //

   	this.enabled := false																	//	disable timer
      this.onTimer := null

      _app.n_1_secondCounter := _app.n_1_secondCounter-_app.nTimerInterval		//	count down 1 second timer intervals
      _app.n_15_SecondCounter := _app.n_15_SecondCounter-_app.nTimerInterval	//	count down 15 second timer interval


      _app.executeMessages()																	//	allow window messages waiting to execute

      do	case
      	case	_app.cStatus == "exit"														//	exit

               _app.oTbForm.oTbTray.deleteIcon()                              // delete system tray icon
               _app.oTbForm.oTbTray := null												//	stubout TaskBarTray object reference
               _app.oTbForm.oTbPopUp := null												//	stubout popUp object reference

               if	type("_app.oTbForm") == "O"											//	have a parent form?
               	_app.oTbForm.release()                                      //	close TaskBarTray object's parent form
               endif

               return TaskBarTrayTestExit()                                   //	do final exit

         otherwise

               //	check 1 second timer

               if	_app.n_1_secondCounter <= 0                       										//	1 second up?

                  _app.nElapsedSeconds++                                 								//	count it

                  TaskBarTrayLog("active - "+ltrim(str(_app.nElapsedSeconds))+" seconds")

                  _app.oTbForm.oTbTray.ToolTip := "TaskBarTrayTest - "+ltrim(str(_app.nElapsedSeconds))+" seconds"
                  _app.oTbForm.oTbTray.modifyIcon()                           						//	update TaskBarTray icon toolTip
                  _app.n_1_secondCounter = _app.nTimerInterval*_app.nIntervalsPer_1_second		//	reset 1 second interval counter

               endif

               //	check 15 second timer

               if	_app.n_15_secondCounter <= 0                   											//	15 seconds up?

                  TaskBarTrayLog("elapsed - 15 seconds timer","both")
               	_app.n_15_secondCounter := _app.nTimerInterval*_app.nIntervalsPer_15_second	//	reset 15 second interval counter

               endif

               //	enable timer

               _app.oTimer.onTimer := TaskBarTrayTimer								//	reset timer handler
               this.enabled := true                                           //	resume heartbeat

      endcase

   	return																						//	go again

   function TaskBarTrayIconOpen(cIcon)
   //
   //	Function to initialise the TaskBarTray object.
   //

      _app.oTbForm = new form()

      _app.oTbForm.metric := 6                                                //	must be pixels
      _app.oTbForm.top := -10000																//	hide the TaskBarTray parent form offscreen
      _app.oTbForm.left := -10000

      _app.oTbForm.escExit := false
      _app.oTbForm.mdi := false
      _app.oTbForm.showTaskBarButton := false
      _app.oTbForm.onClose := TaskBarTrayFormClose										//	set form onClose() event handler

      _app.oTbForm.open()

      _app.oTbForm.oTbTray = new Taskbartray(_app.oTbForm)							//	instantiate TaskBarTrayIcon

      _app.oTbForm.oTbTray.IconFilename := cIcon                              //	set icon
      _app.oTbForm.oTbTray.ToolTip := "TaskBarTrayTest - "+ltrim(str(_app.nElapsedSeconds))+" seconds"
      _app.oTbForm.oTbTray.addIcon()                                          //	display it in system tray
      _app.oTbForm.oTbTray.attachPopUp("TaskBarTrayTest.pop","TaskBarTrayTestPopUp")

      _app.oTbForm.oTbPopUp = _app.oTbForm.oTbTray.getPopUpObjRef()           // get popUp object reference

      _app.oTbForm.oTbTray.onLeftMouseDown := TaskBarTrayIcon_onLeftMouseDown //	set mouse event handlers
      _app.oTbForm.oTbTray.onLeftMouseUp := TaskBarTrayIcon_onLeftMouseUp
      _app.oTbForm.oTbTray.onMiddleMouseDown := TaskBarTrayIcon_onMiddleMouseDown
      _app.oTbForm.oTbTray.onMiddleMouseUp := TaskBarTrayIcon_onMiddleMouseUp
      _app.oTbForm.oTbTray.onRightMouseDown := TaskBarTrayIcon_onRightMouseDown
      _app.oTbForm.oTbTray.onRightMouseUp := TaskBarTrayIcon_onRightMouseUp

      return

	function TaskBarTrayFormClose()
   //
   //	Function to close the TaskBarTray object's parent form.
   //

      do	case
      	case	_app.cStatus == "exit"														//	user exit via popUp

         		//	Insert code here specific to a POPUP (user) request to exit.
               //
               //	The ONLY locations the application status property _app.cStatus
               //	gets set to "exit" is in the TaskBarTrayTestPopUp::EXIT_onClick()
               //	method and in this function below.
               //
               //	This means the user has exited the application using the popUp
               //	and the timer has closed the TaskBarTray object's parent form.

         otherwise																				//	windows message to close

               //	Insert code here specific to a WINDOWS request to exit.
               //
               //	Given the application status property _app.cStatus # "exit"
               //	Windows must have issued a close message to the application
               //	from a Log Off, Shutdown, termination etc.
               //
               //	Set the application status property to an exit and allow the
               //	timer to complete the application exit.

      			_app.cStatus := "exit"														//	set status property to force an exit
      endcase

      return

   function TaskBarTrayLog(cLogItem,cExtraLinePosition)

   	if	_app.lCommandPaneLogging

         local	lExtraLine

         lExtraLine = (argcount() >= 2).and.(.not.empty(cExtraLinePosition))

         if	lExtraLine.and.(trim(cExtraLinePosition) == "before".or.trim(cExtraLinePosition) == "both")
            ?                                                              	//	output extra line
         endif

         ?program()+" - "+trim(cLogItem)													//	output log item

         if	lExtraLine.and.(trim(cExtraLinePosition) == "after".or.trim(cExtraLinePosition) == "both")
            ?																						//	output extra line
         endif

      endif

      return

   //
   //	TraskBarTray icon mouse vent handlers
   //

   function TaskBarTrayIcon_onLeftMouseDown(flags, col, row)
      return

   function TaskBarTrayIcon_onLeftMouseUp(flags, col, row)
      return

   function TaskBarTrayIcon_onMiddleMouseDown(flags, col, row)
      return

   function TaskBarTrayIcon_onMiddleMouseUp(flags, col, row)
      return

   function TaskBarTrayIcon_onRightMouseDown(flags, col, row)
      return

   function TaskBarTrayIcon_onRightMouseUp(flags, col, row)
      return this.displayPopup()																//	display popUp


