//----- bigMath.cc -----

/*****************************************************************
** 
** These routines allow you to add, subtract, multiply and divide
** numbers that are larger than those dBase Plus is capable of 
** handling.  You pass it strings and it returns the answer as
** a string.

	Programmers :  Romain Strief, Ivar B. Jessen, Marc Van den Berghen
   
   BigAdd(c_nr1 , c_nr2)  
   BigSub(c_nr1 , c_nr2)
   BigMul(c_nr1 , c_nr2)
   BigDiv(c_nr1 , c_nr2)
   BigPow(c_nr1 , c_nr2)
   BigMod(c_nr1 , c_nr2)  // this was added, because it is much quicker
   							  // than the BigDiv function

   History : Start ???  Romain Strief	
   			 03-02-2007	Ivar B.Jessen	publication of the functions
             09-02-2007	Marc VdB			added BigMod, BigSub now returns 
             									negative results too, added some
                                       comments	, change of BigDiv, to be
                                       compatible with the new BigSub
*/

********************************************************************
**
** Addition of 2 big numbers by adding digit by digit
**
********************************************************************

Function BigAdd(c1,c2)
	local cResult,nRest,nLen,c1,c2,n,nCal

	cResult	= ""
	nRest		= 0
	nLen		= max(len(c1),len(c2))

   *** adjust the length of both strings

	c1			= right(space(nLen)+c1,nLen)
	c2			= right(space(nLen)+c2,nLen)

   *** add digit by digit

   for n=nLen to 1 step -1
   	nCal   = val(substr(c1,n,1))+val(substr(c2,n,1))+nRest
	   cResult= right(str(nCal,2,0,"0"),1)+cResult
   	nRest  = val(left(str(nCal,2,0,"0"),1))
	endfor

   *** if we have an overflow, add 1 more digit

   if nRest > 0
	  cResult=str(nRest,1,0)+cResult
	endif

return cResult

***********************************************************************
**
** Subtraction of 2 big numbers digit by digit
**
***********************************************************************

Function BigSub(c1,c2)
	local cResult,ncarry,nLen,c1,c2,n,nCal,isnegative

   *** we have to add leading zeros, to make a meaningfull comparison

 	nLen=max(len(c1),len(c2))

   *** adjust the length of the numbers with zeros

	c1=right(replicate("0",nLen)+c1,nLen)
	c2=right(replicate("0",nLen)+c2,nLen)

   *** if we would get a negative result, switch the numbers
   *** and add the - sign add the end

   isnegative = false
   if c1<c2 
		cresult = c1
      c1 = c2
      c2 = cresult
      isnegative = true
   endif      	

	cResult	=''
	mcarry 	= .f.

   *** do the math digit by digit

	for n=nLen to 1 step -1
     c1ThisDigit = val(substr(c1,n,1)) 
     c2ThisDigit = val(substr(c2,n,1)) + iif(mcarry,1,0)

     if c1ThisDigit < c2ThisDigit
        nCal   = c1ThisDigit+10-c2ThisDigit
        Mcarry = .t.
     else   
        nCal   = c1ThisDigit-c2ThisDigit
        mcarry = false
     endif
     cResult= right(str(nCal,2,0,"0"),1)+cResult
   endfor

   *** delete the leading zeros in case the two numbers are close

   do while substr(cResult,1,1) = '0'
	   cResult = substr(cResult,2,1000)
	enddo

   *** if we have a negative Result, add the minus sign
   if isnegative
  		cResult="-"+cresult
	endif

return cResult

***********************************************************************
**
** Multiplication of 2 big numbers digit by digit
**
***********************************************************************

Function BigMul(c1,c2)
	local aResults,n1,n2,n,nR1,nR2,cMul,nRest,cResult

	aResults=new array()    

   *** First multiply the big number c2 with each digit of c1
   *** and store the results in the array aresults

	for n1=len(c1) to 1 step -1
	   cResult=""
   	nRest=0
	   nR1=val(substr(c1,n1,1)) 
	   for n2=Len(c2) to 1 step -1
      	nR2=val(substr(c2,n2,1))
	      cMul=str((nR1*nR2)+nRest,2,0,"0")
     		cResult=right(cMul,1)+cResult
	 	   nRest=val(left(cMul,1))
   	endfor 
	   if nRest>0
   	   cResult=str(nRest,1,0)+cResult
	   endif
   	aResults.add(cResult)
   endfor

   *** now add all the results together, taking into account the power
   *** of 10 of each digit in c1 (we have to use BigAdd)
	cResult=""
	for n=1 to aResults.size
  		cResult=BigAdd((cResult), aResults[n]+replicate("0",n-1))
	endfor

return cResult

***********************************************************************
**
** Division of 2 big numbers digit by digit
**
***********************************************************************

function BigDiv(numerator,denominator)
	local numerator,denominator,mon,carry,Mtemp,Moldtemp,mcount,cresult

	** This function returns the numerator divided by the denominator.
	** It does NOT return decimals.  Instead, if there is a remainder
	** it returns:   342913,44   where 342913 is the whole number and 
	** 44 is the remainder
	** As with all the other functions, they require that the passed 
	** parameters be strings.

	mOn = 2
	carry = substr(numerator,1,1)
	cresult = ''
	do while mon-1 <= len(numerator)
	   if left(bigsub((carry),(denominator)),1)=='-'
	      carry = ltrim(carry)
	      cresult += "0"
	      carry +=substr(numerator,mon,1)
	      mon++
	   else
   	   Mcount = 1
      	do while .t.
	         Mtemp = bigmul((denominator),ltrim(str(Mcount)))
   	      if left(bigsub((carry),(Mtemp)),1)=='-'
      	      cresult += tran(Mcount-1,'9')
	            exit
	         endif
   	      MoldTemp = Mtemp
      	   Mcount++
	      enddo
   	   carry = bigsub((carry),(Moldtemp))+substr(numerator,mon,1)
	      mon++
   	endif
	enddo
   do while substr(carry,1,1) = '0'
	   carry = substr(carry,2,1000)
	enddo
	if not empty(carry)
	   cresult += ","+carry
	endif

   *** delete the leading zeros 

   do while substr(cResult,1,1) = '0'
	   cResult = substr(cResult,2,1000)
	enddo


return cresult    

***********************************************************************
**
** Power of 2 numbers by consecutive multiplication of 1 big number 
** with the exponent
** 
** Programemr : Ivar B. Jessen
**
***********************************************************************

function BigPow(cNumber, cExponent)
	local i, cResult, nExponent

	cResult   = "1" 
	nExponent = val(cExponent)

	for i = 0 to nExponent - 1 
		cResult = bigmul(cResult, cNumber )
	endfor

return cResult

***********************************************************************
** 
** Function for calculation of the modulus (remainder) of one large
** number divided by another number
**
**   X = BigMod("500105170123456789131400" , "97" )
**   The first parameter is the Dividend as string and the second
**   parameter is the Divisor as string
**   The algorithm works only for divisors between 10 und 100
**   Returnvalue = modulus, or -1 in case of error
**   Programmer : Marc Van den Berghen
**   Date : 03-02-2007  
**
***********************************************************************

Function BigMod(c_nr,c_div)
 local a1,a2,res,i,pos,zw,n_div
   res = val(right(c_nr,2))
   a1  = 1
   a2  = 10
   n_div = val(c_div)
   pos  = len(c_nr)-1
   if n_div<11 or n_div>99
    return -1
   endif
   for i=1 to pos-1
      zw = mod((100-n_div)*a1,n_div)
      res=res+val(substr(c_nr,pos-i,1))*zw
      a1=a2 ; a2=zw
   endfor

return res%n_div 


// endoffile BigMath.cc




