// Metric: 0 - Chars and 6 - Pixels only
// -- File: COMBO.CC

/* 
   ----------------------------------------------------------------
    Description:
 
       This UI file is designed to give some optional comboboxes
       which can be used either 'as is' or you can even subclass 
       them.
 
    Programmers: 
 
       Ken Mayer
       Peter Rorlick
 
    History: 
 
       03/01/1999-- First version of this custom class file.

    Instantiation: 
 
       set procedure to combo.cc additive
       * create/modify a form, place an instance of one of
       * these comboboxes on the form as a normal combobox ...
 
    Classes:
       More details on each of these is contained in the commentary
       for each class ...
 
       BooleanCombobox
          Allows a user to select "Unspecified" (null), "Yes" (true)
          or "No" (false) for a boolean field ...
   ----------------------------------------------------------------
*/

/*
   BooleanComboBox class
   By Peter Rorlick

   For comboBoxes that are datalinked to boolean DBF fields.

   For user-friendliness we'll offer the user 3 choices:
     Unspecified
     Yes
     No
   and do field-morphing to translate these strings to/from
   true/false/null.

   On forms where we provide the user with a default value, we 
   could remove the "Unspecified" option.

   In fact, it would be easy to create a BooleanCombobox class that
   performs the field-morphing automatically in its OnOpen, if 
   the datalink is not empty.

   Dependencies:
      LocalType() -- included below
*/

CLASS BooleanComboBox( FormObj ) OF ComboBox( FormObj ) Custom

  with (this)
    height = 1.05
    style = 2  // dropdown list
  endwith
  this.width++  // Make it wider, to accommodate "Unspecified"

   function onDesignOpen( lFromPalette )
      if not lFromPalette
         return
      endif
      if form.metric # 0 and form.metric # 6
         msgbox( "Please use either form metric of 0 (Chars) or "+;
                 "6 (Pixels) for this control." )
         return
      endif
      if form.Metric == 6 // Pixels
         this.height := Chars2Pixels(this.height,"Y" )
//         this.width  := Chars2Pixels(this.width, "X" )
      endif
   return


  Proc OnOpen
    this.datasource := 'Array {"Unspecified","Yes","No"}'
    if not empty(this.dataLink)
      this.datalink.beforeGetValue = {|| iif(this.value==null,'Unspecified',iif(this.value,'Yes','No')) }
      this.dataLink.CanChange = {|newValue|; if localType(newValue)=='C' ; this.value:=iif(newvalue=='Unspecified',null,(newValue=='Yes')) ; endif ; return false }
    endif
  return

ENDCLASS

************************************************
Function LocalType   // Call this without the quotes to Type() a local var.
   Param gValue
Return Type('gValue')

// Convert Char dimensions to Pixel -- code by
// Rich Muller (Autotraker):
*************************************************
function Chars2Pixels(Chars,cType)
*************************************************
   local YFactor, XFactor, YInPixels, XInPixels
   YFactor = .0455 // Vertical Factor
   XFactor = .1429 // Horizontal Factor
   YInPixels = Chars / YFactor
   XInPixels = Chars / XFactor
return iif(cType="X",XInPixels,YInPixels)


/*
   ----------------------------------------------------------------
   End of File: COMBO.CC
   ----------------------------------------------------------------
*/