/*
   ----------------------------------------------------------------
    KenWebClass.cc
    Author...: Ken Mayer
    Date.....: August, 2000
               Updated: February, 2001
                        March, 2002
                          // streamTitle updated to use
                          // font size, rather than H1 tag,
                          // giving a cleaner header ...
                        May, 2003
                          Minor modification to streamFooter()
                          method, streaming different
                          image, and removing text, as not
                          necessary ...

                          Added overridden version of streamHeader()
                          to set a new custom property, which
                          is used in errorPage, errorMessage ...

    Notes....: This is a subclass of WebClass.cc as provided 
               by dBASE, Inc. in dBASE Plus.

    I have added some new functionality, but don't want to mess
    with the original class.

    This version contains the same methods as are in KenWebForm
    for streaming out HTML form controls -- the original in the
    dUFLP did not ... unlike KenWebForm which creates a static
    HTML page (one that will exist after the application is done),
    the version here uses the StdIn and StdOut ports just like the
    rest of the HTML that is streamed out ... 

    Details below ...

    New Methods:
      errorMessage    -- streams out a message on a form that
                         has already started to display
      streamHeader    -- streams out the HTML header with the 
                         TITLE tags ... also sets a custom property
                         used in a couple of other methods.
      streamTitle     -- stream out the title at the top of the page
      streamDetail    -- streams out individual lines of text and/or
                         html
      streamFormBegin -- steams out the <FORM> code with the
                         cgi definition you specify for what to do
                         with the form when it's submitted
      streamFormEnd   -- streams out the end tag for a form
      streamText      -- streams out an HTML entryfield
      streamPassword  -- streams out an HTML entryfield with a
                         password mask
      streamRadio     -- streams out an HTML radio button
      streamCheckbox  -- streams out an HTML checkbox
      streamReset     -- streams out an HTML "Reset" button
      streamSubmit    -- streams out an HTML "Submit" button
      streamTextArea  -- streams out an HTML Editor control
      streamSelectBegin -- streams out an HTML Combobox or Listbox
                           (see details in documentation for
                           this method)
      streamOption    -- stream out elements used in Select
      streamSelectEnd -- end tag for a select object
      streamHidden    -- special object used to pass values
                         in standard Name/Value pairs to
                         an application when a form is posted,
                         which do not have any UI (i.e., the
                         user cannot see/interact with them!)
      streamFile      -- stream out the HTML FILE tag, used to
                         obtain a filename, adds a browse button ...

   Overriden Methods:
      These are methods that were lifted from the WebClass.cc
      and stuffed in here and then modified.

      streamBody -- streams out the BODY tag, but with some 
                    parameters not in the original ...
      SorryPage  -- modified to use the methods defined here, and
                    passing colors for the title ...
      errorPage  -- modified to handle passing colors through for
                    the title ...

   Usage:
      To use this class:

      set procedure to kenwebclass.cc additive
      oCGI = new KenWebClass()

      // set default colors (this is pretty garish, and it's
      // just an example):
      oCGI.BackgroundColor := "black"
      oCGI.BackgroundImage := "myimage.gif"
      oCGI.TextColor       := "green"
      oCGI.LinkColor       := "red"
      oCGI.vLinkColor      := "lightblue"
      oCGI.aLinkColor      := "navy"
      oCGI.TitleBackColor  := "white"
      oCGI.TitleTextColor  := "black"
      // NOTE: these colors can also use the RGB
      // color format: "#000000" == black
      //               "#00FF00" == green
      //    etc. You need to look up the colors
      //    you want for that. I like using names,
      //    but not all colors have names ...

      // From there, use the class as you would any other ...
      // You could also subclass this class to create your
      // own, and set the colors there, or override any
      // of the classes shown here.

   NOTE: When building your executable, you will need to be
         sure to include the webclass.co as well as kenwebclass.co
         in the executable. 

   ********************
   This class uses the source alias :WebWizards: which points 
   to the "WebWizards" folder installed by dBASE. This is where
   webclass.cc resides.

   ----------------------------------------------------------------
*/
class KenWebClass of CGISession from :webwizards:webclass.cc

   // Constructor code:

   // create default color and background image settings here
   // that will affect the whole HTML document and any document
   // using this instance of the class:
   
   // NOTE: colors must be in either text format (i.e., "BLUE" )
   //       or in hexidecimal (RGB) with the "#" in front of
   //       them ...

   // These are used by the streamBody() method if
   // no values are passed:
   this.BackgroundImage = "" // image that is displayed over the whole
                        // document
   this.BackgroundColor = "" // default background color of an HTML 
                        // document
   this.TextColor       = "BLACK" // default color of text on a page
   this.LinkColor       = "" // default link color (default is blue)
   this.VLinkColor      = "" // default Visited link color (one that the
                        // user has clicked on -- default is purple)
   this.ALinkColor      = "" // default color of link that is 'active' --
                        // mouse is over it and the left mouse button
                        // is down -- default is red(??)

   // These are used by the streamTitle() method if 
   // no values are passed:
   this.TitleBackColor  = "LIGHTBLUE" // Light blue
   this.TitleTextColor  = "WHITE"     // white

   // This property is used to only stream out some tags
   // if necessary:
   this.isHeaderStreamed = false // default to false, set true 
                               // when header has been streamed out.

 
   // -----------------------------------------------------------------
   // Streaming of HTML Documents -- I am modifying some of the 
   // original code in WebClass.cc so I have to copy the code
   // itself to here. In addition, I am giving a bit more flexibility
   // in the appearance and such ... this allows you the ability
   // to customize a bit easier ...
   // -----------------------------------------------------------------
   function streamHeader( Title )
      /*
         Subclassed version of the streamHeader() method
         in webclass.cc, this version calls the code
         from there, and also sets the custom
         property HeaderStreamed to true.
      */
     
      if not this.isHeaderStreamed
         super::StreamHeader(Title)
         this.isHeaderStreamed := true
      endif

   return true

   function streamTitle( cTitle, cBackColor, cTextColor )
      /*
          streamTitle is used to stream out a title that works
          like the one in the primary webclass for the sorryPage
          and errorPage methods ... it's simply done with
          the use of a table and setting the background for
          the cell in the table. However, this allows you to
          pass a parameter for the two colors (if you wish),
          and if you wish, you can set a default color here
      */
      // if no value passed, use default ...
      cBackColor = iif( empty( cBackColor ), this.TitleBackColor, cBackColor )
      cTextColor = iif( empty( cTextColor ), this.TitleTextColor, cTextColor )

      // Title:
      if not empty( cBackColor )
         this.streamDetail([<TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0" WIDTH="100%">])
         this.streamDetail([<TR>])
         this.streamDetail([<TD BGCOLOR="]+cBackColor+[">])
      endif
      this.streamDetail([<CENTER><FONT SIZE="6" COLOR="]+cTextColor+[">]+cTitle+[</FONT></CENTER>])
      if not empty( cBackColor )
         this.streamDetail([</TD>])
         this.streamDetail([</TR>])
         this.streamDetail([</TABLE>])
      endif

      // Blank Line:
      this.streamDetail( [<P>] )
   // end of method: streamTitle      

   function streamDetail( cString )
      // cString is the string you want streamed out -- it can
      // contain HTML ...
      // NOTE: This should be called after the streamHeader()
      // and streamBody() methods are called ...
      this.fOut.puts( ""+cString )
   // end of method: streamDetail

   // -----------------------------------------------------------------
   // Overridden methods of the CGISession class in Webclass.cc
   // -----------------------------------------------------------------
   function streamBody(cBackImage, cBackColor, cTextColor, cLinkColor, cVLinkColor, cALinkColor)
      /*
         This streams out the HTML body tag used at the top
         of an HTML document ... however, note that you can 
         pass the colors that you want to display in your
         HTML document ... all settings for a page get set here.
         Note that the colors must use either standard named
         colors (e.g., "BLUE", "LIGHTBLUE", etc.) or standard
         hexidecimal colors with the "#" in front of them (e.g.,
         "#999999" which is white) 

         If the color parameters are left out, the default is a
         white page with black text, etc.

         Parameters:
            cBackImage -- background image used on the form
            cBackColor -- background color (under the image, if any)
            cTextColor -- text color -- this is important for text
                          on top of a background color ...
            cLinkColor -- default color of links
            cVLinkColor -- Viewed links (once that have been clicked on
                           by a user)
            cAlinkColor -- Active link (if the cursor is over the link
                           and the mouse button is pressed ...)
      */

      // note that you can set defaults in the constructor code:
      cBackImage  = iif(empty(cBackImage),  this.BackgroundImage, cBackImage )
      cBackColor  = iif(empty(cBackColor),  this.BackgroundColor, cBackColor )
      cTextColor  = iif(empty(cTextColor),  this.TextColor,       cTextColor )
      cLinkColor  = iif(empty(cLinkColor),  this.LinkColor,       cLinkColor )
      cVLinkColor = iif(empty(cVLinkColor), this.VLinkColor,      cVLinkColor )
      cALinkColor = iif(empty(cALinkColor), this.ALinkColor,      cALinkColor )

      cOut = [<BODY]
      if not empty( cBackImage )
         cOut += [ BACKGROUND="]+cBackImage+["]
      endif
      if not empty( cBackColor )
         cOut += [ BGCOLOR="]+cBackColor+["]
      endif
      if not empty( cTextColor )
         cOut += [ TEXT="]+cTextColor+["]
      endif
      if not empty( cLinkColor )
         cOut += [ LINK="]+cTextColor+["]
      endif
      if not empty( cVLinkColor )
         cOut += [ VLINK="]+cVLinkColor+["]
      endif
      if not empty( cALinkColor )
         cOut += [ ALINK="]+cALinkColor+["]
      endif

      cOut+= ">" // close that tag
      this.streamDetail( cOut )
   // end of method: streamBody

   // streamFooter is an overwritten method:
   function streamFooter()
      /*
         Stream a footer with 
         "Powered By dBASE" image and link to dBASE website...
      */
      this.streamDetail( "<P>" )
      this.streamDetail( "<HR>" )
      this.streamDetail( "<P>" )
      this.streamDetail( [<A HREF="http://www.dbase.com">] )
      this.streamDetail( [<IMG SRC="PoweredBydBASE.gif" border="0" align="absmiddle" alt="dBASE"></A>] )
      this.streamDetail( "" )
      this.streamDetail( "</BODY>" )
      this.streamDetail( "</HTML>" )

   // streamFormBegin streams out the form tag
   function streamFormBegin( cCGI )
       /*
          The form tag needs to know what to do when
          the user submits it, hence the "cCGI" parameter ... 
          if this is left out, the form will not actually
          do anything if you click the submit button ... it will
          work, but ...

          Your CGI/Action ought to be something like:
             "/cgi-bin/myprog.exe" 

          This adds the form URL-Encoding, which handles
          passing spaces and some characters across
          through browsers and such in a way that ensures
          that the values aren't changed in some way (the
          problem can occur with diacriticals, for example).
       */
      cOut = [<FORM METHOD="POST" ]
      if pCount() => 1
         cOut += [ACTION="]+cCGI+["]
      endif
      cOut += [ enctype="application/x-www-form-urlencoded">]

      this.streamDetail( cOut )
   // end of method: streamFormBegin

   // streamFormEnd streams out the end form tag
   function streamFormEnd
      this.streamDetail( "</FORM>" )
   // end of method: streamFormEnd

   // Form controls:
   function streamText( cName, nSize, cValue, nMaxLength )
      /*
         The 'text' control for HTML forms is
         the same as an entryfield ...
         Parameters:
           cName is the name -- if you want this to match a field,
                make sure you use *exact* case every time you 
                reference this, as the case is vital to the way 
                the web classes work (they are subclassed from 
                an Associative Array, and it is case sensitive).
           nSize is the width of the entryfield on the form --
                the size is character based ...
           cValue is the value you want to display -- could
                be the contents of the field
           nMaxLength is optional, and is the maximum amount
                of data you want to allow to be entered.
      */
      cOut = [<INPUT TYPE="TEXT" NAME="]+cName+["]
      if pCount() > 1
         cOut+= [ SIZE="]+nSize+["]
      endif
      if pCount() > 2
         cOut+= [ VALUE="]+cValue+["]
      endif
      if pCount() > 3
         cOut+= [ MAXLENGTH="]+nMaxLength+["]
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamText

   function streamPassword( cName, nSize, cValue, nMaxLength )
      /*
          the password control in HTML is exactly the same
          as the text control, except that a password mask
          is displayed for each character entered, rather
          than displaying the character.

          The parameters for this control are exactly the
          same as the text control -- see comments for
          this above.
      */
      cOut = [<INPUT TYPE="PASSWORD" NAME="]+cName+["]
      if pCount() > 1
         cOut+= [ SIZE="]+nSize+["]
      endif
      if pCount() > 2
         cOut+= [ VALUE="]+cValue+["]
      endif
      if pCount() > 3
         cOut+= [ MAXLENGTH="]+nMaxLength+["]
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamPassword

   function streamRadio( cName, cValue, cText, lChecked )
      /*
         radio controls in HTML use multiple controls
         just like in dBASE. In HTML they are grouped
         by the NAME property (cName parameter)
         The VALUE is returned for whichever radiobutton
         was checked when the HTML form was submitted ...

         Parameters:
             cName --    as text control
             cValue --   value returned for radiobutton selected
             cText --    text that appears to the right of the 
                         radiobutton
             lChecked -- is this radiobutton defaulting to 
                         a 'selected' state?
      */
      cOut = [<INPUT TYPE="RADIO" NAME="]+cName+["]
      cOut+= [ VALUE="]+cValue+["]
      if pCount() > 3 and lChecked 
         cOut+= [ CHECKED]
      endif
      cOut+= [>]
      if pCount() > 2
         cOut+= cText
      endif

      this.streamDetail( cOut )
   // end of method: streamRadio

   function streamCheckbox( cName, cValue, cText, lChecked )
      /*
          the Checkbox works similar to a dBASE checkbox,
          the 'value' is actually the text assigned to
          the checkbox. Note that the HTML checkbox 
          returns the value property, not a true or false
          property ... when checking your return values
          you would want to check for the value -- i.e.,
            if you pass a value property to this method
            of "A Test", and a name of "MyCheckbox", 
            if the checkbox is checked, the value returned is:
               oCGI["MyCheckbox"] = "A Test"
            OTHERWISE, the name is not returned at all -- 
            so your code would need to check for the existance
            of the key:
               if oCGI.isKey( "MyCheckbox" )
                  // it was checked and the value was passed on
               endif
          Parameters:
             cName --    as text control
             cValue --   value returned if checkbox was checked
             cText --    text that appears to the right of the 
                         Checkbox
             lChecked -- is this checkbox defaulting to a 'checked' 
                         state?
      */
      cOut = [<INPUT TYPE="CHECKBOX" NAME="]+cName+["]
      cOut+= [ VALUE="]+cValue+["]
      if pCount() > 3 and lChecked 
         cOut+= [ CHECKED]
      endif
      cOut+= [>]
      if pCount() > 2
         cOut+= cText
      endif

      this.streamDetail( cOut )
   // end of method: streamCheckbox

   function streamReset( cText )
      /*
         The reset button simply clears out the values in
         an HTML form ...

         Parameters: cText -- text displayed on the button
                              this is optional ...
      */
      cOut = [<INPUT TYPE="RESET"]
      if pCount() => 1 // anything beyond 1 is ignored
         cOut+= [ VALUE="]+cText+["]
      endif
      cOut+= [>]

      this.streamDetail( cOut )
   // end of method: streamReset

   function streamSubmit( cText )
      /*
         The submit button submits the contents of the form ...

         Parameters: cText -- text displayed on the button
                              this is optional ...
      */
      cOut = [<INPUT TYPE="SUBMIT"]
      if pCount() => 1 // anything beyond 1 is ignored
         cOut+= [ VALUE="]+cText+["]
      endif
      cOut+= [>]

      this.streamDetail( cOut )
   // end of method: streamSubmit

   function streamTextArea( cName, nRows, nCols, cValue )
      /*
          A textArea control in HTML is like an editor control
          In HTML this one actualy has two tags -- the second
          is used after the default value ...
          

          Parameters:
              cText -- as other controls except the buttons
                       reset/submit
              nRows -- this is the number of rows to take up
                       on a form -- it does not limit the
                       input, just the display
              nCols -- same as nRows -- how many columns to use.
              cValue -- optional, this is a default value if
                       adding data, or the contents of the field
                       you want to display ...
      */
      cOut = [<TEXTAREA NAME="]+cName+["]
      if pCount() > 1
         cOut+= [ ROWS="]+nRows+["]
      endif
      if pCount() > 2
         cOut+= [ COLS="]+nCols+["]
      endif
      cOut+= [>]
      this.streamDetail( cOut )
      if pCount() > 3
         this.streamDetail( cValue )
      endif
      cOut = "</TEXTAREA>"
      this.streamDetail( cOut )
   // end of method: streamTextArea

   // --------------------------------------------------
   // The select control is interesting and different
   // from all the other HTML controls ... takes
   // some extra work. For our purposes, it's easier
   // to break it out into parts, allowing you to define
   // your SELECT in as much detail as you need ...
   // ex:
   //     oCGI.streamSelectBegin( "MyList" )
   //     oCGI.streamOption( "Option1", "Display this", true )
   //     oCGI.streamOption( "Option2", "Display another" )
   //     oCGI.streamSelectEnd()
   // --------------------------------------------------
   function streamSelectBegin( cName, nSize, lMultiple )
      /*
          This method starts the Select object, but you must
          also call the streamOption method for each item
          you wish to appear in your SELECT, and when done
          you must call the streamSelectEnd() method to
          stuff the </SELECT> tag into the form ... (without
          that tag your form is hosed!)

          Parameters:
             cName -- see other controls
             nSize -- if size is zero, or omitted, 
                      you will get a combobox. If
                      the size is larger than that,
                      you will get a listbox ...
             lMultiple -- allow multiple selections?
                      NOTE: If you select this, you 
                      will, by definition, get a listbox,
                      not a combobox ...
      */
      cOut = [<SELECT NAME="]+cName+["]
      if pCount() > 1
         if nSize > 1
            cOut+= [ SIZE="]+nSize+["]
         endif
      endif
      if pCount() > 2 and lMultiple
         cOut+= [ MULTIPLE]
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamSelect

   function streamOption( cValue, cText, lSelected )
      /*
          This method streams out the individual options that
          are needed for your select control.

          cValue -- is the value returned when the form is submitted
                    if this option is selected (required)
          cText  -- is the text displayed (it can be different from
                    the value) (required)
          lSelected -- is this selected when the form is displayed 
                    the first time?  (optional)
      */
      cOut = [<OPTION VALUE="]+cValue+["]
      if pCount() > 2 and lSelected
         cOut+= [ SELECTED]
      endif
      cOut+= [>]
      this.streamDetail( cOut )

      this.streamDetail( cText )
   // end of method: streamOption

   function streamSelectEnd
      // a simple method, this just makes sure we have
      // the end tag for the select option -- without it,
      // we have a messed up form ...
      this.streamDetail( "</SELECT>" )
   // end of method: streamSelectEnd()

   function streamHidden( cName, cValue )
      /*
         The 'Hidden' control for HTML forms is
         special -- the user never sees it, but it
         can be used to pass persistant values
         from one HTML form to another program;
         etc.

         Parameters:
           cName is the name -- if you want this to match a field,
                make sure you use *exact* case every time you 
                reference this, as the case is vital to the way 
                the web classes work (they are subclassed from 
                an Associative Array, and it is case sensitive).
           cValue is the value you want to 'Hide' -- could
                be the contents of the field
      */
      cOut = [<INPUT TYPE="HIDDEN" NAME="]+cName+["]
      cOut+= [ VALUE="]+cValue+[">]
      this.streamDetail( cOut )
   // end of method: streamHidden

   function streamFile( cName, nSize )
      /*
          streamFile streams out the HTML <INPUT TYPE="FILE" ...>
          tag, which gives an entryfield, and a "Browse" button,
          allowing a user to select a filename. This is quite
          useful when working with FTP or other methods of 
          uploading a file to a server.

          Parameters:
             cName -- the name of the name/value pair
             nSize -- width of the entryfield
      */ 
      cOut = [<INPUT TYPE="FILE" NAME="]+cName+["]
      if pCount() > 1
         cOut+= [ SIZE="]+nSize+["]
      endif
      cOut += [>]
      this.streamDetail( cOut )
   // end of method: streamFile

   function errorMessage( oErr, cTitle, cTitleBackColor, cTitleTextColor, cContact1, cContact2 )
      /*
          errorMessage was created after discovering that
          in some cases, an error caught by a try/catch
          will attempt to create a new page in the already
          streamed page, which causes one of two problems:
             1) It just plain looks wrong (the information
                streamed out in the streamHeader() messes things 
                up)
             2) The second body tag causes the browser to not 
                display anything at all ...

          The purpose of this method then, is to display an error
          after your header and body tags have been streamed
          out to your user. 

          I copied code from sorryPage() and errorPage()
          where it seemed appropriate ...

          Use: 
             After your form's header/body have been streamed
             out, wrap the rest of your program in a try/catch
             and when an error occurs, pass it to *this* method
             instead ...

          The parameters are:
          oErr: The error object generated. 
          cTitle: The title to display (default is "Error!" )
          cTitleBackColor: this is the background of the title
                 displayed across the top of the page --
                 if none is given, this method will use
                 the defaults set for the class.
          cTitleTextColor: this is the color of the text used
                 in the title at the top of the page ... as
                 above.
          cContact1: default is "Please contact the"
          cContact2: default is "Webmaster" 
      */

      // make sure we have something besides a null for
      // the streamTitle() method:
      cTitleBackColor = iif( empty( cTitleBackColor), this.TitleBackColor, cTitleBackColor )
      cTitleTextColor = iif( empty( cTitleTextColor), this.TitleTextColor, cTitleTextColor )
      if empty( cTitle )
         cTitle = "Error!"
      endif

      // if the header hasn't streamed out, don't do this:
      if this.isHeaderStreamed
         this.streamDetail( "</CENTER>" ) // in case a center tag was left 'open'
         this.streamDetail( "</TABLE>" ) // in case an open table tag is out there
      endif

      aError = new array()                // Create array for messages
      with (aError)
         add('File: '+oErr.Filename)     // Add file/line and message
         add('Line No:'+oErr.LineNo)
         add(''+oErr.Message)
      endwith

      // If WebMaster eMail address exists, use it as a Mailto: address
      // on the bottom of the error page.
      if empty ( cContact1 )
         cContact1 = "Please contact the"
      endif
      if empty( cContact2 )
         cContact2 = "Webmaster"
      endif
      if len(this["WebMasterEMail"]) > 0
         cRecover  = '<I><FONT FACE="Arial, Helvetica">'+cContact1+' </FONT>'+;
                     '</I><A HREF="mailto:'+this["WebMasterEMail"]+'"><I>'+;
                     '<FONT FACE="Arial, Helvetica">'+cContact2+'</FONT></I></A>'+;
                     '<I><FONT FACE="Arial, Helvetica">.</FONT></I>'
      else
         cRecover = cContact1+" "+cContact2//'Contact the WebMaster'
      endif

      // stream title -- using the title, it will be big and bold
      // and grab someone's attention
      this.streamDetail( "<P>" )
      cTitle = iif( empty( cTitle ), "Error!", cTitle )
      this.streamTitle( cTitle, cTitleBackColor, cTitleTextColor )
      this.streamDetail( "<P>" )

      ////// Traverse the message array
      for n = 1 to aLen(aError,1)
          this.streamDetail( aError[n] )
          this.streamDetail( '<P>' )
      next

      // stream out the recover message:
      this.streamDetail( cRecover )

      // stream the end of the HTML document
      this.streamFooter()

      // stop the application here
      quit
   // end of method: errorMessage

   function errorPage( oErr, cTitle, cServerError, cTitleBackColor, cTitleTextColor, cContact1, cContact2 )
      /*
          errorPage is modified in the same way that
          sorryPage is below ...

          The parameters are:
          oErr: The error object generated. 
          cTitle: Title -- default is "Error!"
          cTitleBackColor: this is the background of the title
                 displayed across the top of the page --
                 if none is given, this method will use
                 the defaults set for the class.
          cTitleTextColor: this is the color of the text used
                 in the title at the top of the page ... as
                 above.
          cContact1: default is "Please contact the"
          cContact2: default is "Webmaster" 

          Note: this uses the sorryPage() method to display
          the error ...
      */

      // make sure we have something besides a 'null' to pass across
      // to sorryPage
      cTitleBackColor = iif( empty( cTitleBackColor), this.TitleBackColor, cTitleBackColor )
      cTitleTextColor = iif( empty( cTitleTextColor), this.TitleTextColor, cTitleTextColor )
      if empty( cTitle )
         cTitle = "Error!" 
      endif
      aError = new array()                // Create array for messages
      if type( "oErr" ) = "C" // Character string, not exception object
         aError.add( "Error: "+oErr )
      else
         with (aError)
            add('File: '+oErr.Filename)     // Add file/line and message
            add('Line No:'+oErr.LineNo)
            add(''+oErr.Message)
         endwith
      endif

      // if the header hasn't streamed out, don't do this:
      if this.isHeaderStreamed
         this.streamDetail( "</CENTER>" ) // in case a center tag was left 'open'
         this.streamDetail( "</TABLE>" ) // in case an open table tag is out there
      endif

      // If WebMaster eMail address exists, use it as a Mailto: address
      // on the bottom of the error page.
      if empty ( cContact1 )
         cContact1 = "Please contact the"
      endif
      if empty( cContact2 )
         cContact2 = "Webmaster"
      endif
      if len(this["WebMasterEMail"]) > 0
         cRecover  = '<I><FONT FACE="Arial, Helvetica">'+cContact1+' </FONT>'+;
                     '</I><A HREF="mailto:'+this["WebMasterEMail"]+'"><I>'+;
                     '<FONT FACE="Arial, Helvetica">'+cContact2+'</FONT></I></A>'+;
                     '<I><FONT FACE="Arial, Helvetica">.</FONT></I>'
      else
         cRecover = cContact1+" "+cContact2
      endif
                                        // Call sorryPage() to display  
      if empty( cServerError )
         cServerError = "An error ocurred on the server"
      endif

      this.sorryPage(aError,cTitle,cServerError,cRecover,;
                     cTitleBackColor, cTitleTextColor )
      quit
   return true
   // end of method: errorPage

   function SorryPage( cMsg, cTitle, cSubTtl, cRcvr, cTitleBackColor, cTitleTextColor )
      /*
          SorryPage is modified to use other methods here ...

          The parameters are:

          cMsg: this is the message you want to display
                for the user -- note that it can be an array --
                the message can then have multiple lines, and
                so on, and the message can have HTML embedded in it
          cTitle: This is the title to display, default is "Sorry!"
          cSubTtl: this is the subtitle -- a title under the
                "sorry" heading
          cRcvr: this is the recovery message -- it explains
                 to the user what to do to recover from the
                 error ...
          cTitleBackColor: this is the background of the title
                 displayed across the top of the page --
                 if none is given, this method will use
                 the defaults set for the class.
          cTitleTextColor: this is the color of the text used
                 in the title at the top of the page ... as
                 above.

      */
  
      aMsg = iif(empty(cMsg),'',cMsg)

      if type('aMsg') = 'C'   // if message is not array,
         aMsg = new array()   // convert to a one-element array
         aMsg.add(cMsg)
      endif
                               // Make private versions for type()
      cRecover =  iif(empty(cRcvr),'Press Back button and try again.',cRcvr)  
      cSubtitle = iif(empty(cSubTtl),'',cSubTtl)

      /////// Stream out header
      if empty( cTitle )
         cTitle = "Sorry!"
      endif
      this.StreamHeader(cTitle)

      ////// Body tag (starts body of page)  
      this.streamBody()

      // stream out the title:
      cTitleBackColor = iif( empty( cTitleBackColor), this.TitleBackColor, cTitleBackColor )
      cTitleTextColor = iif( empty( cTitleTextColor), this.TitleTextColor, cTitleTextColor )
      this.streamTitle( cTitle, cTitleBackColor, cTitleTextColor )

      // and here we stream out the message ... this is going
      // to be indented a bit. Unlike the original, which uses
      // a table, this is going to just use the BLOCKQUOTE tags
      // which is a lot simpler ...
      this.streamDetail( "<BLOCKQUOTE>" )

      ////// Subtitle
      if not empty( cSubTitle )
         this.streamDetail('<I><FONT SIZE="3" FACE="Arial, Helvetica">'+cSubTitle+'</FONT></I>')
         this.streamDetail('<P>')
      endif

      // Set the font:
      this.streamDetail( [<FONT SIZE="2" FACE="Arial, Helvetica">] )
      ////// Traverse the message array
      for n = 1 to aLen(aMsg,1)
          this.streamDetail( aMsg[n] )
          this.streamDetail( '<P>' )
      next

      ////// If recovery instructions exist (Contact Webmaster, Press Back
      ////// 		Button, etc.), stream it here
      if not empty(cRecover)
         this.streamDetail( cRecover )
      endif

      // turn it off ...
      this.streamDetail( [</FONT>] )

      // end the blockquote
      this.streamDetail( "</BLOCKQUOTE>" )

      // and stream out the end of the HTML document
      this.streamFooter()

      // if we output this page, the mini-application is done: 
      quit
   // end of method: SorryPage

endclass

