/*
   -------------------------------------------------------------------
   logFile.cc

   Programmer:
          Erich Zippel

   Version History
          December 10, 2004 -- original dUFLP release

   Description:

   Custom class for logging information to a file.
   
   Esp. usefull in any serverbased cleanup tasks, as those are not
   allowed to display messages.

   Beware:
   There is a specific errorhandling problem.
   If a Logfile is blocked by a viewing process an error is thrown
   and has to be catched without a message.
   To keep things simple information is redirected to a second file
   named 'error' taking the messages.
   If both are blocked nothing is recorded at all.

   Functions:
          writelogfile(logfile, logline)
              writes the logline to a new or adds to an existing file
          Errorlogfile(logline)
              takes the messages from writelogfile, when logfile is
              blocked. Can also be used directly
          Newlogfile(logfile)
              Generates/starts a new logfile
              This is especially useful to delete existing entries as
              wirtelogfile will also create a new one but will try to
              ammend if the file already exists.
           

   Instantiation:
          set procedure to :dUFLP:logfile.cc additive
          writelogfile("mylog", "mymessage")
       
   Dependencies:
          None
          Existence of folder to store files is not checked, so beware.
   -------------------------------------------------------------------
*/

#define logfilepath     "" // change to something like	"c:\log\" or "\\server\log\"
#define logfileext      ".log"       //default extension

procedure writelogfile(logfile, logline)
	// generates logentries
	local f
   private clf
	clf=logfilepath+logfile+logfileext
	f = new File()
	try
		if f.exists(clf)
			set alternate to &clf additive
		else
			set alternate to &clf  //neu protokollieren
		endif
		set alternate on
		?  new date()+0
		?? " : "+logline
		set alternate off
		close alternate
	catch (exception e)
		errorlogfile(logline)
	endtry
return

procedure errorlogfile(logline)
	// protokolliert Logfehler, insb. wenn das Logfile offen
	local ef
	private cerf //keine Makrosubstitution fr lokale Variablen
	try
		cerf=logfilepath+"error"+logfileext
		ef = new File()
		if ef.exists(cerf)
			set alternate to &cerf additive 
		else
			set alternate to &cerf 
		endif
		set alternate on
		?  new date()+0
		?? " error logging : "+logline
		set alternate off
		close alternate
	catch (exception e)
		// wenn auch offen
	endtry
return

procedure newlogfile(logfile)
	//legt beliebige neue Logdatei an
	private clf
	try 
		clf=logfilepath+logfile+logfileext
		set alternate to &clf //protokollieren
		set alternate on
		?  new date()+0
		?? " : LOGFILE created"
		set alternate off
		close alternate
	catch (exception e)
		errorlogfile("error creating "+logfile)
	endtry
return


