// ------------------------------------------------
// Height/Width of buttons based on Manifest
// file being used or not. Unfortunately,
// the only way to make this work is
// to comment/uncomment code -- in this case
// if you are NOT using a Manifest file,
// comment out the next statement:
#define Manifest true

#ifdef Manifest
   // Manifest file sizes:
   #define pButtonHeight 30
   #define pButtonWidth  30
#else
   // No manifest file, use 'old' settings:
   #define pButtonHeight 26
   #define pButtonWidth  25
#endif
// ------------------------------------------------
#define Spacer         3 // pixels between containers

/*
   --------------------------------------------------------------
   pCustButt.cc
   Author...: Ken Mayer
   Date.....: July 19, 2004
   Notes....: This is a set of controls that are subclassed
              from :dUFLP:CustButt.cc -- these buttons
              are and toolbars are defined specifically
              to be used on forms that have a metric
              of 6 (Pixels). These should work exactly
              as the buttons in CustButt.cc, and since the
              only thing that really has changed is the 
              metric and size definitions, you should be
              able to use them exactly as before.

              NOTE: If you try to use these on a form that
              has a metric other than 6 (Pixels), then 
              the buttons will look odd, the same for the
              toolbar containers ...

              The names of the controls have been changed,
              in most cases "km" was replaced with "p" 
              (for 'Pixels').

              If the code in CustButt.cc is modified for any
              of the buttons, it should be reflected here.
              However, without jumping through hoops I didn't
              feel up to jumping through, the code for the
              toolbars is duplicated here ... so any changes
              to code in CustButt.cc would need to be 
              replicated here as well.

              Changes to images in CustButt.cc would be 
              reflected here as well. However, changes
              to images here would not be reflected back
              to CustButt.cc. 

   Dependencies:
              :dUFLP:CustButt.cc
   --------------------------------------------------------------
*/

// -------------------------
// The Toolbars
// -------------------------
class pKensToolbar( parentObj ) of CONTAINER(parentObj) custom
   with (this)
      transparent := true
      // Calculate width of container based on width of buttons
      // used, spacer between, and rowstate object's width:
      width       := (4*pButtonWidth)+(3*pButtonWidth)+(5*pButtonWidth)+;
                     (pButtonWidth)+(4*Spacer)+66 // 66 is width of rowstate
      height      := 30.00
      metric      := 6        // Pixels
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.NavigateButtons = new pNavigateButtons(this)
   with (this.NAVIGATEBUTTONS)
      left   := 0
      top    := 0
      metric := 6        // Pixels
   endwith

   this.CLIPBOARDBUTTONS = new pCLIPBOARDBUTTONS(this)
   with (this.CLIPBOARDBUTTONS)
      left   := this.NavigateButtons.left+this.NavigateButtons.width+Spacer // 108
      top    := 0
      metric := 6        // Pixels
   endwith

   this.EDITBUTTONS = new pEDITBUTTONS(this)
   with (this.EDITBUTTONS)
      left   := this.ClipboardButtons.left+this.ClipboardButtons.width+Spacer //189
      top    := 0
      metric := 6        // Pixels
   endwith

   this.EXITBUTTONS = new pEXITBUTTONS(this)
   with (this.EXITBUTTONS)
      left   := this.EditButtons.left+this.EditButtons.width+Spacer //322
      top    := 0
      metric := 6        // Pixels
   endwith

   this.ROWSTATE = new pROWSTATE(this)
   with (this.ROWSTATE)
      left   := this.ExitButtons.left+this.ExitButtons.width+Spacer // 367
      top    := 4
      metric := 6        // Pixels
   endwith

   function onOpen 
      // if the developer has not assigned a rowset
      // to this container, assume the form's rowset:
      if this.rowset == null
         this.rowset = form.rowset
      endif
      // June 2001 Ronnie MacGregor added the following line
      // to allow checking of the old rowset's state before
      // switching to a new rowset.
      this.oldRowset = this.rowset
      class::initRowset(this.rowset)
      return

   function initRowset(oNewRowset)
      local nState, nAnswer
      // Modifications June 2001 Ronnie MacGregor
      // This function re-written to deal better
      // with dynamic switching of rowsets.
      //
      // We now check the state of the current rowset first.
      // If we are editing/appending the current rowset, 
      // we don't want to switch to another rowset or navigate.
      //
      // This function will now accept a parameter for the new rowset,
      // although the old method of setting the rowset property
      // and calling this function with no parameter will still work.
      //
      // This function now returns a logical value to allow the
      // developer to assess the success or failure of the attempted
      // change of rowset.
      //
      if not empty(argvector(1))
         // Set the toolbar's new rowset
         this.rowset = oNewRowset
      endif
      // Check the state of the old rowset first
      // if we are editing/adding a row, we don't want 
      // to navigate.
      try
         nState = this.oldRowset.state
      catch(exception e)
         nState = 0 // no rowset assigned
      endtry
      // 
      do case
         case nState == 2 or nState == 3  // Edit Append
              if this.oldRowset.modified
                 nAnswer = msgbox(SAVECHANGES, ;
                                  DATACHANGED,32+3)
                 do case
                    case nAnswer == 6
                       this.oldRowset.save()
                    case nAnswer == 7
                       this.oldRowset.abandon()
                    otherwise
                       this.rowset = this.oldRowset
                       return false
                 endcase
              else
                 this.oldRowset.abandon()
              endif
         case nState == 4 // filter
              msgbox( INFILTERMODE,;
                      CANTNAVIGATE, 16 )
              this.rowset = this.oldRowset
              return false
         case nState == 5 // locate
              msgbox( INLOCATEMODE,;
                      CANTNAVIGATE, 16 )
              this.rowset = this.oldRowset
              return false
      endcase
      this.oldRowset = this.rowset
      // Set the component rowsets
      this.NavigateButtons.rowset := this.rowset
      this.EditButtons.rowset     := this.rowset
      this.ExitButtons.rowset     := this.rowset
      this.rowState.initRowset()
      //
      return true

endclass

// ---------------------
class pkmToolBar2(parentObj) of CONTAINER(parentObj) custom
   with (this)
      transparent := true
      width       := (4*pButtonWidth)+(3*pButtonWidth)+(6*pButtonWidth)+;
                     (pButtonWidth)+(4*Spacer)+66
      height  := 32
      metric  := 6        // Pixels
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.NAVIGATEBUTTONS = new pNAVIGATEBUTTONS(this)
   with (this.NAVIGATEBUTTONS)
      left   := 0
      top    := 0
      metric := 6        // Pixels
   endwith

   this.CLIPBOARDBUTTONS = new pCLIPBOARDBUTTONS(this)
   with (this.CLIPBOARDBUTTONS)
      left   := this.NavigateButtons.left+this.NavigateButtons.Width+Spacer// 104
      top    := 0
      metric := 6        // Pixels
   endwith

   this.EDITBUTTONS = new pEDITBUTTONS2(this)
   with (this.EDITBUTTONS)
      left   := this.ClipboardButtons.left+this.ClipboardButtons.width+Spacer //183
      top    := 0
      metric := 6        // Pixels
   endwith

   this.EXITBUTTONS = new pEXITBUTTONS(this)
   with (this.EXITBUTTONS)
      left   := this.EditButtons.left+this.EditButtons.width+Spacer // 336
      top    := 0
      metric := 6        // Pixels
   endwith

   this.ROWSTATE = new pROWSTATE(this)
   with (this.ROWSTATE)
      left   := this.ExitButtons.left+this.ExitButtons.width+Spacer // 367
      top    := 4
      metric := 6        // Pixels
   endwith

   function onOpen 
      // if the developer has not assigned a rowset
      // to this container, assume the form's rowset:
      if this.rowset == null
         this.rowset = form.rowset
      endif
      // June 2001 Ronnie MacGregor added the following line
      // to allow checking of the old rowset's state before
      // switching to a new rowset.
      this.oldRowset = this.rowset
      class::initRowset(this.rowset)
      return

   function initRowset(oNewRowset)
      local nState, nAnswer
      // Modifications June 2001 Ronnie MacGregor
      // This function re-written to deal better
      // with dynamic switching of rowsets.
      //
      // We now check the state of the current rowset first.
      // If we are editing/appending the current rowset, 
      // we don't want to switch to another rowset or navigate.
      //
      // This function can now accept a parameter for the new rowset,
      // although the old method of setting the rowset property
      // and calling this function with no parameter will still work.
      //
      // This function now returns a logical value to allow the
      // developer to assess the success or failure of the attempted
      // change of rowset.
      //
      if not empty(argvector(1))
         // Set the toolbar's new rowset
         this.rowset = oNewRowset
      endif
      // Check the state of the old rowset first
      // if we are editing/adding a row, we don't want 
      // to navigate.
      try
         nState = this.oldRowset.state
      catch(exception e)
         nState = 0 // no rowset assigned
         ? "No rowset" //////////////////////////////////
      endtry
      // 
      do case
         case nState == 2 or nState == 3 // Edit Append
              if this.oldRowset.modified
                 nAnswer = msgbox(SAVECHANGES, ;
                                  DATACHANGED,32+3)
                 do case
                    case nAnswer == 6
                       this.oldRowset.save()
                    case nAnswer == 7
                       this.oldRowset.abandon()
                    otherwise
                       this.rowset = this.oldRowset
                       return false
                 endcase
              else
                 this.oldRowset.abandon()
              endif
         case nState == 4 // filter
              msgbox( INFILTERMODE,;
                      CANTNAVIGATE, 16 )
              this.rowset = this.oldRowset
              return false
         case nState == 5 // locate
              msgbox( INLOCATEMODE,;
                      CANTNAVIGATE, 16 )
              this.rowset = this.oldRowset
              return false
      endcase
      this.oldRowset = this.rowset
      // Set the component rowsets
      this.NavigateButtons.rowset := this.rowset
      this.EditButtons.rowset     := this.rowset
      this.ExitButtons.rowset     := this.rowset
      this.rowState.initRowset()
      //
      return true
      
endclass

// ---------------------
class pNavigateButtons( oParent ) of CONTAINER(oParent) custom
   with (this)
      transparent := true
      metric  := 6 // Pixels
      height  := 32
      width   := (4*pButtonWidth)
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.KMTOPBUTTON1 = new pTOPBUTTON(this)
   with (this.KMTOPBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

   this.KMPREVIOUSBUTTON1 = new pPREVIOUSBUTTON(this)
   with (this.KMPREVIOUSBUTTON1)
      metric := 6 // Pixels
      top    := 0
      left   := width //25
   endwith

   this.KMNEXTBUTTON1 = new pNEXTBUTTON(this)
   with (this.KMNEXTBUTTON1)
      metric := 6 // Pixels
      left   := 2*width // 50
      top    := 0
   endwith

   this.KMENDBUTTON1 = new pENDBUTTON(this)
   with (this.KMENDBUTTON1)
      metric := 6 // Pixels
      left   := 3*width // 75
      top    := 0
   endwith
endclass

// ---------------------
class pClipboardButtons( oParent ) of CONTAINER(oParent) custom
   with (this)
      transparent := true
      metric  := 6 // Pixels
      height  := 32
      width   := (3*pButtonWidth)
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.KMCUTBUTTON1 = new pCUTBUTTON(this)
   with (this.KMCUTBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

   this.KMCOPYBUTTON1 = new pCOPYBUTTON(this)
   with (this.KMCOPYBUTTON1)
      metric := 6 // Pixels
      left   := width //25
      top    := 0
   endwith

   this.KMPASTEBUTTON1 = new pPASTEBUTTON(this)
   with (this.KMPASTEBUTTON1)
      metric := 6 // Pixels
      left   := 2*width // 50
      top    := 0
   endwith


endclass

// ---------------------
class pEditButtons( oParent ) of CONTAINER(oParent) custom
   with (this)
      transparent := true
      metric  := 6 // Pixels
      height  := 32
      width   := (5*pButtonWidth) //131
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.KMADDBUTTON1 = new pADDBUTTON(this)
   with (this.KMADDBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

   this.KMEDITBUTTON1 = new pEDITBUTTON(this)
   with (this.KMEDITBUTTON1)
      metric := 6 // Pixels
      top    := 0
      left   := width // 25
   endwith

   this.KMSAVEBUTTON1 = new pSAVEBUTTON(this)
   with (this.KMSAVEBUTTON1)
      metric := 6 // Pixels
      left   := 2*width // 50
      top    := 0
   endwith

   this.KMABANDONBUTTON1 = new pABANDONBUTTON(this)
   with (this.KMABANDONBUTTON1)
      metric := 6 // Pixels
      left   := 3*width // 75
      top    := 0
   endwith

   this.KMDELETEBUTTON1 = new pDELETEBUTTON(this)
   with (this.KMDELETEBUTTON1)
      metric := 6 // Pixels
      left   := 4*width // 100
      top    := 0
   endwith
endclass

// ---------------------
class pEditButtons2( oParent ) of CONTAINER(oParent) custom
   with (this)
      transparent := true
      metric  := 6 // Pixels
      height  := 32
      width   := (6*pButtonWidth) //151
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.KMADDBUTTON1 = new pADDBUTTON(this)
   with (this.KMADDBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

   this.KMCARRYBUTTON1 = new pCARRYBUTTON(this)
   with (this.KMCARRYBUTTON1)
      metric := 6 // Pixels
      top    := 0
      left   := width // 25
   endwith

   this.KMEDITBUTTON1 = new pEDITBUTTON(this)
   with (this.KMEDITBUTTON1)
      metric := 6 // Pixels
      top    := 0
      left   := 2*width // 50
   endwith

   this.KMSAVEBUTTON1 = new pSAVEBUTTON(this)
   with (this.KMSAVEBUTTON1)
      metric := 6 // Pixels
      left   := 3*width // 75
      top    := 0
   endwith

   this.KMABANDONBUTTON1 = new pABANDONBUTTON(this)
   with (this.KMABANDONBUTTON1)
      metric := 6 // Pixels
      left   := 4*width // 100
      top    := 0
   endwith

   this.KMDELETEBUTTON1 = new pDELETEBUTTON(this)
   with (this.KMDELETEBUTTON1)
      metric := 6 // Pixels
      left   := 5*width // 125
      top    := 0
   endwith

endclass

// ---------------------
class pFindButtons(parentObj) of CONTAINER(parentObj) custom
   with (this)
      transparent := true
      metric := 6 // Pixels
      width  := (3*pButtonWidth) //81
      height := 32
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith

   this.KMSEARCHBUTTON1 = new pSEARCHBUTTON(this)
   with (this.KMSEARCHBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

   this.KMREPLACEBUTTON1 = new pREPLACEBUTTON(this)
   with (this.KMREPLACEBUTTON1)
      metric := 6 // Pixels
      left   := width // 25
      top    := 0
   endwith

   this.KMSEARCHAGAINBUTTON1 = new pSEARCHAGAINBUTTON(this)
   with (this.KMSEARCHAGAINBUTTON1)
      metric := 6 // Pixels
      left   := 2*width // 50
      top    := 0
   endwith
endclass


// ---------------------
class pExitButtons( oParent ) of CONTAINER(oParent) custom
   with (this)
      transparent := true
      metric  := 6 // Pixels
      height  := 32
      width   := pButtonWidth //31
      colorNormal := "BtnFace"
      borderStyle := 3   // none
   endwith
   this.rowset = null

   this.KMCLOSEBUTTON1 = new pCLOSEBUTTON(this)
   with (this.KMCLOSEBUTTON1)
      metric := 6 // Pixels
      left   := 0
      top    := 0
   endwith

endclass

// ------------------------
// The Pushbuttons
// ------------------------
class pBaseSpeedButton(parentObj) of BaseSpeedButton(parentObj) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass


class pTopButton( parentObj ) of kmTopButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pPreviousButton( parentObj ) of kmPreviousButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pNextButton( parentObj ) of kmNextButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pEndButton( parentObj ) of kmEndButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pAddButton( parentObj ) of kmAddButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pCarryButton( parentObj ) of kmCarryButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pEditButton( parentObj ) of kmEditButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pSaveButton( parentObj ) of kmSaveButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pDeleteButton( parentObj ) of kmDeleteButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pAbandonButton( parentObj ) of kmAbandonButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pFilterButton( parentObj ) of kmFilterButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pLocateButton( parentObj ) of kmLocateButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pCutButton( parentObj ) of kmCutButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pCopyButton( parentObj ) of kmCopyButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pPasteButton( parentObj ) of kmPasteButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pCloseButton( parentObj ) of kmCloseButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pYesButton( parentObj ) of kmYesButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pNoButton( parentObj ) of kmNoButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pCancelButton( parentObj ) of kmCancelButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pQuestionButton( parentObj ) of kmQuestionButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pLightningButton( parentObj ) of kmLightningButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pInfoButton( parentObj ) of kmInfoButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pFromDiskButton( parentObj ) of kmFromDiskButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pToDiskButton( parentObj ) of kmToDiskButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pOpenFolderButton( parentObj ) of kmOpenFolderButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pWrenchButton( parentObj ) of kmWrenchButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pPrintButton( parentObj ) of kmPrintButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pSearchButton( parentObj ) of kmSearchButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pSearchAgainButton( parentObj ) of kmSearchAgainButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pReplaceButton( parentObj ) of kmReplaceButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pSearchButton2( parentObj ) of kmSearchButton2( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pExitButton( parentObj ) of kmExitButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pFirstPageButton( parentObj ) of kmFirstPageButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pNextPageButton( parentObj ) of kmNextPageButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pPreviousPageButton( parentObj ) of kmPreviousPageButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

class pLightbulbButton( parentObj ) of kmLightbulbButton( parentObj ) from ":dUFLP:CustButt.cc" custom
   with (this)
      metric := 6 // Pixels
      height := pButtonHeight
      width  := pButtonWidth
   endwith
endclass

// -----------------------
// The RowState indicator:
// -----------------------
class pRowState( parentObj ) of kmRowState( parentObj ) from ":dUFLP:CustButt.cc" Custom
   with (this)
      metric := 6 // Pixels
      height := 18
      width  := 66
   endwith
endclass


