// Metric: Non Specific
//------------------------------------------------------------------------
//
//  SeekerSQL.cc  --  Incremental search custom control
//  Cloned from Seeker.cc (K Chan)
//  
//  Visual dBASE Samples Group, and Ken Chan
//
//  $Revision:   1.3  $
//
//  Copyright (c) 1997-1999, Borland International, Inc. and
//  1999, dBASE, Inc.
//  Version for SQL databases (Oracle etc)  David Baily 16/07/2001
//  Modifications signified by // SQLRev
//  This code is mostly Seeker.cc renamed to avoid confusion.
// 
//  Explanation....
//    SQL databases do not have dBase style indexes.
//  The seek controls in oSeek (below) do not work on these SQL
//  tables. (unsurprisingly).  oSeek has been modified to search SQL
//  tables using ApplyLocate().  A short delay is to be expected
//  when first using the seekerSQL control, but I have found that
//  subsequent searches are quicker when re-defining the search.
//  With the speed of modern PCs this should not be a significant
//  problem, but if it happens use the opportunity to think of
//  higher things.
//
//  NOTE: The query in question must have an ORDER BY clause
//  This code also sets and then resets the rowset's locateOptions
//  setting. 
//
//  Note.. Better programmers than me should be able to incorporate
//  this method of search into a standard seeker control by spotting
//  when there is not an index present.  This method should also
//  work on un-indexed dBase tables, but I have not tried it.
//
//  All rights reserved.
//
// Usage Notes:
//
// By default, this control should be in the Component Palette
// when you create a form, on the "Custom" tab. If it is not,
// then before creating a form or modifying a form, issue
// the command (in the Command Window):
//    set procedure to _dbwinhome()+"CUSTOM\SEEKER.CC" additive
// 
// In the form designer, make sure that the rowset you wish
// to use this control with has the indexName property set to
// an index tag that is character, and created with the UPPER()
// function, so that it is indexed on the upper-case of any
// characters in the field(s) index on.
//
// Drag the SEEKER control to the design surface. When the form
// is run, entering text in the entryfield will cause an
// incremental search to occur, using the index tag mentioned
// above.
//
// NOTE: do not attempt to datalink this custom entryfield, 
// first, it is "protected" so that you cannot access this
// property in the designer, and second, the code will not function
// properly if datalinked to a field in the table.
//
//------------------------------------------------------------------------
//

CLASS SeekerSQL( oCont ) of Entryfield( oCont ) CUSTOM

  protect lNear, lExact, lFound, lRefresh, nMaxPos
  protect seekRowset
  // This control uses the active index to seek; DO NOT datalink!
  protect dataLink

  // Set stock properties
  this.selectAll := true
  this.value     := ""
  
  // Custom properties
  this.rowset    = null    // Set to override form.rowset
  this.tabAfter  = 0       // Automatically tab after typing N chars
  this.seek      = this.xSeek

  // Internal properties
  this.lNear     = ( SET( "NEAR"  ) == "ON" )
  this.lExact    = ( SET( "EXACT" ) == "ON" )
  this.lRefresh  = this.form.RefreshAlways
  this.lFound    = true
  this.nMaxPos   = 0       // Max position of cursor in entryfield

  FUNCTION onGotFocus
    if form.rowset == null       // Xbase settings
      this.lNear    = ( SET( "NEAR"  ) == "ON" )
      this.lExact   = ( SET( "EXACT" ) == "ON" )
      this.lRefresh = form.RefreshAlways
      set near on                // To do incremental matching
      set exact off              // To return FOUND() true on substring match
      form.RefreshAlways = true
      this.seek     = this.xSeek // Use method with Xbase SEEK()
    else
      // For OODML, use findKeyNearest(), which always does a
      // partial key match (like EXACT OFF)
      this.seek     = this.oSeek
      // Assign protected object reference to either form's rowset
      // or control's rowset
      this.seekRowset = iif( this.rowset == null, form.rowset, this.rowset )
      // ----------------------------------------------
      // FOR SQL VERSION:
      // save current setting 
      this.oldLocateOptions = this.seekRowset.locateOptions
      // change to 3
      this.seekRowset.locateOptions := 3
      // ----------------------------------------------
    endif
    this.lFound  = true
    this.nMaxPos = 0

  FUNCTION onKey( nChar, nPosition, bShift, bControl )
    if nextkey() == 0
      // Wait until there are no keystrokes pending
      if not empty( this.value ) and ;
          ( this.lFound or nPosition <= this.nMaxPos )
        this.lFound = this.seek()
      endif
      if this.tabAfter > 0 and nPosition > this.tabAfter
        // Stuff tab if position is greater than tabAfter
        this.keyboard( chr(9) )
      endif
      this.nMaxPos = max( nPosition, ;
          min( this.nMaxPos, len( trim( this.value ))) )
    endif

  FUNCTION xSeek
    local lRet
    lRet = seek( this.normalizedValue() )
    if eof()               // Always move record pointer to real record
      goto bottom
    endif
  RETURN lRet

  FUNCTION oSeek
    local lRet, SQLString, n1
    // SQLRev -- check to see if an indexName exists,
    // if it does, then use the standard findKeyNearest code
    // and the index.
    if NOT empty( this.seekRowset.indexName )
       lRet = this.seekRowset.findKeyNearest( this.normalizedValue() )
    else // no indexName, let's try checking for an ORDER BY clause:
       // this is the code to handle the SQL version of the seek:
       lRet = true
       //SQLRev Parse the sql string and find the primary order column
       //Parse and you will find (proverb)
       SQLString = this.seekrowset.parent.sql.toUpperCase()
       if "ORDER BY" $ SQLString
          n1 = SQLString.indexOf("ORDER BY")+8
          SQLString = SQLString.substring(n1,SQLString.length).lefttrim()
          for i = 1 to SQLString.length
             c1 = SQLString.charAt(i-1)
             if c1 $ " ,"
                SQLString = SQLString.substring(0,i-1)
             endif
          next
       else
          lRet = false
       endif
       if lRet
          lRet = this.seekRowset.ApplyLocate( SQLString+" = '"+this.normalizedValue()+"'")
//? SQLString+" = '"+this.normalizedValue()+"'"
       endif
       if ( this.seekRowset.endOfSet )
         this.seekRowset.next( -1 )
       endif
    endif
  RETURN lRet

  FUNCTION normalizedValue
  RETURN this.value.rightTrim().toUpperCase()

  FUNCTION onLostFocus
    if form.rowset == null
      if not this.lNear
        set near off
      endif
      if this.lExact
        set exact on
      endif
      form.RefreshAlways := this.lRefresh
    else
      // SQL version -- reset locateOptions
      this.seekRowset.locateOptions := this.oldLocateOptions
    endif

ENDCLASS
