/*
    SetTableLevel.prg
    Author: Vic McClung
    Date..: July 18, 2000

    This function "simply" changes the level of table
    created by dBASE code, without having to exit
    dBASE.

    This is actually pretty basic, except it
    uses a bunch of API calls to do it:
      1) Close down the BDE
      2) Set the new table level in the registry
      3) Restart the BDE

    --------------------------------------------------
    Note: See "Dependencies" below -- there are quite
    a few, and you should make sure that all of these
    are available wherever you are running this code.
    This may mean for any application you need this
    ability, you copy the files noted ... either that
    or modify the code for the #INCLUDE statements,
    so that they point to the directory that contains
    the header files.

    The code has been modified here to point to the 
    to program files in the SAMPLES directory, using
    _dbwinhome to find them. However, #INCLUDE evaluates
    before _dbwinhome is available ...

    --------------------------------------------------
    Note: you cannot have any open tables or 
    databases. This code will return an error
    if you do ...

    ****************************************************
    Note: Check your BDE Administrator, in the 
       Configuration tab,
         Select Configuration,
                Drivers
                Native
                dBASE
         Look at the first item in the list on the right:
             VERSION
         If you have Version 4, this will not work.
         If you have Version 1, it will work.

         How do you change versions? I have no idea ...
    ****************************************************

    Usage:

       SetTableLevel( <cLevel> )
          Where cLevel is  '3', '4', '5' or '7'
          Any other value is invalid and
          an error will be returned ...

    Returns:
       Previous table level as a character value
       (by 'Previous', we mean the one before setting
        it using this function)
        OR it returns an empty string if there's
        an error ...

    Examples:
       SetTableLevel( "4" )
       ? SetTableLevel( "5" )
       cOldTableLevel = SetTableLevel( "7" )

    Dependencies:
       BDE4API.h     In SAMPLES directory
                     // Copy it to the INCLUDE 
                     // directory
       WinReg.h      In INCLUDE directory
       Registry.prg  In SAMPLES directory
       WinReg.h      In SAMPLES directory
       VDBASE.h      In SAMPLES directory
       STRUCTAPI.h	In SAMPLES directory
       STRUCTURE.PRG In SAMPLES directory
*/

#include <bde4api.h>
#undef CHAR
#include <winreg.h>
#include <vdbase.h>
#include <structapi.h>

function SetTableLevel(cLevel)
   local reg, cOldLevel, sInfo, ndbasesOpen
   local hs, nCursors
   private oSession
   // Check for valid level:
   if not (cLevel $ '3,4,5,7')
      AlertMessage('Invalid Table Level!', 'Error')
      return ""
   endif

   // Need these early on:
   if type('DbiSetCurrSession') # "FP"
      extern cword DbiSetCurrSession(chandle) idapi32.dll
   endif
   if type('DbiGetSesInfo') # "FP"
      extern cword DbiGetSesInfo(cptr) idapi32.dll
   endif

   // -------------------------
   // Check for open databases:
   set procedure to _dbwinhome+"samples\structure.prg" additive
   // Create BDE SESInfo structure
   sInfo = new SESInfo()
   // Check for sessions with open databases/tables
   oSession = findInstance('Session')
   ndbasesOpen = 0
   nCursorsOpen = 0
   // next item eliminates problems from one of the header files
   // defining handle to chandle
   #undef handle
   do while type('oSession') == "O"
      hs = oSession.handle
      DbiSetCurrSession(hs)
      // retrieve information on no. of open databases
      DbiGetSesInfo(sInfo.value)
      ndbasesOpen += sInfo.getMember('iDatabases')
      nCursorsOpen += sInfo.getMember('iCursors')
      oSession = findInstance('Session', oSession)
   enddo
   if ndbasesOpen > 1 or nCursorsOpen > 0
      alertMessage("You have databases and/or tables open!", "Error!")
      return ""
   endif

   // --------------
   // Close the BDE
   if type('DbiExit') # "FP"
      extern cword DbiExit(cvoid) idapi32.dll
   endif
   if type('DbiInitFn') # "FP"
      extern cword DbiInitFn(cword, cptr) idapi32.dll
   endif
   DbiExit()

   // change Table Level in the registry
   // use registry.prg in samples directory
   set procedure to _dbwinhome+"samples\registry.prg" additive
   // create registry object
   reg = new Registry(HKEY_LOCAL_MACHINE, ;
         "SOFTWARE\Borland\Database Engine\Settings" + ;
         "\DRIVERS\DBASE\TABLE CREATE" )
   cOldLevel = reg.queryValue("LEVEL")
   // set the table level
   reg.SetValue("LEVEL", cLevel, REG_SZ)
   // close reg object
   reg.close()

   // Restart the BDE
   DbiInitFn(300, null)
   close procedure _dbwinhome+"samples\registry.prg"
   close procedure _dbwinhome+"samples\structure.prg"

return cOldLevel

// structure used in code above
class SESInfo of Structure
   super::addMember( TYPE_USHORT, "iSession" )
   super::addMember( TYPE_DBINAME, "szName", SIZEOF_DBINAME )
   super::addMember( TYPE_USHORT, "iDatabases" )
   super::addMember( TYPE_USHORT, "iCursors" )
   super::addMember( TYPE_UINT, "iLockWait"  )
   super::addMember( TYPE_DBIPATH, "szNetDir", SIZEOF_DBIPATH)
   super::addMember( TYPE_DBIPATH, "szPrivDir", SIZEOF_DBIPATH)
endclass

