/*
    Setup.prg
    dUFLP Program used to set up all the appropriate files
    and settings for the dUFLP -- including:
       SourceCode Alias (dUFLP)
       Copy .h files to dBASE\Plus\INCLUDE folder
       Component Palette (icons for dUFLP controls)

    Date: June, 2007

    NOTE: This should be run from the folder that the
    dUFLP is residing in, not from some other folder,
    as it will set the source code alias to the wrong place,
    try to copy non-existant .h files, and so on.

    Dependencies:
       SetupSourceAlias.prg
       ComponentPaletteUpdate.prg
*/

// check for specific files in the current folder -- 
// if they don't exist, return an error that this is
// being run from the wrong place on the hard drive.
cDir = set("dire")
cDir+= "\"
if not file( cDir+"SetupSourceAlias.prg" ) and not file( cDir+"CRETable7.prg" )
   msgbox( "This program needs to be run"+chr(13)+;
           "from the dUFLP folder -- the"+chr(13)+;
           "folder you installed the dUFLP to."+chr(13)+;
           "Please copy this to that folder,"+chr(13)+;
           "and try again.",;
           "Program Run from Wrong Folder", 16 )
   return
endif 

// make sure that everything gets recompiled:
run( false, "cleanup.bat" )

// run SetupSourceAlias if we're here:
   // add flag in SetupSourceAlias 
   // to see if the program was called from
   // here ...
do SetupSourceAlias with "","",true

// copy the .H files to the Plus\INCLUDE folder:
local aHFiles, cInclude, cSafety
cSafety = set("safety")
aHFiles = new array()
aHFiles.dir( "*.h" )
cInclude = _dbwinhome+"Include\"
set safety off
// loop through array of file names
for i = 1 to aHFiles.size/5 // five columns of data
    copy file ( aHFiles[i,1] ) to ( cInclude+aHFiles[i,1] )
next
// deal with the BDEApi.h file in the SAMPLES folder:
copy file ( _dbwinhome+"Samples\Bde4api.h" ) to ( cInclude+"Bde4api.h" )

cCmd = [set safety ]+cSafety
&cCmd.

// run the ComponentPaletteUpdate program:
do ComponentPaletteUpdate with true

// ask if they want to restart dBASE now?

if msgbox( "The source alias 'dUFLP' has been set to"+chr(13)+;
           "'"+set("dire")+"', and is now in"+chr(13)+;
           "the "+iif( __version__ < 2, "dB2K.ini", "Plus.ini" )+;
           " file. In order for this to "+chr(13)+;
           "take effect, however, you will need to exit and "+chr(13)+;
           "restart dBASE. Selecting 'Yes' will exit and "+chr(13)+;
           "restart dBASE, but you may need/want to change the"+chr(13)+;
           "path back to the dUFLP folder ..."+chr(13)+chr(13)+;
           "In addition, the next time you open a form or report "+chr(13)+;
           "in the designer that uses one of the visual control"+chr(13)+;
           "files in the dUFLP, or if creating a new form/report"+chr(13)+;
           "and you have opened at least one of these files,"+chr(13)+;
           "the Component Palette should have new tabs "+chr(13)+;
           "with these components grouped ..."+chr(13)+chr(13)+;
           "When you come back to the dUFLP folder, you may wish"+chr(13)+;
           "to run the file Library.wfm -- double click it in"+chr(13)+;
           "the Navigator, or in the command window type:"+chr(13)+;
           "                do Library.wfm"+chr(13)+;
           "-- to see what is available to you for your use ...",;
           "Setup Complete - Exit/Restart dBASE?", 36 ) == 6 // Yes
   run(true,_app.EXEName)
   quit
endif

