*------------------------------------------------------------
*   Purpose:  
*     Create Object to hold various video settings
*     Classname:      Video
*------------------------------------------------------------
*     Platform:         Visual dBASE 7.01, 7.50
*     Original Date:    04/27/2000
*     Author:           Rick Miller
*         
*     History:          
*              Jan. 2005 - R. Wingerter
*              Added one line to get current DPI setting for screen. 
*
*              April 2005 - R. Wingerter
*              Added code to save and restore settings of
*              _app.statusbar and _app.speedbar. 
*
*              December 2005 - Ken Mayer
*              Added fonts property, based on code provided
*              by Roland Wingerter, the code by Marc VdB.
*                       
*------------------------------------------------------------
*   Syntax:
*     object = new Video()
*               the object.properties can now be queried
*               for the Video resolution and the
*               mdimaxheight, mdimaxwidth,
*               sdimaxheight, sdimaxwidth properties
*------------------------------------------------------------
*
*   This example will size the form to the maximum
*   client workarea on the screen
*
*   Example:
*     Set Procedure TO Videores.cc ADDITIVE
*     form.video1 = new VIDEO()
*     form.left   = 0
*     form.top    = 0
*     form.width  = form.video1.mdimaxwidth
*     form.height = form.video1.mdimaxheight
*
*   Note: 1) This uses PIXELS for the property values
*         2) Assigning to _app makes the object available
*            to all forms.
*            ie: _app.video1 = new Video()
*------------------------------------------------------------
*   Features:
*     Provides the user's current Video Resolution
*     Provides the maximum form width and height values
*     that can be used for a form respective of MDI or SDI
* 
*     new property: fonts
*          Returns: character
*------------------------------------------------------------

class Video

  class::open()

  function open
     LOCAL dcval, ncolors, nFonts, lSpeedbar, lStatusbar

     // Save settings
	  lSpeedbar  = _app.speedbar
     lStatusbar = _app.statusbar

     // Change settings
     _app.speedbar  := false
     _app.statusbar := false
     
     IF TYPE( "GetSystemMetrics" ) # "FP"
       extern CINT GetSystemMetrics( CINT ) USER32
     ENDIF
     this.videowidth   = GetSystemMetrics( 0 )
     this.cxborder     = GetSystemMetrics( 5 )
     this.cxfullscreen = GetSystemMetrics( 16 )
     this.cxframe      = GetSystemMetrics( 32 )
     this.videoheight  = GetSystemMetrics( 1 )
     this.cycaption    = GetSystemMetrics( 4 )
     this.cyborder     = GetSystemMetrics( 6 )
     this.cymenu       = GetSystemMetrics( 15 )
     this.cyfullscreen = GetSystemMetrics( 17 )
     this.cyframe      = GetSystemMetrics( 33 )
     this.mdimaxheight = this.cyfullscreen -;
                         ( 4 * this.cyborder ) -;
                         ( 2 * this.cyframe ) -;
                         this.cycaption -;
                         this.cymenu
     this.sdimaxheight = this.cyfullscreen -;
                         this.cymenu - ;
                         ( 4 * this.cyborder ) -;
                         ( 2 * this.cyframe )
     this.mdimaxwidth  = this.cxfullscreen -;
                         ( 4 * this.cxborder ) -;
                         ( 2 * this.cxframe )
     this.sdimaxwidth  = this.mdimaxwidth

     IF TYPE( "GetDeviceCaps" ) # "FP"
        extern CINT  GetDeviceCaps( CHANDLE, CINT ) GDI32
     ENDIF
     IF TYPE( "ReleaseDC" ) # "FP"
        extern CINT  ReleaseDC( CHANDLE, CHANDLE ) USER32
     ENDIF
     IF TYPE( "GetDC" ) # "FP"
        extern CHANDLE  GetDC( CHANDLE ) USER32
     ENDIF
     dcval = GetDC( 0 )
     ncolors 	= GetDeviceCaps( dcval, 12 )
     // line added Jan. 2005 
     this.DPI 	= GetDeviceCaps( dcval, 88 ) 
     ReleaseDC( 0, dcval )
     DO CASE
        Case ncolors > 24
             this.color = "32 bit True-Color"
        Case ncolors >= 24
             this.color = "24 bit True-Color"
        Case ncolors >= 16
             this.color = "16 bit Hi-Color"
        Case ncolors >= 8
             this.color = "256 Colors"
        Case ncolors >= 4
             this.color = "16 Colors"
        Otherwise
             this.color = "UnKnown"
     ENDCASE

     // Added: December, 2005
     nFonts = GetSystemMetrics( 4 ) // sm_cycaption
     do case
        case nFonts == 26
             this.Fonts = "Normal Fonts (96 dpi)"
        case nFonts == 30
             this.Fonts = "Large Fonts (120 dpi)"
        case nFonts == 36
             this.Fonts = "Very Large Fonts (??? dpi)"
        otherwise
             this.Fonts = "Custom Font Size"
     endcase

     // Restore settings
     _app.speedbar  := lSpeedbar
     _app.statusbar := lStatusbar

   return
endclass
*------------------------------------------------------------