// WinEvents.cc

/*
    Author: Nicholas Martin
    Date..:March 28, 1999
    Notes: Posted in dBASE, Inc. newsgroups, used with permission

    Allows forms refresh and user's interactions during 
    execution of a task that occupies the CPU at full load. 
    The doNow() method achieves the same as VB's 
    doEvent or Delphi's Tapplication.processMessage, 
    briefly, it enables the OS to dequeue all pending 
    commands (messages) to the forms.

    Methods: DoNow()
             Repaint()

    See TestWinEvents.WFM for an example of the
    use of this control.
       
*/

#include "Winuser.h" // in dbase's INCLUDE folder

/* 
   According to Romain Strieff advice, a paintbox here is better 
   for memory saving than a rectangle. Having not experienced 
   any memory trouble, I've left the rectangle, as a text 
   can be visible in design mode when the component is 
   placed on the form. 
*/

class winEvents( f ) of rectangle( f ) custom
   // Unused or hidden properties and methods are protected :
   protect  ;
      className, colorNormal,  default, disabledBitmap, ;
      downBitmap, focusBitmap, group, speedBar, text, ;
      textLeft, toggle, upBitmap, before, borderStyle, ;
      enabled, fontBold, fontItalic, fontName, fontSize,;
      fontStrikeout, fontUnderline, helpFile, helpId, ;
      mousePointer, pageNo, printable, speedTip, ;
      statusMessage, visible, statusMessage, tabStop, ;
      lpMess, onClick, onDesignOpen, onLeftDblClick, ;
      onLeftMouseDown, onLeftMouseUp, onMiddleDblClick, ;
      onMiddleMouseDown, onMiddleMouseUp, onMouseMove, ;
      onOpen, onRightDblClick, onRightMouseDown, ;
      onRightMouseUp, when, setFocus

   // API Prototypes:
   if type( "DispatchMessage" ) # "FP"
      extern LONG DispatchMessage(CPTR) user32 ;
                  from "DispatchMessageA"
   endif
   if type( "PeekMessage" ) # "FP"
      extern BOOL PeekMessage(LPSTRUCTURE, CHWND, ;
                  UINT, UINT, UINT) user32 ;
                  from "PeekMessageA"
   endif
   if type( "TranslateMessage" ) # "FP"
	   extern BOOL TranslateMessage(CPTR) user32
   endif

   with ( this )
      visible     := false // do not display object in 
                           // run time mode
      tabStop     := false
      height      := 1.2727
      left        := 9.7143
      top         := 1.5455
      width       := 13.7143
      colorNormal := 'Yellow/Teal'
      text        := 'winEvents'
   endwith

   // Creates a MSG structures
   this.lpMess = replicate( chr( 0 ) , 20 ) 

   /*
      This method will execute all pending Windows events 
      that are currently stacked in the system. Each one 
      is called 'a message' in Windows formalism. These 
      messages, which can be keyboard entries, mouse 
      moves and clicks, timer ticks,... are 'queued' by
      windows in the form's queue as they arrive. To 
      de-queue these messages, we need to tell explicitely 
      to Windows to do that, as Windows considers that this 
      is part of the allocated application processing time.
   */
   function doNow
     if __version__ < 2.5
        // Checks for pending Windows messages for any window 
        // of the current thread
        do while PeekMessage( this.lpMess, Null, 0, 0, 1 )
           TranslateMessage( this.lpMess )
           DispatchMessage( this.lpMess )
        enddo
     else
        _app.executeMessages()
     endif
  return

  /* 
     Experience has shown that the form object in Visual 
     dBASE sometimes has trouble re-drawing itself if the 
     software is busy, or the resources start getting low. 
     This can make your forms look really bad. Here's a 
     quick and dirty bit of code that you can place into 
     the primary custom form, and call it to repaint the 
     form. This is particularly handy if you have a 
     lookup form that pops up on top of your data form
     -- sometimes the underlying form doesn't get redrawn 
     well -- this will force it to be refreshed properly. 
     The code uses Windows API calls, and was written by 
     Jim Sare. 
 */
   function Repaint
      parameter oForm
      if type( "oForm" ) == "U" or empty( oForm )
         oForm = form
      endif
      if type( "UpdateWindow" ) # "FP"
         extern CLOGICAL UpdateWindow( CHANDLE ) USER32
      endif
   return UpdateWindow( oForm.hWnd )

endclass


